/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLCpp.h
 *
 * Purpose:     C++ root header. Exception support, exception specifications,
 *              language support, utility classes.
 *
 * Created:     3rd March 1993
 * Updated:     3rd April 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Public License)
 *
 *              Copyright 1993-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_SLBASE
# include <SLBase.h>
#endif /* _SYNSOFT_INCL_H_SLBASE */

////////////////////////////////////////

#ifndef _SYNSOFT_INCL_H_MLCPP
#define _SYNSOFT_INCL_H_MLCPP

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLCPP_MAJOR    2
# define _SYNSOFT_VER_H_MLCPP_MINOR    16
# define _SYNSOFT_VER_H_MLCPP_REVISION 3
# define _SYNSOFT_VER_H_MLCPP_EDIT     1045
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */

#ifndef __cplusplus
# error MLCpp.h can only be included into C++ compilation units
#endif // !__cplusplus

////////////////////////////////////////////////////////////////////////////////

__SYNSOFT_GEN_NS_START(SynesisStd)

////////////////////////////////////////////////////////////////////////////////
// C++ feature support discrimination
//
//

// Determines whether the compiler supports templates
#ifdef __SYNSOFT_DBS_TEMPLATE_SUPPORT
# undef __SYNSOFT_DBS_TEMPLATE_SUPPORT
#endif // __SYNSOFT_DBS_TEMPLATE_SUPPORT

// Determines whether the compiler supports exceptions
#ifdef __SYNSOFT_DBS_EXCEPTION_SUPPORT
# undef __SYNSOFT_DBS_EXCEPTION_SUPPORT
#endif // __SYNSOFT_DBS_EXCEPTION_SUPPORT

// Determines whether class may legally have operator XXX() methods, where XXX
// is not a built-in type
#ifdef __SYNSOFT_DBS_OPERATOR_NON_BUILT_IN_TYPE_SUPPORT
# undef __SYNSOFT_DBS_OPERATOR_NON_BUILT_IN_TYPE_SUPPORT
#endif // __SYNSOFT_DBS_OPERATOR_NON_BUILT_IN_TYPE_SUPPORT

// Determines whether overloaded operators to have default arguments
#ifdef __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT
# undef __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT
#endif // __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT

// Determines whether pure members can have implementation
#ifdef __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT
# undef __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT
#endif // __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT

// Determines whether destructors can be pure virtual
#ifdef __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT
# undef __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT
#endif // __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT

// Defined when a placement delete is required to match a placement new.
#ifdef __SYNSOFT_DBS_PLACEMENT_DELETE_SUPPORT
# undef __SYNSOFT_DBS_PLACEMENT_DELETE_SUPPORT
#endif // __SYNSOFT_DBS_PLACEMENT_DELETE_SUPPORT

#ifdef __SYNSOFT_DBS_RTTI_SUPPORT
# undef __SYNSOFT_DBS_RTTI_SUPPORT
#endif // __SYNSOFT_DBS_RTTI_SUPPORT

// New keywords explicit, mutable and typename
#ifdef __SYNSOFT_DBS_EXPLICIT_SUPPORT
# undef __SYNSOFT_DBS_EXPLICIT_SUPPORT
#endif // __SYNSOFT_DBS_EXPLICIT_SUPPORT

#ifdef __SYNSOFT_DBS_MUTABLE_SUPPORT
# undef __SYNSOFT_DBS_MUTABLE_SUPPORT
#endif // __SYNSOFT_DBS_MUTABLE_SUPPORT

#ifdef __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT
# undef __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT
#endif // __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT

#ifdef __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT
# undef __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT
#endif // __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT

#ifdef __SYNSOFT_DBS_STRICT_OPERATOR_NEW_RETURN
# undef __SYNSOFT_DBS_STRICT_OPERATOR_NEW_RETURN
#endif // __SYNSOFT_DBS_STRICT_OPERATOR_NEW_RETURN


// __SYNSOFT_DBS_TEMPLATE_SUPPORT
#ifndef _SYB_NO_TEMPLATES
# if (  (__SYNSOFT_DVS_COMPILER_VER_BORLAND > 0 && defined(__TEMPLATES__)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_XLC >= __SYNSOFT_GEN_COMPILER_VERSION(3, 1)) /* || \
        (__SYNSOFT_DVS_COMPILER_VER_WATCOM >= __SYNSOFT_GEN_COMPILER_VERSION(12, 0)) */)
#  define __SYNSOFT_DBS_TEMPLATE_SUPPORT
# endif // __SYNSOFT_DVS_COMPILER
#endif // _SYB_NO_TEMPLATES


// __SYNSOFT_DBS_EXCEPTION_SUPPORT
#ifndef _SYB_NO_EXCEPTIONS
# if (  (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(4, 2)) || \
        (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX && defined(__EXCEPTIONS)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0) && defined(_CPPUNWIND)) || \
        (__SYNSOFT_DVS_COMPILER_VER_XLC >= __SYNSOFT_GEN_COMPILER_VERSION(3, 1)))
#  define __SYNSOFT_DBS_EXCEPTION_SUPPORT
# endif // (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)  && _CPPUNWIND) || _SYNSOFT_VAL_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(4, 2)
#endif // _SYB_NO_EXCEPTIONS


// __SYNSOFT_DBS_OPERATOR_NON_BUILT_IN_TYPE_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)))
# define __SYNSOFT_DBS_OPERATOR_NON_BUILT_IN_TYPE_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)))
# define __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

#ifdef __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT
# define __SyOpDefArg(x) =   x
#else
# define __SyOpDefArg(x)
#endif // __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT


// __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT
#if (__SYNSOFT_DVS_COMPILER_VER_MSVC > 0)
# define __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

#ifdef __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT
# define __SyMemberPure  =   0
#else
# define __SyMemberPure
#endif // __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT


// __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT
#if (__SYNSOFT_DVS_COMPILER_VER_MSVC > 0)
# define __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

#ifdef __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT
# define __SyDtorPure    =   0
#else
# define __SyDtorPure
#endif // __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT

// __SYNSOFT_DBS_PLACEMENT_DELETE_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)))
# define __SYNSOFT_DBS_PLACEMENT_DELETE_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_DBS_RTTI_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1) && defined(__RTTI)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0) && defined(CPPRTTI)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_RTTI_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_DBS_EXPLICIT_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_EXPLICIT_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

#ifdef _SYB_NO_EXPLICIT
# undef __SYNSOFT_DBS_EXPLICIT_SUPPORT
#endif /* _SYB_NO_EXPLICIT */

#ifndef __SYNSOFT_DBS_EXPLICIT_SUPPORT
# undef explicit
# define explicit
#endif // !__SYNSOFT_DBS_EXPLICIT_SUPPORT


// __SYNSOFT_DBS_MUTABLE_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_MUTABLE_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

#ifdef _SYB_NO_MUTABLE
# undef __SYNSOFT_DBS_MUTABLE_SUPPORT
#endif /* _SYB_NO_MUTABLE */

#ifndef __SYNSOFT_DBS_MUTABLE_SUPPORT
# undef mutable
# define mutable
#endif // !__SYNSOFT_DBS_MUTABLE_SUPPORT

#ifdef __SYNSOFT_DBS_MUTABLE_SUPPORT
# define SyAccessMutable(cls, var_type, inst, var)   (var)
#else
# define SyAccessMutable(cls, var_type, inst, var)   (SyCastConst(_mP(cls), inst)->var)
#endif // __SYNSOFT_DBS_MUTABLE_SUPPORT



// __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

#if defined(_SYB_NO_TYPENAME) || \
    defined(_SYB_NO_TYPENAME_ARG)
# undef __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT
#endif /* _SYB_NO_TYPENAME */

#ifdef __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT
# undef typename_arg_t
# define typename_arg_t     typename
#else
# undef typename_arg_t
# define typename_arg_t     class
#endif // __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT


// __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC > __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

#if defined(_SYB_NO_TYPENAME) || \
    defined(_SYB_NO_TYPENAME_TYPE)
# undef __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT
#endif /* _SYB_NO_TYPENAME */

#ifdef __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT
# undef typename_type_t
# define typename_type_t    typename
#else
# undef typename_type_t
# define typename_type_t
#endif // __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT


// __SYNSOFT_DBS_STRICT_OPERATOR_NEW_RETURN
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)))
# define __SYNSOFT_DBS_STRICT_OPERATOR_NEW_RETURN
#endif // __SYNSOFT_DVS_COMPILER

#ifndef __SYNSOFT_DBS_TEMPLATE_SUPPORT
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    C++: Not using templates")
# endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#else

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# pragma message("    C++: Using templates")

# ifdef __SYNSOFT_DBS_EXCEPTION_SUPPORT
#  pragma message("    C++: Using exceptions")
# else
#  pragma message("    C++: Not using exceptions")
# endif // __SYNSOFT_DBS_EXCEPTION_SUPPORT

# ifdef __SYNSOFT_DBS_OPERATOR_NON_BUILT_IN_TYPE_SUPPORT
#  pragma message("    C++: non-built-in type operators supported")
# endif // __SYNSOFT_DBS_OPERATOR_NON_BUILT_IN_TYPE_SUPPORT

// Determines whether overloaded operators to have default arguments
# ifdef __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT
#  pragma message("    C++: operator default arguments supported")
# endif // __SYNSOFT_OPERATOR_DEFAULT_ARGS_SUPPORT

// Determines whether pure members can have implementation
# ifdef __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT
#  pragma message("    C++: pure method implementation supported")
# endif // __SYNSOFT_OPERATOR_PURE_IMPLEMENTATION_SUPPORT

// Determines whether destructors can be pure virtual
# ifdef __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT
#  pragma message("    C++: pure destructor implementation supported")
# endif // __SYNSOFT_OPERATOR_PURE_DTOR_SUPPORT

# ifdef __SYNSOFT_DBS_PLACEMENT_DELETE_SUPPORT
#  pragma message("    C++: placement-delete supported")
# endif // __SYNSOFT_DBS_PLACEMENT_DELETE_SUPPORT

# ifdef __SYNSOFT_DBS_RTTI_SUPPORT
#  pragma message("    C++: RTTI supported")
# endif // __SYNSOFT_DBS_RTTI_SUPPORT

# ifdef __SYNSOFT_DBS_EXPLICIT_SUPPORT
#  pragma message("    C++: explicit keyword supported")
# endif // __SYNSOFT_DBS_EXPLICIT_SUPPORT

# ifdef __SYNSOFT_DBS_MUTABLE_SUPPORT
#  pragma message("    C++: mutable keyword supported")
# endif // __SYNSOFT_DBS_MUTABLE_SUPPORT

# ifdef __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT
#  pragma message("    C++: typename keyword supported (as template argument)")
# endif // __SYNSOFT_DBS_TYPENAME_ARG_SUPPORT

# ifdef __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT
#  pragma message("    C++: typename keyword supported (as template type specifier)")
# endif // __SYNSOFT_DBS_TYPENAME_TYPE_SUPPORT

# ifdef __SYNSOFT_DBS_STRICT_OPERATOR_NEW_RETURN
#  pragma message("    C++: operator requires void * return (not typedef)")
# endif // __SYNSOFT_DBS_STRICT_OPERATOR_NEW_RETURN

#endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE

// /////////////////////////////////////////////////////////////////////////////
// Templates
//
// Use the symbol __SYNSOFT_DBS_TEMPLATE_SUPPORT for template-conditional code.
// Define _SYB_NO_TEMPLATES to disable the use of templates.
//
// Various template behaviour is also discriminated:
//
// 1    __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT is defined when templates
//      can derive implicit return types from template arguments.
// 2    __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT is defined when operator
//      methods must use the class name to qualify the operator type.
// 3    __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT is defined when member templates
//      are supported by the compiler. __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT
//      is defined when constructor templates are supported by the compiler.
// 4    __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT is defined when
//      templates can have default template argument of class type.
// 5    __SYNSOFT_DBS_TEMPLATE_MEMBER_TEMPLATE_DEFAULT_ARGUMENT_SUPPORT
//      is defined when member templates can have default template arguments.
// 6    __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT is defined
//      when templates can have default template arguments types derived from
//      the other (prior) template arguments.
// 7    __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE is defined when
//    compilers use the obsolete syntax:
//
//       template <class T>
//       X
//       {};
//
//    new:
//
//       template <>
//       X<int>
//       {};
//
//    old:
//
//       X<int>
//       {};
//
// 8    __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS
//      is defined when the compiler cannot parse/instantiate a function
//      template that does not use all template parameters in the function
//      argument list (often requiring that dummy/unused arguments are inserted
//      into the function argument list, with default values)
// 9    __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT is defined when
//      partial specialisation is supported by the compiler
// 10   __SYNSOFT_DBS_TEMPLATE_ARROW_OPERATOR_SUPPORT is defined when the
//      compiler supports the -> operator for templates.


#ifdef __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT
# undef __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT
#endif // __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT

#ifdef __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT
# undef __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT
#endif // __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT

#ifdef __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT
# undef __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT
#endif // __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT

#ifdef __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT
# undef __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT
#endif // __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT

#ifdef __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT
# undef __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT
#endif // __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT

#ifdef __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT
# undef __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT
#endif // __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT

#ifdef __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE
# undef __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE
#endif // __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE

#ifdef __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS
# undef __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS
#endif // __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS

#ifdef __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
# undef __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
#endif // __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT

#ifdef __SYNSOFT_DBS_TEMPLATE_ARROW_OPERATOR_SUPPORT
# undef __SYNSOFT_DBS_TEMPLATE_ARROW_OPERATOR_SUPPORT
#endif // __SYNSOFT_DBS_TEMPLATE_ARROW_OPERATOR_SUPPORT

// __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT
#if __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)
# define __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT
#if (__SYNSOFT_DVS_COMPILER_VER_MSVC > 0 && __SYNSOFT_DVS_COMPILER_VER_MSVC <= __SYNSOFT_GEN_COMPILER_VERSION(5, 0))
# define __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

// __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER

// __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_DBS_TEMPLATE_MEMBER_TEMPLATE_DEFAULT_ARGUMENT_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
# define __SYNSOFT_DBS_TEMPLATE_MEMBER_TEMPLATE_DEFAULT_ARGUMENT_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)))
# define __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT
#endif // __SYNSOFT_DVS_COMPILER


// __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE
#if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND > 0 && __SYNSOFT_DVS_COMPILER_VER_BORLAND < __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC > 0 && __SYNSOFT_DVS_COMPILER_VER_MSVC < __SYNSOFT_GEN_COMPILER_VERSION(5, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_XLC > 0 && __SYNSOFT_DVS_COMPILER_VER_XLC <= __SYNSOFT_GEN_COMPILER_VERSION(3, 1)))
# define __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE
#else
// Can use template <> ...
#endif // 0 < __SYNSOFT_DVS_COMPILER_VER_MSVC <= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)


// __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS
# if (   (__SYNSOFT_DVS_COMPILER_VER_MSVC > 0 && __SYNSOFT_DVS_COMPILER_VER_MSVC <= __SYNSOFT_GEN_COMPILER_VERSION(4, 2)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX > 0 && __SYNSOFT_DVS_COMPILER_VER_DECCXX < __SYNSOFT_GEN_COMPILER_VERSION(6, 2)))
#  define __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS
# endif // 0 < __SYNSOFT_DVS_COMPILER_VER_DECCXX < __SYNSOFT_GEN_COMPILER_VERSION(6, 2)


// __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
# if (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)))
#  define __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
# endif // __SYNSOFT_DVS_COMPILER_VER_xxxx

// __SYNSOFT_DBS_TEMPLATE_ARROW_OPERATOR_SUPPORT
# if (   0)
#  define __SYNSOFT_DBS_TEMPLATE_ARROW_OPERATOR_SUPPORT
# endif //

# ifdef __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): return-type derivation supported")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_TEMPLATE_DERIVED_RETURN_SUPPORT

# ifdef __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): nested operator supported")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT

# ifdef __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): member-templates supported")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_MEMBER_TEMPLATE_SUPPORT

# ifdef __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): constructor templates supported")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_CONSTRUCTOR_TEMPLATE_SUPPORT

# ifdef __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): default template arguments supported")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_TEMPLATE_CLASS_DEFAULT_ARGUMENT_SUPPORT

# ifdef __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): derived default template arguments supported")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_TEMPLATE_DERIVED_DEFAULT_ARGUMENT_SUPPORT

# ifdef __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): class template specialisation obsolete syntax")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_TEMPLATE_SPECIALISATION_SYNTAX_OBSOLETE

# ifdef __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): function template instantiation requires template parameters to be used in function argument list")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_TEMPLATE_FUNCTION_TEMPLATE_REQUIRES_ALL_TEMPLATE_PARAMETERS

# ifdef __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    C++ (templates): partial template specialisation supported")
#  endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif // __SYNSOFT_DBS_TEMPLATE_PARTIAL_SPECIALISATION_SUPPORT

#endif // !__SYNSOFT_DBS_TEMPLATE_SUPPORT

// Template method qualifiers.
#ifdef __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT
# define __SYNSOFT_GEN_TEMPLATEOPERATOR(cls, method)    cls::method
#else
# define __SYNSOFT_GEN_TEMPLATEOPERATOR(cls, method)    method
#endif // __SYNSOFT_DBS_TEMPLATE_NESTED_OPERATOR_SUPPORT


// /////////////////////////////////////////////////////////////////////////////
// Exception handling
//
// Define _SYNSOFT_NO_EXCEPTIONS to disable the use of exceptions.

#ifdef __SYNSOFT_DBS_EXCEPTIONSIGNATURE_SUPPORT
# undef __SYNSOFT_DBS_EXCEPTIONSIGNATURE_SUPPORT
#endif // __SYNSOFT_DBS_EXCEPTIONSIGNATURE_SUPPORT


// __SYNSOFT_DBS_EXCEPTIONSIGNATURE_SUPPORT
#ifdef __SYNSOFT_DBS_EXCEPTION_SUPPORT
# if (  (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        /* (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
         */(__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)))
#  define __SYNSOFT_DBS_EXCEPTIONSIGNATURE_SUPPORT
# endif // _SYNSOFT_VAL_COMPILER
#endif // __SYNSOFT_DBS_EXCEPTION_SUPPORT


#ifdef __SYNSOFT_DBS_EXCEPTIONSIGNATURE_SUPPORT
# define SyThrow_None()                             throw ()
# define SyThrow1(x1)                               throw (x1)
# define SyThrow2(x1, x2)                           throw (x1, x2)
# define SyThrow3(x1, x2, x3)                       throw (x1, x2, x3)
# define SyThrow4(x1, x2, x3, x4)                   throw (x1, x2, x3, x4)
# define SyThrow5(x1, x2, x3, x4, x5)               throw (x1, x2, x3, x4, x5)
# define SyThrow6(x1, x2, x3, x4, x5, x6)           throw (x1, x2, x3, x4, x5, x6)
# define SyThrow7(x1, x2, x3, x4, x5, x6, x7)       throw (x1, x2, x3, x4, x5, x6, x7)
# define SyThrow8(x1, x2, x3, x4, x5, x6, x7, x8)   throw (x1, x2, x3, x4, x5, x6, x7, x8)
#else
# define SyThrow_None()
# define SyThrow1(x1)
# define SyThrow2(x1, x2)
# define SyThrow3(x1, x2, x3)
# define SyThrow4(x1, x2, x3, x4)
# define SyThrow5(x1, x2, x3, x4, x5)
# define SyThrow6(x1, x2, x3, x4, x5, x6)
# define SyThrow7(x1, x2, x3, x4, x5, x6, x7)
# define SyThrow8(x1, x2, x3, x4, x5, x6, x7, x8)
#endif // __SYNSOFT_DBS_EXCEPTIONSIGNATURE_SUPPORT

// These two macros wrap (when exceptions are supported) expressions in untyped
// try-catch blocks, so that all exceptions are caught. SyCatchAll simply catches
// whereas SyCatchAllAndAssert asserts when an exception is caught.

#ifdef __SYNSOFT_DBS_EXCEPTION_SUPPORT
#define SyCatchAll(_expr)               try { _expr; } catch(...) { }
#define SyCatchAllAndAssert(_expr)      try { _expr; } catch(...) { SyAssert(0); }
#else
#define SyCatchAllAndAssert(_expr)      do { _expr; } while(0)
#define SyCatchAll(_expr)               do { _expr; } while(0)
#endif // __SYNSOFT_DBS_EXCEPTION_SUPPORT


// Operator new failure exceptions
//
// The symbol __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT is defined when the compiler
// provides functionality for operator new to throw exceptions. The symbol
// __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS is defined when operator new throws
// exceptions upon memory exhaustion. In this case the type opnew_exception_t is
// the type of the exception that is thrown.
//
// The symbol _SYB_NO_OPNEW_EXCEPTION may be defined to override the compiler
// default and set the new-handler. This is only supported on certain platforms.

#ifdef __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT
# undef __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT
#endif /* __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT */

#ifdef __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS
# undef __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS
#endif /* __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS */

#ifdef __SYNSOFT_DBS_EXCEPTION_SUPPORT

// Include new or new.h, for new-handler, etc.
__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_XLC
#  include <new.h>
# elif  __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX || \
        __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC || \
        __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS
#  include <new>
# else
#  error Cannot determine new_handler/set_new_handler for unknown compiler
# endif // __SYNSOFT_DVS_COMPILER
__SYNSOFT_GEN_NS_START(SynesisStd)

// The new_handler_t type, and the set_new_handler function.
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND
#  define __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT
#  define __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS
typedef new_handler                                                 new_handler_t;
#  define __SySetNewHandler                                         ::set_new_handler
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_XLC
#  define __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT
typedef void(*new_handler_t)(void);
#  define __SySetNewHandler                                         ::set_new_handler
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
#  define __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT
typedef _PNH                                                        new_handler_t;
#  define __SySetNewHandler                                         ::_set_new_handler
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS
typedef __SYNSOFT_GEN_NS_3PTY_QUAL_OR_GLBL(::std, new_handler)      new_handler_t;
#  define __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT
#  define __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS
#  define __SySetNewHandler                                         __SYNSOFT_GEN_NS_QUAL_OR_GLBL(::std, set_new_handler)
# else
#  error Cannot determine new_handler/set_new_handler for unknown compiler
# endif // __SYNSOFT_DVS_COMPILER

# ifdef __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT

// Custom new_handler.
//
// Where customised new_handler behaviour is required, the symbol
// _SYB_USE_CUSTOM_NEWHANDLER must be defined. This then declares the new
// handler function _sycstm_new_handler, which has a signature compatible with
// new_handler_t.
//
// For example, with the Dec C++ 6.1 compiler the signature would be
//
//    void _sycstm_new_handler(void);
//
// The implementation of this function must be provided and linked into the
// module build.

// To use a symbol other than _sycstm_new_handler, the symbol
// _SYB_USE_DECLARE_NEWHANDLER must be defined, which will cause the inclusion
// of the file M3CstNew.h which must defined to contain a definition of the
// _sycstm_new_handler symbol to that required by the .
//
// For example,
//
//    #define _sycstm_new_handler   MyNewHandler
//

#  ifdef _SYB_USE_CUSTOM_NEWHANDLER

#   ifndef __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS
#    define __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS
#   endif /* !__SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS */

#   ifdef _SYB_USE_DECLARE_NEWHANDLER
#    include <M3CstNew.h>
#   endif // _SYB_USE_DECLARE_NEWHANDLER

// Declare the custom new-handler in the global namespace.
__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)
#   if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND
SYFNCDECL(void) _sycstm_new_handler(void);
#   elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
SYFNCDECL(int) _sycstm_new_handler(size_t );
#   elif ( __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX || \
        __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC || \
        __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_XLC)
void _sycstm_new_handler(void);
#   endif // __SYNSOFT_DVS_COMPILER
__SYNSOFT_GEN_NS_START(SynesisStd)

// class NewHandlerInitiator
class NewHandlerInitiator
{
public:
    typedef  NewHandlerInitiator  Class;
public:
    NewHandlerInitiator()
        : m_old_handler(__SySetNewHandler(_sycstm_new_handler))
    {}
    ~NewHandlerInitiator() SyThrow_None()
    {
        __SySetNewHandler(m_old_handler);
    }
private:
    new_handler_t   m_old_handler;

// Not to be implemented
private:
    NewHandlerInitiator(const NewHandlerInitiator &rhs);
    const NewHandlerInitiator &operator =(const NewHandlerInitiator &rhs);
};

#  endif // _SYB_USE_CUSTOM_NEWHANDLER
# endif // __SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT
#endif // __SYNSOFT_DBS_EXCEPTION_SUPPORT

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# ifdef __SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS
#  pragma message("    C++: operator new throws exceptions")
# else
#  pragma message("    C++: operator new returns null")
# endif // __SyInline
#endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE

#if defined(__SYNSOFT_DBS_OPNEW_EXCEPTION_SUPPORT) && !defined(__SYNSOFT_DBS_OPNEW_THROWS_EXCEPTIONS)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    C++:    Warning: non-standard behaviour. Throwing of exceptions from operator new is supported, but not used!")
# endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#endif /* __SYNSOFT_DBS_MULTITHREADED_PLATFORM) && !__SYNSOFT_DBS_MULTITHREADED_SUPPORT */

// /////////////////////////////////////////////////////////////////////////////
// Inlining
//
// The default inlining behaviour is enabled in release versions, and disabled
// in debug versions.
//
// To disable inlining in release versions define the symbol _SYNSOFT_NO_INLINE,
// To enable inlining in debug versions define the symbol _SYNSOFT_INLINE

// Allow override to prevent inlining in release version.
#ifndef __SYNSOFT_DBS_DEBUG
# ifndef _SYNSOFT_NO_INLINE
#  define _SYNSOFT_INLINE
# endif // _SYNSOFT_NO_INLINE
#endif // !__SYNSOFT_DBS_DEBUG

// _SYNSOFT_NO_INLINE takes precedence over _SYNSOFT_INLINE
#ifdef _SYNSOFT_NO_INLINE
# ifdef _SYNSOFT_INLINE
#  undef _SYNSOFT_INLINE
# endif // _SYNSOFT_INLINE
#endif // _SYNSOFT_NO_INLINE

#ifdef __SyInline
# error You may not define __SyInline
# undef __SyInline
#endif // __SyInline

// INLINE_RETURN. If inlines are disabled then return statements may not be
// needed in functions, in which case use INLINE_RETURN(x). When inlines are
// enabled the return will be included.

#ifndef _SYNSOFT_NO_INLINE
# define INLINE_RETURN(x)               return x
#else
# define INLINE_RETURN(x)
#endif // !_SYNSOFT_NO_INLINE

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE

# ifndef MLEVAL
#  define MLEVAL(T,t)  static _mR(T) _##t(void) { static T t; return t; }
# endif // MLEVAL

# ifdef _SYNSOFT_INLINE
#  pragma message("    C++: Inlining")
# else
#  pragma message("    C++: Out-of-line-ing")
# endif // __SyInline
#endif // __SYNSOFT_DBS_COMPILATION_IS_VERBOSE

// Interface method definitions
//
// The definition (and access specification) of interface methods, particularly
// ctors and dtors can aid in catching compilation errors.
//
// By default, release mode compilations will define the symbols
// _SYB_NO_INTERFACE_PROTECTED_CTORS and _SYB_NO_INTERFACE_PROTECTED_DTORS in
// case any compilers do not correctly optimise out these methods, unless the
// _SYB_NO_INTERFACE_DEFAULT is defined.

#ifndef _SYB_NO_INTERFACE_DEFAULT
# ifndef __SYNSOFT_DBS_DEBUG
#  ifndef _SYB_NO_INTERFACE_PROTECTED_CTORS
#   define _SYB_NO_INTERFACE_PROTECTED_CTORS
#  endif /* _SYB_NO_INTERFACE_PROTECTED_CTORS */
#  ifndef _SYB_NO_INTERFACE_PROTECTED_DTORS
#   define _SYB_NO_INTERFACE_PROTECTED_DTORS
#  endif /* _SYB_NO_INTERFACE_PROTECTED_DTORS */
# endif /* !__SYNSOFT_DBS_DEBUG */
#endif /* !_SYB_NO_INTERFACE_DEFAULT */

////////////////////////////////////////////////////////////////////////////////

#ifdef __SYNSOFT_DBS_TEMPLATE_SUPPORT

////////////////////////////////////////////////////////////////////////////////
// Classes

// Typing
//
// Classes TypeBaseNoRef, TypeBaseNoSize & TypeBase provide types to derived
// classes. They do not contain any implementation, and hence represent no
// overhead.

// class TypeBaseNoRef
//
// This class is used simply to provide types to derived classes, but does not
// include reference types, i.e. can be used for void.
template <typename_arg_t T>
struct TypeBaseNoRef
{
public:
    typedef T           Type;
    typedef _mP(Type)   TypePtr;
    typedef _mPC(Type)  TypeConstPtr;
    typedef _mPV(Type)  TypeVolatilePtr;
};

// class TypeBaseNoSize
//
// This class is used simply to provide types to derived classes, but may be
// used for incomplete types.
template <typename_arg_t T>
struct TypeBaseNoSize
{
public:
    typedef T           Type;
    typedef _mP(Type)   TypePtr;
    typedef _mPC(Type)  TypeConstPtr;
    typedef _mPV(Type)  TypeVolatilePtr;
    typedef _mC(Type)   TypeConst;
    typedef _mV(Type)   TypeVolatile;
    typedef _mR(Type)   TypeRef;
    typedef _mRC(Type)  TypeConstRef;
    typedef _mRV(Type)  TypeVolatileRef;
    typedef _mRP(Type)  TypePtrRef;
    typedef _mRPC(Type) TypeConstPtrRef;
    typedef _mRPV(Type) TypeVolatilePtrRef;
};

// class TypeBase
//
// This class is used simply to provide types to derived classes
template <typename_arg_t T>
struct TypeBase
    : public TypeBaseNoSize<T>
{
public:

   enum { TypeSize = (Int)sizeof(T) };
};

// class ClassBase
//
// This less-used class is used simply to provide class types to
// derived classes
template <class C>
struct ClassBase
{
public:
    typedef C               Class;
    typedef _mC(Class)      ClassConst;
    typedef _mV(Class)      ClassVolatile;
    typedef _mP(Class)      ClassPtr;
    typedef _mPC(Class)     ClassConstPtr;
    typedef _mPV(Class)     ClassVolatilePtr;
    typedef _mR(Class)      ClassRef;
    typedef _mRC(Class)     ClassConstRef;
    typedef _mRV(Class)     ClassVolatileRef;
    typedef _mRP(Class)     ClassPtrRef;
    typedef _mRPC(Class)    ClassConstPtrRef;
    typedef _mRPV(Class)    ClassVolatilePtrRef;
};

////////////////////////////////////////////////////////////////////////////////
// Functions
//
// These functions delete, and set to NULL, pointers to objects or pointers to
// arrays.

// fn DeleteSetNull
template <class T>
inline void DeleteSetNull(_mRP(T) rpt)
{
    delete rpt;

    rpt = NULL;
}

// fn DeleteArraySetNull
template <class T>
inline void DeleteArraySetNull(_mRP(T) rpt)
{
    delete [] rpt;

    rpt = NULL;
}

#endif // __SYNSOFT_DBS_TEMPLATE_SUPPORT

////////////////////////////////////////////////////////////////////////////////
// Miscellaneous

#define SySterileClassDef(cl)           class __m__##cl{__m__##cl(){}friend class cl;};class cl:public __m__##cl

////////////////////////////////////////
// Copy ctor & operator equals protection

#define _SY_ACCESS_COPYCTOR2(C,a,T) \
    a: \
        C(const T &);

#define _SY_ACCESS_COPYCTOR(a, T) _SY_ACCESS_COPYCTOR2(T, a, T)

#define _SY_ACCESS_OPEQUALS(a, T) \
    a: \
        T &operator =(const T &);

#define _SY_PRIVATE_COPYCTOR(T)     _SY_ACCESS_COPYCTOR(private, T)
#define _SY_PRIVATE_COPYCTOR2(C, T) _SY_ACCESS_COPYCTOR2(C, private, T)
#define _SY_PROTECT_COPYCTOR(T)     _SY_ACCESS_COPYCTOR(protected, T)
#define _SY_PROTECT_COPYCTOR2(C, T) _SY_ACCESS_COPYCTOR2(C, protected, T)
#define _SY_PRIVATE_OPEQUALS(T)     _SY_ACCESS_OPEQUALS(private, T)
#define _SY_PROTECT_OPEQUALS(T)     _SY_ACCESS_OPEQUALS(protected, T)

#define _SY_PRIVATE_COPYCTOR_AND_OPEQUALS2(C, T) \
    _SY_PRIVATE_COPYCTOR2(C, T) \
    _SY_PRIVATE_OPEQUALS(T)

#define _SY_PRIVATE_COPYCTOR_AND_OPEQUALS(T)   _SY_PRIVATE_COPYCTOR_AND_OPEQUALS2(T, T)

#define _SY_PROTECT_COPYCTOR_AND_OPEQUALS2(C, T) \
    _SY_PROTECT_COPYCTOR2(C, T) \
    _SY_PROTECT_OPEQUALS(T)

//#define _SY_PROTECT_COPYCTOR_AND_OPEQUALS(T)   _SY_PROTECT_COPYCTOR_AND_OPEQUALS2(T, T)

#define _SY_OPNEW_DECL() \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) siAlloc); \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) siAlloc, __SYNSOFT_GEN_NS_QUAL(::SynesisStd, HAtor) hAtor); \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) siAlloc, _mP(void) pv);
#define _SY_OPNEW_STD() \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) siAlloc) \
        { return ::operator new(siAlloc); } \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) siAlloc, __SYNSOFT_GEN_NS_QUAL(::SynesisStd, HAtor) hAtor) \
        { return ::operator new(siAlloc,hAtor); } \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) siAlloc, _mP(void) pv) \
        { return ::operator new(siAlloc,pv); }
#define _SY_OPNEW_STUB() \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) /*siAlloc*/) \
        { return NULL; } \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) /*siAlloc*/, __SYNSOFT_GEN_NS_QUAL(::SynesisStd, HAtor) /*hAtor*/) \
        { return NULL; } \
    _mP(void) operator new(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Size) /*siAlloc*/, _mP(void) /*pv*/) \
        { return NULL; }

#define _SY_OPDELETE_DECL() \
    void operator delete(_mP(void) pvMem) SyThrow_None();
#define _SY_OPDELETE_STD() \
    void operator delete(_mP(void) pvMem) SyThrow_None() \
        { ::operator delete(pvMem); }
#define _SY_OPDELETE_STUB() \
    void operator delete(_mP(void) /*pvMem*/) SyThrow_None() \
        { ; }

#define _SY_ACCESS_OPNEW_DECL(a) \
    a: \
        _SY_OPNEW_DECL()
#define _SY_ACCESS_OPNEW_STD(a) \
    a: \
        _SY_OPNEW_STD()
#define _SY_ACCESS_OPNEW_STUB(a) \
    a: \
        _SY_OPNEW_STUB()

#define _SY_ACCESS_OPDELETE_DECL(a) \
    a: \
        _SY_OPDELETE_DECL()
#define _SY_ACCESS_OPDELETE_STD(a) \
    a: \
        _SY_OPDELETE_STD()
#define _SY_ACCESS_OPDELETE_STUB(a) \
    a: \
        _SY_OPDELETE_STUB()

#define _SY_ACCESS_OPNEW_AND_OPDELETE_DECL(a) \
    _SY_ACCESS_OPNEW_DECL(a) \
    _SY_ACCESS_OPDELETE_DECL(a)

#define _SY_ACCESS_OPNEW_AND_OPDELETE_STD(a) \
    _SY_ACCESS_OPNEW_STD(a) \
    _SY_ACCESS_OPDELETE_STD(a)

#define _SY_ACCESS_OPNEW_AND_OPDELETE_STUB(a) \
    _SY_ACCESS_OPNEW_STUB(a) \
    _SY_ACCESS_OPDELETE_STUB(a)

#define _SY_PRIVATE_OPNEW_AND_OPDELETE() _SY_ACCESS_OPNEW_AND_OPDELETE_DECL(private)
#define _SY_PROTECT_OPNEW_AND_OPDELETE() _SY_ACCESS_OPNEW_AND_OPDELETE_DECL(protected)

////////////////////////////////////////////////////////////////////////////////

__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)

////////////////////////////////////////////////////////////////////////////////

#if defined(__SYNSOFT_DBS_EXCEPTION_SUPPORT) && defined(_SYB_USE_CUSTOM_NEWHANDLER)
static __SYNSOFT_GEN_NS_QUAL(::SynesisStd, NewHandlerInitiator)   s_newhandlerinitiator;
#endif // __SYNSOFT_DBS_EXCEPTION_SUPPORT && _SYB_USE_CUSTOM_NEWHANDLER

////////////////////////////////////////////////////////////////////////////////

#endif // _SYNSOFT_INCL_H_MLCPP

////////////////////////////////////////////////////////////////////////////////
