/* ////////////////////////////////////////////////////////////////////////////
 * File:        MLThread.h
 *
 * Purpose:     Synesis Software platform independent thread handling.
 *
 * Date:        14th May 2000
 * Updated:     18th February 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Public License)
 *
 *              Copyright 2002, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLTHREAD
#define _SYNSOFT_INCL_H_MLTHREAD

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLTHREAD_MAJOR        1
# define _SYNSOFT_VER_H_MLTHREAD_MINOR        4
# define _SYNSOFT_VER_H_MLTHREAD_REVISION     3
# define _SYNSOFT_VER_H_MLTHREAD_EDIT         31
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif // _SYNSOFT_INCL_H_MLBASE
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */

/* ////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

__SYNSOFT_GEN_OPAQUE(HThread)

/* ////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/* This function returns a platform-independent token representing the thread
 * identifier of the calling thread.
 */

#ifdef __SYNSOFT_DBS_MULTITHREADED_SUPPORT

SYFNDECL(ThreadId) GetActiveThreadId(void);

#else

# ifdef __cplusplus
inline ThreadId GetActiveThreadId(void)
{
    return 1;
}
# else
#  define GetActiveThreadId()       SyCastStatic(ThreadId, 1)
# endif /* __cplusplus */

#endif /* __SYNSOFT_DBS_MULTITHREADED_SUPPORT */


/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLTHREAD */

/* ////////////////////////////////////////////////////////////////////////// */
