/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLAssert.h
 *
 * Purpose:     Synesis Software debugging API.
 *
 * Created:     21st October 1994
 * Updated:     19th March 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1994-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLASSERT
#define _SYNSOFT_INCL_H_MLASSERT

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLASSERT_MAJOR        2
# define _SYNSOFT_VER_H_MLASSERT_MINOR        3
# define _SYNSOFT_VER_H_MLASSERT_REVISION     2
# define _SYNSOFT_VER_H_MLASSERT_EDIT         403
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

__SYNSOFT_GEN_NS_START(SynesisDev)
__SYNSOFT_GEN_USING(::SynesisStd::Int)
__SYNSOFT_GEN_USING(::SynesisStd::SInt32)
__SYNSOFT_GEN_USING(::SynesisStd::Boolean)
__SYNSOFT_GEN_USING(::SynesisStd::PCAChar)

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

#ifdef __SYNSOFT_DBS_DEBUG
SYFNDECL(SInt32)    Debug_Init(void);   /* Failed if -ve. */
SYFNDECL(void)      Debug_Uninit(void);
SYFNDECL(void)      Debug_Freeze(Boolean bFreeze);
SYFNDECL(Boolean)   Debug_Assert(PCAChar file, Int line, PCAChar expression);
#else /* ? __SYNSOFT_DBS_DEBUG */
# define Debug_Init()           ((SInt32)0)
# define Debug_Uninit()         ((void)0)
# define Debug_Freeze(b)        ((void)0)
# define Debug_Assert(f, l, x)  ((void)true)
#endif /* __SYNSOFT_DBS_DEBUG */

/* /////////////////////////////////////////////////////////////////////////////
 * Macros
 *
 * SyAssert evaluates and validates an expression in debug-mode. In release-mode
 * the expression is not evaluated and not validated.
 * SyVerify evaluates and validates an expression in debug-mode. In release-mode
 * the expression is evaluated but not validated.
 */

#ifdef SyAssert
# undef SyAssert
#endif /* SyAssert */

#ifdef SyVerify
# undef SyVerify
#endif /* SyVerify */

#ifdef __SYNSOFT_DBS_DEBUG
# define SyAssert(_x)       ((void)((!(_x)) ? ((void)(__SYNSOFT_GEN_NS_QUAL(::SynesisDev, Debug_Assert)(__FILE__, __LINE__, #_x))) : ((void)0)))
# define SyVerify(_x)       SyAssert(_x)
#else
# define SyAssert(_x)       ((void)0)
# define SyVerify(_x)       _x
#endif /* __SYNSOFT_DBS_DEBUG */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisDev)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLASSERT */

/* ////////////////////////////////////////////////////////////////////////// */
