/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLBfWrap.h
 *
 * Purpose:     Synesis Software buffer wrapper class.
 *
 * Created:     20th January 1999
 * Updated:     28th March 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLBFWRAP
#define _SYNSOFT_INCL_H_MLBFWRAP

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLBFWRAP_MAJOR       1
# define _SYNSOFT_VER_H_MLBFWRAP_MINOR       1
# define _SYNSOFT_VER_H_MLBFWRAP_REVISION    2
# define _SYNSOFT_VER_H_MLBFWRAP_EDIT        10
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

#include "MLBuffer.h"
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */
#include "MLMemFns.h"
#include <MLCpp.h>

__SYNSOFT_GEN_NS_START(SynesisStd)

////////////////////////////////////////
// class BufferWrap
//
// Simple (very) wrapper for Buffer_*
// API. No callbacks, no destruction, just
// storage. No exceptions (just asserts).
template <typename_arg_t T>
class BufferWrap
    : TypeBase<T>
{
    typedef BufferWrap<T>   Class;
    typedef _mP(Class)      ClassPtr;
    typedef _mR(Class)      ClassRef;
    typedef _mRC(Class)     ClassConstRef;
public:
    typedef T               *pointer;
    typedef T const         *const_pointer;
    typedef pointer         iterator;
    typedef const_pointer   const_iterator;
    typedef UInt            size_type;

// Construction / destruction
public:
    BufferWrap();
    explicit BufferWrap(size_type cItems);
    explicit BufferWrap(ClassConstRef rhs);
    ~BufferWrap() SyThrow_None();

    ClassRef operator = (ClassConstRef rhs);

    void Empty();

// Size / item count
public:
    void        SetItemCount(size_type cItems);
    void        resize(size_type cItems);
    size_type    GetItemCount() const;
    size_type   size() const;

// Accessors & state
public:
    TypeConstRef    operator [] (size_type index) const;     // Does not expand
    TypeRef         operator [] (size_type index);               // Expands, using SetItemCount
    void            SetItem(size_type index, TypeConstRef item); // Expands, using SetItemCount

    TypePtr         GetBufferPtr();
    TypeConstPtr    GetBufferPtr() const;

// Iteration
public:
    const_iterator  begin() const;
    const_iterator  end() const;
    iterator        begin();
    iterator        end();

// Modification
public:
    void            InsertItem(size_type index, TypeConstRef item);  // Inserts & expands, using SetItemCount
    iterator        insert(iterator it, TypeConstRef item);
    void            insert(iterator it, size_type n, const T& x);
    void            insert(iterator it, const_iterator first, const_iterator last);
    void            RemoveItem(size_type index);                 // Removes, using SetItemCount
    void            push_back(TypeConstRef item);
    iterator        erase(iterator it);
    iterator        erase(iterator first, iterator last);

// Implementation
protected:
    void            _Copy(ClassConstRef rhs);

protected:
    TypePtr m_p;
};

// Typedefs for standard types
typedef BufferWrap<Byte>    ByteBuffer;
typedef BufferWrap<Short>   ShortBuffer;
typedef BufferWrap<Int>     IntBuffer;
typedef BufferWrap<UInt>    UIntBuffer;
typedef BufferWrap<Long>    LongBuffer;
typedef BufferWrap<ULong>   ULongBuffer;
typedef BufferWrap<Double>  DoubleBuffer;

typedef BufferWrap<Bool>    BoolBuffer;
typedef BufferWrap<Handle>  HandleBuffer;
typedef BufferWrap<Ptr>     PtrBuffer;

// class BufferWrapInit
//
// Initialiser class, which will be specialised
/*
template <typename_arg_t T>
void BufferWrapInit(TypeBase<T>::TypePtr pitems, MInt cItems)
{
//  Mem_SetN(pitems,
}
*/

template <typename_arg_t T>
inline void BufferWrap<T>::_Copy(typename_type_t BufferWrap<T>::ClassConstRef rhs)
{
    SetItemCount(rhs.GetItemCount());

    Mem_Copy(m_p, rhs.m_p, GetItemCount() * TypeSize);
}


// Construction / destruction
template <typename_arg_t T>
inline BufferWrap<T>::BufferWrap()
    : m_p(NULL)
{
}

template <typename_arg_t T>
inline BufferWrap<T>::BufferWrap(UInt nItems)
    : m_p(NULL)
{
    SetItemCount(nItems);
}

template <typename_arg_t T>
inline BufferWrap<T>::BufferWrap(typename_type_t BufferWrap<T>::ClassConstRef rhs)
    : m_p(NULL)
{
    _Copy(rhs);
}

template <typename_arg_t T>
inline BufferWrap<T>::~BufferWrap() SyThrow_None()
{
    Empty();
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::ClassRef BufferWrap<T>::operator = (typename_type_t BufferWrap<T>::ClassConstRef rhs)
{
    _Copy(rhs);

    return *this;
}

template <typename_arg_t T>
inline void BufferWrap<T>::Empty()
{
    Buffer_Close(SyCastRaw(RPVoid, m_p));
}

// Size / item count
template <typename_arg_t T>
inline void BufferWrap<T>::SetItemCount(UInt cItems)
{
    UInt    cCurrItems = GetItemCount();

    if(cItems != cCurrItems)    // Efficiency
    {
        // (Re-)allocate using new item count
        m_p = (TypePtr)Buffer_Create(m_p, cItems * TypeSize);

        if(cItems > cCurrItems)
        {
            // Will need to initialise the additional allocation
/*          BufferWrapInit<Type>(m_p + cCurrItems, cItems - cCurrItems);
*/          Mem_Zero(m_p + cCurrItems, (cItems - cCurrItems) * TypeSize);
        }
    }
}

template <typename_arg_t T>
inline void BufferWrap<T>::resize(UInt cItems)
{
    SetItemCount(cItems);
}

template <typename_arg_t T>
inline UInt BufferWrap<T>::GetItemCount() const
{
    return Buffer_GetSize(m_p) / TypeSize;
}

template <typename_arg_t T>
inline size_t BufferWrap<T>::size() const
{
    return GetItemCount();
}

// Accessors & state
template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::TypeConstRef BufferWrap<T>::operator [] (UInt index) const
{
    SyAssert(index<GetItemCount());

    return m_p[index];
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::TypeRef BufferWrap<T>::operator [] (UInt index)
{
    if(index >= GetItemCount())
    {
        SetItemCount(index + 1);
    }

    return m_p[index];
}

template <typename_arg_t T>
inline void BufferWrap<T>::SetItem(UInt index, typename_type_t BufferWrap<T>::TypeConstRef item)
{
    operator [] (index) = item;
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::TypePtr BufferWrap<T>::GetBufferPtr()
{
    return m_p;
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::TypeConstPtr BufferWrap<T>::GetBufferPtr() const
{
    return m_p;
}

// Iteration
template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::const_iterator BufferWrap<T>::begin() const
{
    return &m_p[0];
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::const_iterator BufferWrap<T>::end() const
{
    return &m_p[size()];
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::iterator BufferWrap<T>::begin()
{
    return &m_p[0];
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::iterator BufferWrap<T>::end()
{
    return &m_p[size()];
}

// Modification
template <typename_arg_t T>
inline void BufferWrap<T>::InsertItem(UInt index, typename_type_t BufferWrap<T>::TypeConstRef item)
{
    UInt    cCurrItems  =   GetItemCount();
    Int     cItems      =   1 + ((index < cCurrItems) ? cCurrItems : index);

    // Expand either by one, or to one plus index
    SetItemCount(cItems);

    // Move existing out of way
    Mem_Move(m_p + index + 1, m_p + index, (cItems - index - 1) * TypeSize);

    // and copy in the new item
    m_p[index] = item;
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::iterator BufferWrap<T>::insert(typename_type_t BufferWrap<T>::iterator it, typename_type_t BufferWrap<T>::TypeConstRef item)
{
    SyAssert(it >= begin());
    SyAssert(it <= end());

    UInt    index   =   it - begin();

    InsertItem(index, item);

    return &m_p[index];
}

template <typename_arg_t T>
inline void BufferWrap<T>::push_back(typename_type_t BufferWrap<T>::TypeConstRef item)
{
    InsertItem(size(), item);
}

template <typename_arg_t T>
inline void BufferWrap<T>::RemoveItem(UInt index)
{
    Int cCurrItems = GetItemCount();

    SyAssert(index<(UInt)cCurrItems);

    Mem_Move(m_p + index, m_p + index + 1, (cCurrItems - index) * TypeSize);

    SetItemCount(cCurrItems - 1);
}

template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::iterator BufferWrap<T>::erase(typename_type_t BufferWrap<T>::iterator it)
{
    SyAssert(it >= begin());
    SyAssert(it < end());

    UInt    index   =   it - begin();

    RemoveItem(index);

    return &m_p[index];
}

#if 0
template <typename_arg_t T>
inline typename_type_t BufferWrap<T>::iterator BufferWrap<T>::erase(typename_type_t BufferWrap<T>::iterator first, typename_type_t BufferWrap<T>::iterator last)
{
}
#endif /* 0 */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLBFWRAP */

/* ////////////////////////////////////////////////////////////////////////// */
