/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLBuffer.h
 *
 * Purpose:     Synesis Software Buffer API.
 *
 * Created:     20th September 1996
 * Updated:     18th February 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1996-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLBUFFER
#define _SYNSOFT_INCL_H_MLBUFFER

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLBUFFER_MAJOR        1
# define _SYNSOFT_VER_H_MLBUFFER_MINOR        5
# define _SYNSOFT_VER_H_MLBUFFER_REVISION     1
# define _SYNSOFT_VER_H_MLBUFFER_EDIT         86
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */
#ifdef __cplusplus
# include <MLResPtr.h>
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* /////////////////////////////////////
 * Buffer API
 *
 * Buffers are simply blocks of memory, with management information prepended,
 * which may be treated in the same way as any normal memory block derived from
 * a system memory.
 *
 * Additional functionality provides for the specification of the allocator,
 * initialisation and sizing. Because the API is very low-level, debug variants
 * which specify the file and line are used.
 */

/* void Buffer_Init(void);
 *
 * Initialises the API (May be called n times)
 */

SYFNDECL(SInt32)    Buffer_Init(void); /* <0 if fail */


/* void Buffer_Init(void);
 *
 * Uninitialises the API (after n calls)
 */

SYFNDECL(void)      Buffer_Uninit(void);


/* void *Buffer_Create(void *buffer, Size cbBuffer);
 *
 * (Re-)creates a buffer of the specified size. May be used to allocate a new
 * buffer, or, by passing in a previously created buffer into the first
 * parameter, will reallocate an existing buffer to a new size (may be smaller).
 */

#ifndef __SYNSOFT_DBS_DEBUG
SYFNDECL(PVoid)     Buffer_Create(  PVoid   pvBuffer,   /* May be null. */
                                    Size    cbBuffer);
#else
SYFNDECL(PVoid)     Buffer_Create_( PVoid   pvBuffer,   /* May be null. */
                                    Size    cbBuffer,
                                    PCAChar file,
                                    Int     line);
#define Buffer_Create(_pvBuffer, _cbBuffer) \
        Buffer_Create_(_pvBuffer, _cbBuffer, __FILE__, __LINE__)
#endif /* !__SYNSOFT_DBS_DEBUG */


/* void *Buffer_AtorCreate(void *buffer, Size cbBuffer, HAtor hator);
 *
 * (Re-)creates a buffer of the specified size. May be used to allocate a new
 * buffer, or, by passing in a previously created buffer into the first
 * parameter, will reallocate an existing buffer to a new size (may be smaller).
 * All memory used to create the buffer, and in subsequent reallocation, will be
 * done using given ator (which may be NULL, in which case default ator is used).
 */

#ifndef __SYNSOFT_DBS_DEBUG
SYFNDECL(PVoid) Buffer_AtorCreate(  PVoid   pvBuffer,   /* May be null. */
                                    Size    cbBuffer,
                                    HAtor   hator);
#else
SYFNDECL(PVoid) Buffer_AtorCreate_( PVoid   pvBuffer,   /* May be null. */
                                    Size    cbBuffer,
                                    HAtor   hator,
                                    PCAChar file,
                                    Int     line);
#define Buffer_AtorCreate(_pvBuffer, _cbBuffer, _hator) \
        Buffer_AtorCreate_(_pvBuffer, _cbBuffer, _hator, __FILE__, __LINE__)
#endif /* !__SYNSOFT_DBS_DEBUG */


/* void Buffer_Close(void *&buffer);
 *
 * Closes and destroys the buffer.
 */


#ifndef __SYNSOFT_DBS_DEBUG
SYFNDECL(void)  Buffer_Close(       RPVoid  ppvBuffer);
#else
SYFNDECL(void)  Buffer_Close_(      RPVoid  ppvBuffer,
                                    PCAChar file,
                                    Int     line);
#define Buffer_Close(_ppvBuffer) \
        Buffer_Close_(_ppvBuffer, __FILE__, __LINE__)
#endif /* !__SYNSOFT_DBS_DEBUG */


/* Size Buffer_Set(void *buffer, Byte ySet);
 *
 * Sets every byte in the buffer to the given byte value. Returns the size.
 */

SYFNDECL(void)  Buffer_Set(         PVoid   pvBuffer,
                                    Byte    ySet);


/* Size Buffer_GetSize(void *buffer);
 *
 * Sets every byte in the buffer to the given byte value. Returns the size.
 */

SYFNDECL(Size)  Buffer_GetSize(     PVoid   pvBuffer);


/* /////////////////////////////////////
 * Buffer resource functor
 */

#ifdef __cplusplus

/*
// class Buffer_ftor
 */

struct Buffer_ftor
{
    void operator ()(PVoid h)
    {
        Buffer_Close(h);
    }
};

typedef Resource_ptr<PVoid, PCVoid, IsNull<PVoid>, Buffer_ftor, SetNull<PVoid> >   Buffer_ptr;

#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLBUFFER */

/* ////////////////////////////////////////////////////////////////////////// */
