/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLMemFns.h
 *
 * Purpose:     Synesis Software memory manipulation functions.
 *
 * Created:     21st October 1994
 * Updated:     18th February 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1994-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLMEMFNS
#define _SYNSOFT_INCL_H_MLMEMFNS

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLMEMFNS_MAJOR          2
# define _SYNSOFT_VER_H_MLMEMFNS_MINOR          4
# define _SYNSOFT_VER_H_MLMEMFNS_REVISION       2
# define _SYNSOFT_VER_H_MLMEMFNS_EDIT           59
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/* Memory manipulation functions.
 */

SYFNDECL(Int)   Mem_Compare(LPCVoid pcv1, LPCVoid pcv2, Size cb);
SYFNDECL(void)  Mem_Copy(LPVoid pvDest, LPCVoid pcvSrc, Size cb);
SYFNDECL(void)  Mem_Move(LPVoid pvDest, LPCVoid pcvSrc, Size cb);
SYFNDECL(void)  Mem_Zero(LPVoid pvDest, Size cb);
SYFNDECL(void)  Mem_Set(LPVoid pvDest, UInt8 cbSrc, Size cb);
SYFNDECL(void)  Mem_Set4(LPVoid pvDest, UInt32 cbSrc, Size cb);
SYFNDECL(void)  Mem_SetN(LPVoid pvDest, LPCVoid pcvSrc, Size cb, Size cbSrc);

/* Memory status functions
 */

#define SYIVMF_READ         0x00000001
#define SYIVMF_WRITE        0x00000002
#define SYIVMF_EXECUTE      0x00000004
#define SYIVMF_READWRITE    (SYIVMF_READ | SYIVMF_WRITE)
#define SYIVMF_READEXEC     (SYIVMF_READ | SYIVMF_EXECUTE)

/* Returns 1 if valid, 0 if invalid, -1 if cannot determine. */
SYFNDECL(Int)   IsValidMemory(LPVoid pv, Size cb, UInt32 flags);

/* Returns 1 if true, 0 if not, -1 if cannot determine. */
SYFNDECL(Int)   IsStackMemory(LPVoid pv, Size cb);
SYFNDECL(Int)   IsHeapMemory(LPVoid pv, Size cb);

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLMEMFNS */

/* ////////////////////////////////////////////////////////////////////////// */
