/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLMemory.h
 *
 * Purpose:     Synesis Software core memory API.
 *
 * Created:     21st October 1994
 * Updated:     18th February 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1994-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLMEMORY
#define _SYNSOFT_INCL_H_MLMEMORY

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLMEMORY_MAJOR       1
# define _SYNSOFT_VER_H_MLMEMORY_MINOR       0
# define _SYNSOFT_VER_H_MLMEMORY_REVISION    4
# define _SYNSOFT_VER_H_MLMEMORY_EDIT        8
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* /////////////////////////////////////////////////////////////////////////////
 * Synesis Memory API
 *
 * Memory (de-)allocation functions.
 * ---------------------------------
 *
 * The Synesis Memory API has two behaviours, dependent on whether the memory
 * compilation is a debug or a release version, as defined by, respectively, the
 * prescence and abscence of a definition of the symbol __SYNSOFT_DBS_DEBUG.
 *
 * In debug mode, the API is split into two separate APIs.
 *
 * (i) Alloc family. The first of these, the Alloc family, is the standard
 * API for the (de-)allocation of memory in Synesis Software library code.
 * (De-)allocations made in this API are subject to (de-)allocation tracking by
 * the underlying tracking mechanisms in accordance with the value of the
 * memory-tracking flags, which are retrieved/altered by use of the
 * Mem_GetTrack and Mem_SetTrack functions.
 * (ii) Alloc_NoTrack family. This family is identical to the Alloc family,
 * except that (de-)allocations are not passed to any underlying tracking system
 * and, therefore, detection of memory leaks in the use of this family of
 * functions is unlikely. These functions are intended to be used by a
 * restricted subset of the Synesis Software libraries.
 *
 * In release mode these families are brought into calls into a single set of
 * functions by use of macros.
 *
 * Allocators: Separate allocators may be used, by the Allocator family, whereby
 * an allocator may be specified for use by the (de-)allocation of memory. The
 * standard and no-track allocators are obtained by the Mem_GetStdAtor and
 * Mem_GetNoTrackAtor functions, respectively, although in non-mem-debug mode
 * they refer to the same allocator.
 *
 * Locking.
 * --------
 *
 * To accomodate 16-bit Windows code, the allocation type of the APIs is MemHnd
 * which should be passed through the Mem_Lock & Mem_Unlock to convert the
 * handle to a valid usable address, and back.
 */

/* /////////////////////////////////////
 * Memory (de-)allocation functions
 */

SYFNDECL(SInt32)    Mem_Init(void); /* Initialiser. Must be called first. Failed if <0. */
SYFNDECL(void)      Mem_Uninit(void);

/* (i) Alloc family. */
#ifdef __SYNSOFT_DBS_DEBUG
SYFNDECL(MemHnd)    Mem_Alloc_(PCAChar file, Int line, Size si);
SYFNDECL(MemHnd)    Mem_Realloc_(PCAChar file, Int line, MemHnd hm, Size si);
SYFNDECL(void)      Mem_Free_(PCAChar file, Int line, MemHnd hm);
SYFNDECL(Size)      Mem_GetSize_(PCAChar file, Int line, MemHnd hm);

# define Mem_Alloc(si)              __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_Alloc_)(__FILE__, __LINE__, (si))
# define Mem_Realloc(hm, si)        __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_Realloc_)(__FILE__, __LINE__, (hm), (si))
# define Mem_Free(hm)               __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_Free_)(__FILE__, __LINE__, (hm))
# define Mem_GetSize(hm)            __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_GetSize_)(__FILE__, __LINE__, (hm))

#else /* !__SYNSOFT_DBS_DEBUG */
# define Mem_Alloc                  __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_Alloc_NoTrack)
# define Mem_Realloc                __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_Realloc_NoTrack)
# define Mem_Free                   __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_Free_NoTrack)
# define Mem_GetSize                __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_GetSize_NoTrack)
#endif /* !__SYNSOFT_DBS_DEBUG */

/* (ii) Alloc_NoTrack family. */
SYFNDECL(MemHnd)    Mem_Alloc_NoTrack(Size si);
SYFNDECL(MemHnd)    Mem_Realloc_NoTrack(MemHnd hm, Size si);
SYFNDECL(void)      Mem_Free_NoTrack(MemHnd hm);
SYFNDECL(Size)      Mem_GetSize_NoTrack(MemHnd hm);

/* (iii) Allocator family. */
SYFNDECL(HAtor)     Mem_GetSmlAtor(void);
SYFNDECL(HAtor)     Mem_GetStdAtor(void);
SYFNDECL(HAtor)     Mem_GetLrgAtor(void);
SYFNDECL(void)      Mem_CloseAtor(HAtor hAtor);

#ifdef __SYNSOFT_DBS_DEBUG
SYFNDECL(MemHnd)    Mem_AtorAlloc_(PCAChar file, Int line, HAtor hAtor, Size si);
SYFNDECL(MemHnd)    Mem_AtorRealloc_(PCAChar file, Int line, HAtor hAtor, MemHnd hm, Size si);
SYFNDECL(void)      Mem_AtorFree_(PCAChar file, Int line, MemHnd hm);
SYFNDECL(Size)      Mem_AtorGetSize_(PCAChar file, Int line, MemHnd hm);

# define Mem_AtorAlloc(hAtr, si)        __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorAlloc_)(__FILE__, __LINE__, (hAtr), (si))
# define Mem_AtorRealloc(hAtr, hm, si)  __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorRealloc_)(__FILE__, __LINE__, (hAtr), (hm), (si))
# define Mem_AtorFree(hm)               __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorFree_)(__FILE__, __LINE__, (hm))
# define Mem_AtorGetSize(hm)            __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorGetSize_)(__FILE__, __LINE__, (hm))

#else /* !__SYNSOFT_DBS_DEBUG */
# define Mem_AtorAlloc                  __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorAlloc_NoTrack)
# define Mem_AtorRealloc                __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorRealloc_NoTrack)
# define Mem_AtorFree                   __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorFree_NoTrack)
# define Mem_AtorGetSize                __SYNSOFT_GEN_NS_QUAL(::SynesisStd, Mem_AtorGetSize_NoTrack)
#endif /* !__SYNSOFT_DBS_DEBUG */

SYFNDECL(MemHnd)    Mem_AtorAlloc_NoTrack(HAtor hAtor, Size si);
SYFNDECL(MemHnd)    Mem_AtorRealloc_NoTrack(HAtor hAtor, MemHnd hm, Size si);
SYFNDECL(void)      Mem_AtorFree_NoTrack(MemHnd hm);
SYFNDECL(Size)      Mem_AtorGetSize_NoTrack(MemHnd hm);

/* /////////////////////////////////////
 * Locking
 */
#if (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN16)==__SYNSOFT_VAL_OS_WIN16
# define Mem_Lock(hm)       ((hm) ? LocalLock(hm) : NULL)
# define Mem_Unlock(hm)     ((hm) ? LocalUnlock(hm) : false)
#else /* !__SYNSOFT_VAL_OS_WIN16 */
# define Mem_Lock(hm)       (hm)
# define Mem_Unlock(hm)     (true)
#endif /* __SYNSOFT_VAL_OS_WIN16 */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLMEMORY */

/* ////////////////////////////////////////////////////////////////////////// */
