/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLSmpCnt.h
 *
 * Purpose:     Synesis Software Containment API.
 *
 * Created:     12th February 1999
 * Updated:     18th February 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLSMPCNT
#define _SYNSOFT_INCL_H_MLSMPCNT

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLSMPCNT_MAJOR          2
# define _SYNSOFT_VER_H_MLSMPCNT_MINOR          2
# define _SYNSOFT_VER_H_MLSMPCNT_REVISION       2
# define _SYNSOFT_VER_H_MLSMPCNT_EDIT           85
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <MLCntBas.h>
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */
#ifdef __cplusplus
#include <MLResPtr.h>
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespaces
 */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

/* Simple container callback function typedefs.
 */
typedef PFnVContCopy    PFnScCopy;
typedef PFnVContErase   PFnScErase;
typedef PFnVContCompare PFnScCompare;
typedef PFnVContProcess PFnScProcess;
typedef PFnVContQuery   PFnScQuery;

/* Define the simple container types.
 */

/* Generic simple container handle. */
__SYNSOFT_GEN_OPAQUE(HSmpCont)

/* Item return types. */
#define ScRes_Err       SyVCont_Err     /* Value returned when item search fails. */
#define ScRes_DefEnd    SyVCont_DefEnd  /* Value returned as end from empty container. */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

SYFNDECL(SInt32)    SimpleContainer_Init(void); /* Failed if <0. */
SYFNDECL(void)      SimpleContainer_Uninit(void);

/* Closes a simple container. */
SYFNDECL(void)      SimpleContainer_Close(HSmpCont hsvct);

/* /////////////////////////////////////
 * Callbacks.
 */

SYFNDECL(PFnScCopy)     SimpleContainer_SetCopyProc(HSmpCont hsvct, PFnScCopy pfnCopy);
SYFNDECL(PFnScCopy)     SimpleContainer_GetCopyProc(HSmpCont hsvct);
SYFNDECL(PFnScErase)    SimpleContainer_SetEraseProc(HSmpCont hsvct, PFnScErase pfnErase);
SYFNDECL(PFnScErase)    SimpleContainer_GetEraseProc(HSmpCont hsvct);
SYFNDECL(PFnScProcess)  SimpleContainer_SetProcessProc(HSmpCont hsvct, PFnScProcess pfnProcess);
SYFNDECL(PFnScProcess)  SimpleContainer_GetProcessProc(HSmpCont hsvct);
SYFNDECL(PFnScQuery)    SimpleContainer_SetQueryProc(HSmpCont hsvct, PFnScQuery pfnQuery);
SYFNDECL(PFnScQuery)    SimpleContainer_GetQueryProc(HSmpCont hsvct);

/* Default copy procedure for simple containers. Equivalent to memcpy. */
SYFNDECL(void) SimpleContainer_DefaultCopy(LPVoid pvDest, LPCVoid pcvSrc, Size siItem);

/* Default compare procedure for simple containers. Equivalent to memcmp. */
SYFNDECL(Int) SimpleContainer_DefaultCompare(LPCVoid pvItem1, LPCVoid pvItem2, Size siItem);

/* /////////////////////////////////////
 * Attributes.
 */
SYFNDECL(Size)      SimpleContainer_GetItemSize(HSmpCont hsvct);
SYFNDECL(Int)       SimpleContainer_GetItemCount(HSmpCont hsvct);

/* /////////////////////////////////////
 * Addition/Insertion/Setting/Removal.
 */

/* SimpleContainer_AddItems.
 *
 * Adds (in an unspecified position - container dependant) the items to the
 * container. Returns the position of the first inserted item, or an error.
 */
SYFNDECL(LPVoid)    SimpleContainer_AddItems(   HSmpCont    hsvct,
                                                LPCVoid     pcvItems,
                                                Int         iItems);
#ifdef __cplusplus
inline LPVoid SimpleContainer_AddItem(HSmpCont hsvct, LPCVoid pcvItem)
{
    return SimpleContainer_AddItems(hsvct, pcvItem, 1);
}
#else
#define SimpleContainer_AddItem(_hsvct, _pcvItem) \
        SimpleContainer_AddItems(_hsvct, _pcvItem, 1)
#endif /* __cplusplus */

/* SimpleContainer_RemoveItems.
 *
 * Removes the items from the container at the specified position pcvAt, which
 * points to the first item in the removal sequence, or is NULL if all item are
 * to be removed. iItems is the number of items to remove, or -1 if all items
 * from pcvAt to the end (inclusive) are to be removed. Returns a pointer to the
 * next item in the container or end, or the error item to indicate an error.
 */
SYFNDECL(LPCVoid) SimpleContainer_RemoveItems(  HSmpCont    hsvct,
                                                LPCVoid     pcvAt,  /* NULL for all */
                                                Int         iItems);/* -1 for pcvAt and all after */
#ifdef __cplusplus
inline LPCVoid SimpleContainer_RemoveItem(HSmpCont hsvct, LPCVoid pcvAt)
{
    return SimpleContainer_RemoveItems(hsvct, pcvAt, 1);
}
inline LPCVoid SimpleContainer_RemoveAllItems(HSmpCont hsvct)
{
    return SimpleContainer_RemoveItems(hsvct, NULL, 0);
}
inline LPCVoid SimpleContainer_RemoveItemsFrom(HSmpCont hsvct, LPCVoid pcvAt)
{
    return SimpleContainer_RemoveItems(hsvct, pcvAt, -1);
}
#else
#define SimpleContainer_RemoveItem(_hsvct, _pcvAt) \
        SimpleContainer_RemoveItems(_hsvct, _pcvAt, 1)
#define SimpleContainer_RemoveAllItems(_hsvct) \
        SimpleContainer_RemoveItems(_hsvct, NULL, 0)
#define SimpleContainer_RemoveItemsFrom(_hsvct, _pcvAt) \
        SimpleContainer_RemoveItems(_hsvct, _pcvAt, -1)
#endif /* __cplusplus */

/* SimpleContainer_CopyItems.
 *
 * Copies the items into the container at the specified position pcvAt, which
 * points to the first item in the sequence. pcvItems points to the source
 * sequence to copy, or is NULL if the items are to be erased. iItems is the
 * number of items in the copy sequence, or -1 if pcvItems is NULL. pcvAt may be
 * NULL, indicating all items should be erased, if pcvItems is NULL. Returns the
 * total number of items successfully copied, or -1 on error.
 */
SYFNDECL(Int) SimpleContainer_CopyItems(HSmpCont    hsvct,
                                        LPCVoid     pcvAt,  /* NULL for all */
                                        LPCVoid     pcvItems,/* NULL for erase - zero item storage. */
                                        Int         iItems);/* -1 for pcvAt and all after */
#ifdef __cplusplus
inline Int SimpleContainer_CopyItem(HSmpCont hsvct, LPCVoid pcvAt, LPCVoid pcvItem)
{
    return SimpleContainer_CopyItems(hsvct, pcvAt, pcvItem, 1);
}
inline Int SimpleContainer_EraseItems(HSmpCont hsvct, LPCVoid pcvAt, Int iItems)
{
    return SimpleContainer_CopyItems(hsvct, pcvAt, NULL, iItems);
}
inline Int SimpleContainer_EraseItem(HSmpCont hsvct, LPCVoid pcvAt)
{
    return SimpleContainer_EraseItems(hsvct, pcvAt, 1);
}
inline Int SimpleContainer_EraseItemsFrom(HSmpCont hsvct, LPCVoid pcvAt)
{
    return SimpleContainer_CopyItems(hsvct, pcvAt, NULL, -1);
}
inline Int SimpleContainer_EraseAllItems(HSmpCont hsvct)
{
    return SimpleContainer_EraseItem(hsvct, NULL);
}
#else
#define SimpleContainer_CopyItem(_hsvct, _pcvAt, _pvItem) \
        SimpleContainer_CopyItems(_hsvct, _pcvAt, _pvItem, 1)
#define SimpleContainer_EraseItems(_hsvct, _pcvAt, iItems) \
        SimpleContainer_CopyItems(_hsvct, _pcvAt, NULL, iItems)
#define SimpleContainer_EraseItem(_hsvct, _pcvAt) \
        SimpleContainer_EraseItems(_hsvct, _pcvAt, 1)
#define SimpleContainer_EraseItemsFrom(_hsvct, _pcvAt) \
        SimpleContainer_CopyItems(_hsvct, _pcvAt, NULL, -1)
#define SimpleContainer_EraseAllItems(_hsvct) \
        SimpleContainer_EraseItem(_hsvct, NULL)
#endif /* __cplusplus */

/* SimpleContainer_Process.
 *
 * Searches a container one item at a time calling the given (or internal)
 * process procedure on every item. Processing is terminated if the procedure
 * returns false. Function returns number of items processed, or returns 0 on
 * error.
 */
SYFNDECL(Int) SimpleContainer_Process(  HSmpCont        hsvct,
                                        LPVoid          pStart,     /* NULL to use start of contained sequence. */
                                        LPVoid          pEnd,       /* NULL to use end of contained sequence. */
                                        PFnScProcess    pfnProcess, /* NULL to use internal */
                                        LPVoid          pvParam);

/* /////////////////////////////////////
 * Access/Querying.
 */

/* SimpleContainer_FindItem.
 *
 * Searches a container one item at a time for the first item that is equal to
 * the given item. pfnCompare is a function to use in the search, or,
 * if NULL, the container will use an internal function (either a function
 * that it has been previously given, or a direct memory comparison).
 * Returns SimpleContainer_EndItem() if no matches are found, or
 * ScRes_Err on error.
 */
SYFNDECL(LPCVoid) SimpleContainer_FindItem( HSmpCont        hsvct,
                                            LPCVoid         pcStart,    /* NULL to use start of contained sequence. */
                                            LPCVoid         pcEnd,      /* NULL to use end of contained sequence. */
                                            LPCVoid         pcItem,
                                            PFnScCompare    pfnCompare);

/* SimpleContainer_Query.
 *
 * Searches a container one item at a time until the given (or internal) query
 * procedure returns non-zero, at which point the function returns a pointer to
 * the item. Returns SimpleContainer_EndItem() if no matches are found, or
 * ScRes_Err on error.
 */
SYFNDECL(LPCVoid) SimpleContainer_Query(HSmpCont    hsvct,
                                        LPCVoid     pcStart,    /* NULL to use start of contained sequence. */
                                        LPCVoid     pcEnd,      /* NULL to use end of contained sequence. */
                                        PFnScQuery  pfnQuery,   /* NULL to use internal */
                                        LPVoid      pvParam);

/* SimpleContainer_GetEndItem
 *
 * Returns the end item in a container, notionally one after the last valid
 * item. The return value NEVER refers to a valid item, and it is used in
 * iteration of the container (via SimpleContainer_GetNextItem()) in order
 * to identify the end point of the iteration process. Although the value may
 * change throughout the life of the container, all implementing containers must
 * ensure that calling any const members of the container causes no change in
 * the value returned by this function, i.e. only changes to the contents of
 * the container may affect the end item. This relieves clients from the
 * expense of eliciting the end-item value upon each iteration.
 */
SYFNDECL(LPCVoid) SimpleContainer_GetEndItem(HSmpCont hsvct);

/* SimpleContainer_IsError
 *
 * Evaluates whether the given item is either the error item for the given
 * container.
 */

#ifdef __cplusplus
inline Bool SimpleContainer_IsError(HSmpCont /* hsvct */, LPCVoid pcvItem)
{
    return pcvItem == ScRes_Err;
}
#else
#define SimpleContainer_IsError(_hsvct, _pItem) \
            (((LPCVoid)(_pItem)) == ScRes_Err)
#endif /* __cplusplus */

/* SimpleContainer_IsEndOrError
 *
 * Evaluates whether the given item is either the end item or the error item for
 * the given container.
 */

#ifdef __cplusplus
inline Bool SimpleContainer_IsEndOrError(HSmpCont hsvct, LPCVoid pcvItem)
{
    return SimpleContainer_IsError(hsvct, pcvItem) || pcvItem == SimpleContainer_GetEndItem(hsvct);
}
#else
#define SimpleContainer_IsEndOrError(_hsvct, _pItem)    \
            (SimpleContainer_IsError((_hsvct), (_pItem)) || ((LPCVoid)(_pItem)) == SimpleContainer_GetEndItem(_hsvct))
#endif /* __cplusplus */

/* SimpleContainer_GetNextItem
 *
 * Returns the next item in a container after the one specified. If the
 * container is empty, or pvItem points to the last item, returns
 * SimpleContainer_EndItem(). If pvItem is NULL, then returns the first
 * item. If pvItem is not a valid item, behaviour is undefined (although
 * containers in debug mode should report some error information).
 */
SYFNDECL(LPCVoid) SimpleContainer_GetNextItem(  HSmpCont    hsvct,
                                                LPCVoid     pvItem);    /* NULL to get first item */


#ifdef __cplusplus

////////////////////////////////////////////////////////////////////////////////
// HSmpCont_ftor
//
// Simple container Resource_ptr-compatible functor.

// class HSmpCont_ftor
struct HSmpCont_ftor
{
    void operator ()(HSmpCont h)
    {
        SimpleContainer_Close(h);
    }
};

typedef Resource_ptr<HSmpCont, HSmpCont, IsNull<HSmpCont>, HSmpCont_ftor, SetNull<HSmpCont> >   HSmpCont_ptr;

#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLSMPCNT */

/* ////////////////////////////////////////////////////////////////////////// */
