/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLSmpLst.h
 *
 * Purpose:     Synesis Software simple vector API.
 *
 * Created:     12th February 1999
 * Updated:     18th February 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLSMPLST
#define _SYNSOFT_INCL_H_MLSMPLST

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLSMPLST_MAJOR       1
# define _SYNSOFT_VER_H_MLSMPLST_MINOR       0
# define _SYNSOFT_VER_H_MLSMPLST_REVISION    2
# define _SYNSOFT_VER_H_MLSMPLST_EDIT        6
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */
#include <MLSmpCnt.h>

__SYNSOFT_GEN_NS_START(SynesisStd)

/* Define the simple vector handle.
 */
typedef HSmpCont    HSmpVct;

/* /////////////////////////////////////////////////////////////////////////////
 * API initialisation.
 */

SYFNDECL(void)      SimpleList_Init(void);
SYFNDECL(void)      SimpleList_Uninit(void);

/* /////////////////////////////////////
 * Creates a simple vector.
 *
 * siItem       -   the size of item unit allocation.
 * pfnCopy      -   the function to copy items. Since a vector
 *                  contains items in contiguous storage, the
 *                  implementation is much faster when pfnCopy
 *                  is NULL, since memory manipulation is done
 *                  on a block, rather than item, basis.
 * pfnErase     -   the function called when items are erased.
 *                  The vector is much more efficient when no
 *                  erasure function is specified, as memory
 *                  is simply overwritten en bloc.
 * pfnProcess   -   the default function called when processing
 *                  the container items.
 * pfnQuery     -   the default function called when querying
 *                  the container items.
 */
SYFNDECL(HSmpVct)   SimpleList_Create(  HAtor           hAtor,      /* May be NULL. */
                                        Size            siItem,
                                        PFnScCopy       pfnCopy,    /* SimpleContainer_DefaultCopy used if NULL */
                                        PFnScErase      pfnErase,
                                        PFnScCompare    pfnCompare,
                                        PFnScProcess    pfnProcess,
                                        PFnScQuery      pfnQuery);


/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLSMPLST */

/* ////////////////////////////////////////////////////////////////////////// */
