/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLSmpVct.h
 *
 * Purpose:     Synesis Software simple vector API.
 *
 * Created:     12th February 1999
 * Updated:     18th February 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLSMPVCT
#define _SYNSOFT_INCL_H_MLSMPVCT

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLSMPVCT_MAJOR          1
# define _SYNSOFT_VER_H_MLSMPVCT_MINOR          3
# define _SYNSOFT_VER_H_MLSMPVCT_REVISION       2
# define _SYNSOFT_VER_H_MLSMPVCT_EDIT           50
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */
#include <MLSmpCnt.h>

__SYNSOFT_GEN_NS_START(SynesisStd)

/* Define the simple vector handle.
 */
typedef HSmpCont    HSmpVct;

/* /////////////////////////////////////////////////////////////////////////////
 * API initialisation.
 */

SYFNDECL(void)      SimpleVector_Init(void);
SYFNDECL(void)      SimpleVector_Uninit(void);

/* SimpleVector_Create
 *
 * Creates a simple vector.
 *
 * siItem       -   the size of item unit allocation.
 * pfnCopy      -   the function to copy items. Since a vector
 *                  contains items in contiguous storage, the
 *                  implementation is much faster when pfnCopy
 *                  is NULL, since memory manipulation is done
 *                  on a block, rather than item, basis.
 * pfnErase     -   the function called when items are erased.
 *                  The vector is much more efficient when no
 *                  erasure function is specified, as memory
 *                  is simply overwritten en bloc.
 * pfnProcess   -   the default function called when processing
 *                  the container items.
 * pfnQuery     -   the default function called when querying
 *                  the container items.
 */
SYFNDECL(HSmpVct)   SimpleVector_Create(HAtor           hAtor,      /* May be NULL. */
                                        Size            siItem,
                                        PFnScCopy       pfnCopy,    /* SimpleContainer_DefaultCopy used if NULL */
                                        PFnScErase      pfnErase,
                                        PFnScCompare    pfnCompare,
                                        PFnScProcess    pfnProcess,
                                        PFnScQuery      pfnQuery);

SYFNDECL(HSmpVct)   SimpleVector_CreateInit(HAtor       hAtor,      /* May be NULL. */
                                        Size            siItem,
                                        UInt32          capacity,   /* Initial (unused) capacity. */
                                        PFnScCopy       pfnCopy,    /* SimpleContainer_DefaultCopy used if NULL */
                                        PFnScErase      pfnErase,
                                        PFnScCompare    pfnCompare,
                                        PFnScProcess    pfnProcess,
                                        PFnScQuery      pfnQuery);


/* SimpleVector_InsertItems.
 *
 * Inserts the items to the container at the specified position pcvAt, which
 * points to the insertion position, or is NULL if the position is unimportant.
 * Returns the total number of items in the container after the operation has
 * completed. Insertion into an empty container is only allowed when pcvAt is
 * NULL.
 */
SYFNDECL(LPVoid)    SimpleVector_InsertItems(   HSmpVct hsvct,
                                                LPCVoid pcvAt, /* NULL for anywhere */
                                                LPCVoid pcvItems,
                                                Int     iItems);
#ifdef __cplusplus
inline LPVoid SimpleVector_InsertItem(HSmpVct hsvct, LPCVoid pcvAt, LPCVoid pcvItem)
{
    return SimpleVector_InsertItems(hsvct, pcvAt, pcvItem, 1);
}
#else
#define SimpleVector_InsertItem(_hsvct, _pcvAt, _pcvItem) \
        SimpleVector_InsertItems(_hsvct, _pcvAt, _pcvItem, 1)
#endif /* __cplusplus */

/* SimpleVector_GetItem
 *
 * Returns a pointer to the nth item in the container. It iItem is < 0, returns
 * notional end, or terminating, item, which for a vector can be envisaged as
 * the pointer to one past the last item. Returns ScRes_Err upon invalid index,
 * or if container cannot be indexed by integer.
 */
SYFNDECL(LPCVoid)   SimpleVector_GetItem(   HSmpVct hsvct,
                                            Int     iItem); /* -1 to get the end item, i.e. pointer to 1 after last item */


/* SimpleVector_GetItems
 *
 * Copies (up to) the number of items requested, starting at pcvAt (NULL to
 * indicate copying from beginning), returning the actual number returned. The
 * retrieved items are copied into the contiguous storage pointed to by pvDest.
 */
SYFNDECL(Int)       SimpleVector_GetItems(  HSmpVct hsvct,
                                            LPCVoid pcvAt,  /* NULL for from start */
                                            LPVoid  pvDest,
                                            Int     iItems);

/* SimpleVector_SetCapacity
 *
 * Increases, if necessary, the capacity so that the vector will have at least
 * the given extent. Also can be used to extent the number of contained elements
 * within the vector.
 *
 * Returns the new capacity, or -1 on error.
 *
 * (i) To retrieve the current capacity.
 *
 * extent   -   0.
 * pcvSet   -   NULL
 * bRemove  -   false
 *
 * (ii) To increase the capacity.
 *
 * extent   -   the new capacity. The vector is unchanged if this is not greater
 *              than the current capacity.
 * pcvSet   -   NULL
 * bRemove  -   false
 *
 * (iii) To increase the number of elements
 *
 * extent   -   the new number of items. The vector is unchanged if this is not
 *              greater than the current number of items.
 * pcvSet   -   pointer to a single valid element content, which will be copied
 *              into all new items
 * bRemove  -   false
 *
 * (iv) To empty the vector.
 *
 * extent   -   0.
 * pcvSet   -   NULL
 * bRemove  -   true
 *
 * (v) To set the current capacity, and potentially truncate the elements.
 *
 * extent   -   the new capacity. If the vector contains more elements than
 *              this value, then it will be truncated. If if contains fewer then
 *              its elements will remain, but the capacity will be increased to
 *              at least this value.
 * pcvSet   -   NULL
 * bRemove  -   true
 *
 * (vi) To set the current number of elements.
 *
 * extent   -   the new number of elements. If the vector contains more elements
 *              than this value, then it will be truncated. If if contains fewer
 *              then additional elements will be added until it contains this
 *              many. The capacity will be increased to at least this value.
 * pcvSet   -   pointer to a single valid element content, which will be copied
 *              into all new items
 * bRemove  -   true
 */
SYFNDECL(Int)       SimpleVector_SetCapacity(   HSmpVct hsvct,
                                                UInt32  extent,
                                                LPCVoid pcvSet,     /* May be NULL. */
                                                Boolean bRemove);

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLSMPVCT */

/* ////////////////////////////////////////////////////////////////////////// */
