/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLTrace.h
 *
 * Purpose:     Synesis Software debug tracing API.
 *
 * Created:     10th October 1994
 * Updated:     4th April 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1994-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLTRACE
#define _SYNSOFT_INCL_H_MLTRACE

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLTRACE_MAJOR       3
# define _SYNSOFT_VER_H_MLTRACE_MINOR       1
# define _SYNSOFT_VER_H_MLTRACE_REVISION    3
# define _SYNSOFT_VER_H_MLTRACE_EDIT        683
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_START(SynesisDev)
__SYNSOFT_GEN_USING(::SynesisStd::DWord)
__SYNSOFT_GEN_USING(::SynesisStd::PCAChar)
__SYNSOFT_GEN_USING(::SynesisStd::PCWChar)

/* ////////////////////////////////////////////////////////////////////////// */

/* Tracing functions.
 */
SYFNCDECL(void)     Debug_TraceA(PCAChar lpsz, ...);
SYFNCDECL(void)     Debug_TraceW(PCWChar lpsz, ...);

SYFNCDECL(void)     Release_TraceA(PCAChar lpsz, ...);
SYFNCDECL(void)     Release_TraceW(PCWChar lpsz, ...);

SYFNCDECL(void)     Runtime_TraceA(PCAChar lpsz, ...);
SYFNCDECL(void)     Runtime_TraceW(PCWChar lpsz, ...);


#ifdef __SYNSOFT_DBS_UNICODE
#define Debug_Trace Debug_TraceW
#else
#define Debug_Trace Debug_TraceA
#endif /* __SYNSOFT_DBS_UNICODE */

#ifdef __SYNSOFT_DBS_DEBUG
SYFNDECL(void)      _SyTraceWin32Error(DWord dwErr);

#define SyTraceA                    Debug_TraceA
#define SyTraceA0(s)                Debug_TraceA(s)
#define SyTraceA1(s,a1)             Debug_TraceA(s,a1)
#define SyTraceA2(s,a1,a2)          Debug_TraceA(s,a1,a2)
#define SyTraceA3(s,a1,a2,a3)       Debug_TraceA(s,a1,a2,a3)
#define SyTraceA4(s,a1,a2,a3,a4)    Debug_TraceA(s,a1,a2,a3,a4)
#define SyTraceW                    Debug_TraceW
#define SyTraceW0(s)                Debug_TraceW(L##s)
#define SyTraceW1(s,a1)             Debug_TraceW(L##s,a1)
#define SyTraceW2(s,a1,a2)          Debug_TraceW(L##s,a1,a2)
#define SyTraceW3(s,a1,a2,a3)       Debug_TraceW(L##s,a1,a2,a3)
#define SyTraceW4(s,a1,a2,a3,a4)    Debug_TraceW(L##s,a1,a2,a3,a4)
#define SyTrace                     Debug_Trace
#define SyTrace0(s)                 Debug_Trace(mT(s))
#define SyTrace1(s,a1)              Debug_Trace(mT(s),a1)
#define SyTrace2(s,a1,a2)           Debug_Trace(mT(s),a1,a2)
#define SyTrace3(s,a1,a2,a3)        Debug_Trace(mT(s),a1,a2,a3)
#define SyTrace4(s,a1,a2,a3,a4)     Debug_Trace(mT(s),a1,a2,a3,a4)
#define SyTraceWin32Error(e)        _SyTraceWin32Error(e)
#ifdef __SYNSOFT_DBS_API_WIN32
#define SyTraceLastError()          SyTraceWin32Error(GetLastError())
#define SyAssertTraceLastError(b)   do { if(!(b)) SyTraceWin32Error(GetLastError()); } while(0)
#define MVerifyTraceLastError       SyAssertTraceLastError
#else
#define SyTraceLastError()          ((void)0)
#define SyAssertTraceLastError(b)   ((void)0)
#define MVerifyTraceLastError(b)    ((void)(b))
#endif /* __SYNSOFT_DBS_API_WIN32 */

#else /* !__SYNSOFT_DBS_DEBUG */

#define SyTrace                     1 ? ((void)0) : Debug_Trace
#define SyTrace0(s)                 ((void)0)
#define SyTrace1(s,a1)              ((void)0)
#define SyTrace2(s,a1,a2)           ((void)0)
#define SyTrace3(s,a1,a2,a3)        ((void)0)
#define SyTrace4(s,a1,a2,a3,a4)     ((void)0)
#define SyTraceA                    1 ? ((void)0) : Debug_TraceA
#define SyTraceA0(s)                ((void)0)
#define SyTraceA1(s,a1)             ((void)0)
#define SyTraceA2(s,a1,a2)          ((void)0)
#define SyTraceA3(s,a1,a2,a3)       ((void)0)
#define SyTraceA4(s,a1,a2,a3,a4)    ((void)0)
#define SyTraceW                    1 ? ((void)0) : Debug_TraceW
#define SyTraceW0(s)                ((void)0)
#define SyTraceW1(s,a1)             ((void)0)
#define SyTraceW2(s,a1,a2)          ((void)0)
#define SyTraceW3(s,a1,a2,a3)       ((void)0)
#define SyTraceW4(s,a1,a2,a3,a4)    ((void)0)
#define SyTraceWin32Error(e)        ((void)0)
#define SyTraceLastError()          ((void)0)
#define SyAssertTraceLastError(b)   ((void)0)
#define MVerifyTraceLastError(b)    ((void)(b))
#endif /* __SYNSOFT_DBS_DEBUG */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisDev)

__SYNSOFT_GEN_USING(::SynesisDev::Debug_TraceA)
__SYNSOFT_GEN_USING(::SynesisDev::Debug_TraceW)
#ifdef __SYNSOFT_DBS_DEBUG
__SYNSOFT_GEN_USING(::SynesisDev::_SyTraceWin32Error)
#endif /* __SYNSOFT_DBS_DEBUG */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLTRACE */

/* ////////////////////////////////////////////////////////////////////////// */
