/* /////////////////////////////////////////////////////////////////////////////
 * File:        MLTssStr.h
 *
 * Purpose:     Synesis Software Intelligent-TSS API.
 *
 * Created:     21st January 1999
 * Updated:     22nd October 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Private Supply License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              Use in source and binary forms, and redistribution in binary
 *              forms, without modification, are permitted provided that the
 *              following conditions are met:
 *
 *              (i) Redistributions of source code is prohibited without
 *              specific written consent from the original supplier.
 *              Redistributions of source code must retain the above copyright
 *              notice and contact information, this list of conditions and the
 *              following disclaimer.
 *
 *              (ii) Redistribution in binary form is subject to any
 *              non-redistribution stipulations provided by Synesis Software Pty Ltd, or passed
 *              on by an intermediate supplier, at the time of distribution.
 *              Specifically, if a document stipulating one or more binary
 *              components are NOT to be redistributed is obtained from the
 *              supplier at the time of distribution then the user must (a) not
 *              redistribute any material named in that document, and (b) must
 *              redistribute that document with any other materials
 *              redistributed.
 *
 *              Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software (Pty) Ltd.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software Pty Ltd nor the names of any
 *              subdivisions, employees or agents of Synesis Software Pty Ltd, nor the names of
 *              any other contributors to this software may be used to endorse
 *              or promote products derived from this software without specific
 *              prior written permission.
 *
 *              (vi) Prominent citations stating the name of the Synesis Software Pty Ltd are
 *              placed into the source-code, executable programs and any
 *              associated literature of any work based on or using the
 *              software. Citations in the executables programs must be in the
 *              About Box (or similar concept) of a program interacting with the
 *              user via a Graphical User Interface (GUI), or in the form of an
 *              information string to be printed to the console along with other
 *              help and version information of a non-GUI or command-line
 *              program.
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is" and any
 *               warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_MLTSSSTR
#define _SYNSOFT_INCL_H_MLTSSSTR

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_MLTSSSTR_MAJOR       1
# define _SYNSOFT_VER_H_MLTSSSTR_MINOR       4
# define _SYNSOFT_VER_H_MLTSSSTR_REVISION    5
# define _SYNSOFT_VER_H_MLTSSSTR_EDIT        33
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef _SYNSOFT_INCL_H_MLBASE
# include <MLBase.h>
#endif /* _SYNSOFT_INCL_H_MLBASE */
#ifdef _SYB_TRACE_INCLUDE_PATH
# pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */
#ifdef __cplusplus
# include <MLResPtr.h>
#endif /* __cplusplus */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

__SYNSOFT_GEN_OPAQUE(HTssKey)   /* Tss handle */
typedef LPVoid  TssValue;       /* Tss value type */
__SyPtrType1(TssValue)

#ifdef __cplusplus
const TssValue TssValue_Null    =   NULL;
#else
# define TssValue_Null          SyCastStatic(TssValue, 0)
#endif /* __cplusplus */

/* Prototype for callback function used for closing thread slot value. If the
 * value is TssValue_Null no callback will be made.
 */
typedef void (__SYNSOFT_MOD_CALLCONV_DEFAULT *PFnTssSlotClose)(TssValue value);

/* Prototype for function to connect to/lock down client resources
 */
typedef void (__SYNSOFT_MOD_CALLCONV_DEFAULT *PFnTssKeyClientConnnect)(void);

/* Prototype for function to disconnect from/un-lock client resources
 */
typedef void (__SYNSOFT_MOD_CALLCONV_DEFAULT *PFnTssKeyClientDisconnnect)(void);

/* /////////////////////////////////////////////////////////////////////////////
 * Tss API
 *
 * Tss_Init         - Initialises the library (may be called n times)
 * Tss_Uninit       - Uninitialises the library (on nth call)
 *
 * Tss_ThreadAttach - Called from within a thread initialisation to
 *                    inform the library of the creation of a new thread,
 *                    This function can be omitted and the Tss will
 *                    still work correctly, but will produce warnings to
 *                    the debug output.
 * Tss_ThreadDetach - Called from within a thread termination to inform
 *                    the library of the termination, and to cause any TSS
 *                    slot values to be cleaned up. This function is
 *                    automatically called when using PTHREADS, and also
 *                    when the library is used from within the Synesis
 *                    core Win32 dynamic library MMCMNBAS.DLL. In other
 *                    circumstances, this must be called manually,
 *                    otherwise slot value data will not be cleaned up
 *                    until the library is uninitialised, and will be
 *                    lost entirely if library uninitialisation is also
 *                    omitted.
 *
 * Tss_Create       - Creates a Tss entry. pfnClose is the function
 *                    used to close/free/discard the datum associated with
 *                    the entry upon thread termination, or when the datum
 *                    is overwritten with a new value via a call to
 *                    Tss_SetSlotValue. pfnConnect is called by the
 *                    API when (if) the entry is successfully created and
 *                    will usually be a function that keeps a refernce or
 *                    lock on the API that contains pfnClose.
 *                    pfnDisconnect will be called when the entry is
 *                    closed, and thus will usually be a function that
 *                    closes a reference on or unlocks the API that
 *                    contains pfnClose.
 * Tss_Close        - Closes a Tss entry, and removes it from the
 *                    store. Will call pfnClose (if non-null) on all the
 *                    values for the entry for each thread, and then
 *                    pfnDisconnect (if non-null) to close/unlock the API
 *                    containing pfnClose.
 *
 * Tss_SetSlotValue - Sets the value (32-bit opaque) for the entry for
 *                    the current thread. If pvalue is null then the
 *                    previous value is closed via pfnClose (if
 *                    pfnClose is non-null and bCloseOnAssign is true).
 *                    If pvalue is non-null then the previous value is
 *                    returned in *pvalue, and no closure is applied.
 * Tss_GetSlotValue - Returns the (32-bit opaque) value associated
 *                    with the entry for the current thread.
 */
SYFNDECL(SInt32)    Tss_Init(void);    /* Failed if <0. */
SYFNDECL(void)      Tss_Uninit(void);

SYFNDECL(void)      Tss_ThreadAttach(void);
SYFNDECL(void)      Tss_ThreadDetach(void);

/* Create a Tss key. Each function may be null. */
SYFNDECL(HTssKey)   Tss_CreateKey(  PFnTssSlotClose             pfnClose
                                ,   PFnTssKeyClientConnnect     pfnConnect
                                ,   PFnTssKeyClientDisconnnect  pfnDisconnect
                                ,   Boolean                     bCloseOnAssign);
/* Closes a Tss key. */
SYFNDECL(void)      Tss_CloseKey(   HTssKey                     hEntry);

/* Sets the datum for the entry for the current thread. */
SYFNDECL(void)      Tss_SetSlotValue(   HTssKey     hEntry
                                    ,   TssValue    value
                                    ,   PTssValue   pprevdata);

/* Gets the datum for the entry for the current thread. */
SYFNDECL(TssValue)  Tss_GetSlotValue(   HTssKey     hEntry);

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 *
 * Three classes are provided for dealing with the Tss API:
 *
 *  TssKey_ftor     -   Resource_ptr compatible functor.
 *  TssKey          -   Wrapper class for Tss API, creating and closing entry
 *                      within object lifetime.
 *  TssSlotScope    -   Scoping class for accessing, manipulating and setting the
 *                      Tss entry current-thread value.
 */

#ifdef __cplusplus

////////////////////////////////////////
// class Tss
//
// Wrapper class for Tss API.
template <typename_arg_t T>
class TssKey
{
public:
    typedef T                                   value_type;
    typedef TssKey<T>                           Class;
    typedef void (__SYNSOFT_MOD_CALLCONV_DEFAULT *PFnSlotClose)(value_type );
    typedef PFnTssKeyClientConnnect             PFnKeyClientConnnect;
    typedef PFnTssKeyClientDisconnnect          PFnKeyClientDisconnnect;

public:
    TssKey( PFnSlotClose            pfnClose
        ,   PFnKeyClientConnnect    pfnConnect
        ,   PFnKeyClientDisconnnect pfnDisconnect
        ,   Boolean                 bCloseOnAssign  =   true);
    ~TssKey() SyThrow_None();

public:
    void        SetSlotValue(value_type value, value_type *pPrevValue = NULL);
    value_type  GetSlotValue() const;

private:
    static HTssKey  Create( PFnSlotClose            pfnClose
                        ,   PFnKeyClientConnnect    pfnConnect
                        ,   PFnKeyClientDisconnnect pfnDisconnect
                        ,   Boolean                 bCloseOnAssign);

private:
    struct Initialiser
    {
    public:
        Initialiser()
        {
            if(!Tss_Init())
            {
//              throw /./
            }
        }
        ~Initialiser()
        {
            Tss_Uninit();
        }
    };

private:
    Initialiser m_initialiser;

#if __SYNSOFT_DVS_COMPILER != __SYNSOFT_VAL_COMPILER_MSVC || \
    __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(7, 0)
private:
    HTssKey m_hKey;

    template <typename_arg_t T>
    friend class TssSlotScope;
#else
public:
    HTssKey m_hKey;
#endif /* __SYNSOFT_DBS_DEBUG */

_SY_PROTECT_COPYCTOR_AND_OPEQUALS2(TssKey, Class)
};

////////////////////////////////////////
// class TssSlotScope
//
// Tss thread value scoping class.
template <typename_arg_t T>
class TssSlotScope
{
public:
    typedef T                   value_type;
    typedef TssSlotScope<T>     Class;

public:
    TssSlotScope(HTssKey key, value_type &value);
    TssSlotScope(TssKey<T> &key, value_type &value);
    ~TssSlotScope() SyThrow_None();

private:
    HTssKey             m_hKey;
    value_type          &m_valueRef;
    value_type const    m_prevValue;

// Not to be implemented
private:
_SY_PROTECT_COPYCTOR_AND_OPEQUALS2(TssSlotScope, Class)
};

/* /////////////////////////////////////////////////////////////////////////////
 * Implementations
 */

// TssKey
template <typename_arg_t T>
inline /* static */ HTssKey TssKey<T>::Create(  PFnSlotClose            pfnClose
                                            ,   PFnKeyClientConnnect    pfnConnect
                                            ,   PFnKeyClientDisconnnect pfnDisconnect
                                            ,   Boolean                 bCloseOnAssign)
{
    return Tss_CreateKey((PFnTssSlotClose)pfnClose, pfnConnect, pfnDisconnect, bCloseOnAssign);
}

template <typename_arg_t T>
inline TssKey<T>::TssKey(   typename_type_t TssKey<T>::PFnSlotClose             pfnClose
                        ,   typename_type_t TssKey<T>::PFnKeyClientConnnect     pfnConnect
                        ,   typename_type_t TssKey<T>::PFnKeyClientDisconnnect  pfnDisconnect
                        ,   Boolean                                             bCloseOnAssign)
    : m_hKey(Create(pfnClose, pfnConnect, pfnDisconnect, bCloseOnAssign))
{}

template <typename_arg_t T>
inline TssKey<T>::~TssKey() SyThrow_None()
{
    SyStaticAssert(sizeof(value_type) == sizeof(TssValue));

    Tss_CloseKey(m_hKey);
}

template <typename_arg_t T>
inline void TssKey<T>::SetSlotValue(value_type value, typename_type_t TssKey<T>::value_type *pPrevValue)
{
    Tss_SetSlotValue(m_hKey, value, SyCastRaw(PTssValue, pPrevValue));
}

template <typename_arg_t T>
inline typename_type_t TssKey<T>::value_type TssKey<T>::GetSlotValue() const
{
    return (value_type)Tss_GetSlotValue(m_hKey);
}

// TssSlotScope
template <typename_arg_t T>
inline TssSlotScope<T>::TssSlotScope(HTssKey hKey, value_type &value)
    : m_hKey(hKey)
    , m_valueRef(value)
    , m_prevValue((value_type)Tss_GetSlotValue(m_hKey))
{
    m_valueRef = m_prevValue;
}

template <typename_arg_t T>
inline TssSlotScope<T>::TssSlotScope(TssKey<T> &key, value_type &value)
    : m_hKey(key.m_hKey)
    , m_valueRef(value)
    , m_prevValue((value_type)Tss_GetSlotValue(m_hKey))
{
    m_valueRef = m_prevValue;
}

template <typename_arg_t T>
inline TssSlotScope<T>::~TssSlotScope() SyThrow_None()
{
    SyStaticAssert(sizeof(value_type) == sizeof(TssValue));

    if(m_valueRef != m_prevValue)
    {
        Tss_SetSlotValue(m_hKey, m_valueRef, NULL);
    }
}

#endif /* __cplusplus */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_DECL(SynesisStd)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_MLTSSSTR */

/* ////////////////////////////////////////////////////////////////////////// */
