/* /////////////////////////////////////////////////////////////////////////////
 * File:        tss_link_test.cpp
 *
 * Purpose:     Implementation file for the tss_link_test project.
 *
 * Created:     22nd October 2005
 * Updated:     22nd October 2005
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright 1999-2005, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              This source code is placed into the public domain 2005
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* Synesis Header Files */
#include <MLBase.h>
#include <MLTssStr.h>

/* Standard C++ Header Files */
#include <exception>
#include <string>
#if 0
#include <algorithm>
#include <iterator>
#include <list>
#include <vector>
#endif /* 0 */

#if !defined(__WATCOMC__) && \
    (   !defined(_MSC_VER) || \
        _MSC_VER >= 1100)

#else /* ? __WATCOMC__ */
namespace std
{
    using ::exception;
}
#endif /* __WATCOMC__ */


/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h>

#if defined(_MSC_VER) && \
	defined(_DEBUG)
# include <crtdbg.h>
#endif /* _MSC_VER) && _DEBUG */


// The following headers are included for a Win32 version. A UNIX version is
// left as an exercise for the reader. ;-)
#include <windows.h>
#include <winstl/error_desc.hpp>

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

typedef std::string		string_t;

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

void __SYNSOFT_MOD_CALLCONV_DEFAULT string_SlotClose(string_t *value)
{
	fprintf(stdout, "[Thread: %d]: string_SlotClose(%s)\n", ::GetCurrentThreadId(), (NULL != value) ? value->c_str() : "<null>");

	delete value;
}

SynesisStd::TssKey<string_t*>	shared_string(string_SlotClose, NULL, NULL);

/* ////////////////////////////////////////////////////////////////////////// */

DWORD WINAPI ThreadProc(void *param)
{
	const DWORD	id	=	::GetCurrentThreadId();
	char		sz[101];

	try
	{
		{ for(size_t i = 0; i < 1000; ++i)
		{
			::sprintf(&sz[0], "Thread: %d; increment: %d", id, i);

			shared_string.SetSlotValue(new string_t(sz));

			::Sleep(10);

			string_t	*curr = shared_string.GetSlotValue();

			SyAssert(NULL != curr);
			SyAssert(*curr == sz);

		}}
	}
	catch(std::exception &x)
	{
		fprintf(stderr, "Exception encountered in thread %d: %s\n", id, x.what());
	}

	return 0;
}

static int main_(int /* argc */, char ** /*argv*/)
{
	// Set something in the main thread
	string_t	*prev;
	string_t	*curr;

	fprintf(stdout, "Main thread: %d\n", ::GetCurrentThreadId());

	shared_string.SetSlotValue(new string_t("A value in the main thread"), &prev);

    curr	=	shared_string.GetSlotValue();

	SyAssert(NULL != curr);
	SyAssert(*curr == "A value in the main thread");

	fprintf(stdout, "Starting worker threads\n");

	HANDLE	threads[10];

	{ for(size_t i = 0; i < NUM_ELEMENTS(threads); ++i)
	{
		DWORD	threadid;

		if(NULL == (threads[i] = ::CreateThread(NULL, 0, ThreadProc, NULL, 0, &threadid)))
		{
			fprintf(stderr, "Failed to create thread: (%d) %s\n", ::GetLastError(), winstl::error_desc().c_str());

			return EXIT_FAILURE;
		}
	}}

	::WaitForMultipleObjects(NUM_ELEMENTS(threads), &threads[0], true, INFINITE);

    return EXIT_SUCCESS;
}

int main(int argc, char *argv[])
{
	int				iRet;

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState	memState;
#endif /* _MSC_VER && _MSC_VER */

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

#if 0
	{ for(size_t i = 0; i < 0xffffffff; ++i){} }
#endif /* 0 */

	if(SynesisStd::Tss_Init() < 0)
	{
		fprintf(stderr, "Could not initialise the TSS Store API\n");

		iRet = EXIT_FAILURE;
	}
	else
	{
		try
		{


			iRet = main_(argc, argv);
		}
		catch(std::exception &x)
		{
			fprintf(stderr, "Unhandled error: %s\n", x.what());

			iRet = EXIT_FAILURE;
		}
		catch(...)
		{
			fprintf(stderr, "Unhandled unknown error\n");

			iRet = EXIT_FAILURE;
		}

		SynesisStd::Tss_Uninit();
	}

#if defined(_MSC_VER) && \
	defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER) && _DEBUG */

	return iRet;
}

/* ////////////////////////////////////////////////////////////////////////// */
