/* ////////////////////////////////////////////////////////////////////////////
 * File:        SLBase.h (originally MLBase.h)
 *
 * Purpose:     Root header file for all Synesis Software development. This
 *              file provides the following:
 *                  1. compiler messaging
 *                  2. compiler discrimination
 *                  3. architecture discrimination
 *                  3.1. byte-order discrimination
 *                  4. operating system discrimination
 *                  4.1. operating system bit-size
 *                  4.2. operating system object alignment determination
 *                  5. function specifications
 *                  5.1. linkage class
 *                  5.2. storage class
 *                  5.3. memory model
 *                  5.4. calling convention
 *                  5.5. function specification generator macros
 *                  6. casting
 *                  7. namespace support discrimination
 *                  8. multithreading support discrimination
 *                  9. debugging support
 *                  10. preprocessor definitions
 *                  11. API discrimination
 *                  12. C-language feature discrimination
 *                  13. type support discrimination
 *                  14. type specification
 *                  15. C & C++-language feature discrimination
 *                  16. C++-language feature discrimination
 *                  17. constant definitions
 *                  18. standard function/macro definition
 *
 * Created:     3rd March 1993
 * Updated:     9th October 2005
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Public License)
 *
 *              Copyright 1993-2005, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au
 *                          http://www.synesis.com.au/software
 *
 *              email:      software@synesis.com.au
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Redistributions in binary form must reproduce the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer in the documentation
 *              and/or other materials provided with the distribution.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iv) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (v) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _SYNSOFT_INCL_H_SLBASE
#define _SYNSOFT_INCL_H_SLBASE

#ifndef _SYNSOFT_DOCUMENTATION_SKIP_SECTION
# define _SYNSOFT_VER_H_SLBASE_MAJOR    4
# define _SYNSOFT_VER_H_SLBASE_MINOR    23
# define _SYNSOFT_VER_H_SLBASE_REVISION 3
# define _SYNSOFT_VER_H_SLBASE_EDIT     2128
#endif /* !_SYNSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 1. Compiler messaging
 */

/* Undefine compiler messaging symbols. All other symbols are undefined in the
 * SLBasUDf.h file.
 *
 * Undefinition of symbols - which adds to compilation time - may not need to be
 * done (partly due to the relative uniqueness of the Synesis symbol name
 * format, ie. beginning with __SYNSOFT_), may be suppressed by defining the
 * symbol _SYB_NO_SLBASE_SYMBOL_UNDEF.
 */

#ifndef _SYB_NO_SLBASE_SYMBOL_UNDEF
# ifdef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
#  undef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
# endif /* __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE */

# ifdef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE_MACRO_EXPANSION
#  undef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE_MACRO_EXPANSION
# endif /* __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE_MACRO_EXPANSION */

# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  undef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* _SYB_NO_SLBASE_SYMBOL_UNDEF */

/* Determine support for "#pragma message" */
#ifndef _SYB_SUPPRESS_PRAGMA_MESSAGE
# if defined(__BORLANDC__) || \
     defined(__DMC__) || \
     defined(__DECCXX) || \
     defined(__INTEL_COMPILER) || \
     defined(_MSC_VER) || \
     defined(__WATCOMC__)
#  define __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
# endif /* __BORLANDC__ || _MSC_VER || __DECCXX || __INTEL_COMPILER || __WATCOMC__ */
#endif /* !_SYB_SUPPRESS_PRAGMA_MESSAGE */

/* Determine support for macro expansion in "#pragma message" calls */
#if (   defined(__BORLANDC__) && \
        __BORLANDC__ >= 0x0550) || \
    defined(__DECCXX) || \
    defined(_MSC_VER)
# define __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE_MACRO_EXPANSION
#endif /* _MSC_VER || __DECCXX */

/* Notify verbose compilation. This is a user-requested option, requiring the
 * definition of the symbol _SYB_COMPILE_VERBOSE.
 *
 * Verbose compilation provides informational compilation step notification, and
 * also affects certain error/warning conditions discriminated during the
 * compilation process
 */
#if defined(_SYB_COMPILE_VERBOSE) && \
    defined(__SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE)
#define __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compilation is verbose. Undefine _SYB_COMPILE_VERBOSE to turn off")
#endif /* _SYB_COMPILE_VERBOSE || __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE */

/* Determine whether tracing of include paths is requested, by the definition of
 * the _SYB_TRACE_INCLUDE_PATH symbol. For compilers that do not support the
 * message# pragma, the _SYB_TRACE_INCLUDE_PATH is undefined.
 */
#ifdef _SYB_TRACE_INCLUDE_PATH
#ifndef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
#undef _SYB_TRACE_INCLUDE_PATH
#endif /* __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE */
#endif /* _SYB_TRACE_INCLUDE_PATH */

/* Various compile-time macros, both general and pertaining to compilation
 * messaging.
 *
 * The macros have the following functions:
 *
 * __synsoft_stringize(x)   -   evaluates to# x
 * __synsoft_quote(x)       -   evaluates to x
 */

#define __synsoft_stringize(x)          #x
#define __synsoft_stringize_symbol(x)   __synsoft_stringize(x) /* x is translated prior to stringizing */
#define __synsoft_quote(x)              x

#define _sscomp_fileline(x)             __FILE__ "(" __synsoft_stringize_symbol( __LINE__ ) "): "
#define _sscomp_fileline_message(x)     _sscomp_fileline(x) ## x

/* /////////////////////////////////////////////////////////////////////////////
 * Trace include path
 */

#ifdef _SYB_TRACE_INCLUDE_PATH
#pragma message(__FILE__)
#endif /* _SYB_TRACE_INCLUDE_PATH */

/* /////////////////////////////////////////////////////////////////////////////
 * Symbol control
 */

#ifndef _SYB_NO_SLBASE_SYMBOL_UNDEF
#include <SLBasUDf.h>
#endif /* _SYB_NO_SLBASE_SYMBOL_UNDEF */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 2. compiler discrimination
 *
 * The currently supported compilers, and their supported versions, are as
 * follows:
 *
 * Compiler             Versions                    Symbols
 * --------             --------                    -------
 *
 * Borland              3.01, 4.02, 5.5(1), 5.6         __BORLANDC__
 *
 * Dec C                5.09                            __DECC
 *
 * Dec C++              6.01, 6.02                      __DECCXX
 *
 * Digital Mars C++     8.26+                           __DMC__
 *
 * GCC                  2.91, 2.95, 2.96, 3.2-4         __GNUC__
 *
 * Intel                6.0, 7.0, 7.1                   __INTEL_COMPILER
 *
 * Metrowerks           2.4, 3.x                        __MWERKS__
 *
 * Microsoft            1.51, 4.0, 4.2, 5.0, 6.0, 7.0   _MSC_VER
 *
 * Watcom C/C++         11.0, 12.0                      __WATCOMC__
 *
 * IBM xlC              3.01                            __xlC__
 */

/* __SYNSOFT_DVS_COMPILER is the version independent symbol that indicates which
 * compiler is detected, and is one of
 *
 * - __SYNSOFT_VAL_COMPILER_BORLAND (0x1000)
 * - __SYNSOFT_VAL_COMPILER_DECC    (0x5000)
 * - __SYNSOFT_VAL_COMPILER_DECCXX  (0x3000)
 * - __SYNSOFT_VAL_COMPILER_DMC     (0x8000)
 * - __SYNSOFT_VAL_COMPILER_GNUC    (0x4000)
 * - __SYNSOFT_VAL_COMPILER_INTEL   (0xa000)
 * - __SYNSOFT_VAL_COMPILER_MWERKS  (0x9000)
 * - __SYNSOFT_VAL_COMPILER_MSVC    (0x2000)
 * - __SYNSOFT_VAL_COMPILER_WATCOM  (0x7000)
 * - __SYNSOFT_VAL_COMPILER_XLC     (0x6000)
 */

#define __SYNSOFT_VAL_COMPILER_BORLAND              (0x1000)
#define __SYNSOFT_VAL_COMPILER_DECC                 (0x5000)
#define __SYNSOFT_VAL_COMPILER_DECCXX               (0x3000)
#define __SYNSOFT_VAL_COMPILER_DMC                  (0x8000)
#define __SYNSOFT_VAL_COMPILER_GNUC                 (0x4000)
#define __SYNSOFT_VAL_COMPILER_MWERKS               (0x9000)
#define __SYNSOFT_VAL_COMPILER_INTEL                (0xa000)
#define __SYNSOFT_VAL_COMPILER_MSVC                 (0x2000)
#define __SYNSOFT_VAL_COMPILER_WATCOM               (0x7000)
#define __SYNSOFT_VAL_COMPILER_XLC                  (0x6000)

/* __SYNSOFT_DVS_COMPILER_VER is the version-dependant value that indicates the
 * version of the compiler. The version is a 32-bit value, whereby the upper
 * 16-bits represent the major version and the lower 16-bits represent the minor
 * version.
 *
 * The version is generated by the __SYNSOFT_GEN_COMPILER_VERSION(h, l) macro.
 */

#define __SYNSOFT_GEN_COMPILER_VERSION(h, l)        (((h) << 16) | (l))

/* __SYNSOFT_DVS_COMPILER_VER_???? symbols are always defined, but the symbol
 * representing each compiler is either the version value (equal to
 * __SYNSOFT_DVS_COMPILER_VER) or is 0 where that compiler is not the current
 * compiler. For example, when compiling with Borland C++ 5.5, the value of
 * __SYNSOFT_DVS_COMPILER_VER_BORLAND is 0x00050005, and those of all other
 * __SYNSOFT_DVS_COMPILER_VER_???? symbols are 0.
 */

/* Determine the compiler, validating that it is recognised. */
#define __SYNSOFT_DVS_COMPILER_VER_BORLAND          0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_DECC             0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_DECCXX           0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_DMC              0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_GNUC             0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_INTEL            0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_MSVC             0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_MWERKS           0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_XLC              0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_WATCOM           0  /* 0 initially. May be overriden below. */
#define __SYNSOFT_DVS_COMPILER_VER_COMO             0  /* 0 initially. May be overriden below. */

/* Borland compilers */
#if defined(__BORLANDC__)
#undef __SYNSOFT_DVS_COMPILER_VER_BORLAND
#define __SYNSOFT_DVS_COMPILER                      __SYNSOFT_VAL_COMPILER_BORLAND
/* Borland compilers prior to 5.5 were for Intel platforms only. */
#if __BORLANDC__ < 0x0550 && \
    !defined(_M_IX86) && \
    !defined(__i386__)
#define __i386__
#endif /* __BORLANDC__ < 0x0550 && !_M_IX86 && !__i386__ */
#if __BORLANDC__ == 0x0410
#define __SYNSOFT_DVS_COMPILER_VER_BORLAND          __SYNSOFT_GEN_COMPILER_VERSION(3, 1)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Borland, Version: 3.01")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __BORLANDC__ == 0x0452
#define __SYNSOFT_DVS_COMPILER_VER_BORLAND          __SYNSOFT_GEN_COMPILER_VERSION(4, 2)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Borland, Version: 4.02")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __BORLANDC__ == 0x0460
#define __SYNSOFT_DVS_COMPILER_VER_BORLAND          __SYNSOFT_GEN_COMPILER_VERSION(4, 52)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Borland, Version: 4.52")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __BORLANDC__ == 0x0550
#define __SYNSOFT_DVS_COMPILER_VER_BORLAND          __SYNSOFT_GEN_COMPILER_VERSION(5, 5)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Borland, Version: 5.5")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __BORLANDC__ == 0x0551
#define __SYNSOFT_DVS_COMPILER_VER_BORLAND          __SYNSOFT_GEN_COMPILER_VERSION(5, 51)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Borland, Version: 5.51")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __BORLANDC__ == 0x0560
# define __SYNSOFT_DVS_COMPILER_VER_BORLAND          __SYNSOFT_GEN_COMPILER_VERSION(5, 6)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Compiler: Borland, Version: 5.6")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __BORLANDC__ == 0x0564
# define __SYNSOFT_DVS_COMPILER_VER_BORLAND          __SYNSOFT_GEN_COMPILER_VERSION(5, 64)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Compiler: Borland, Version: 5.64")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#else
#error Could not recognise the Borland compiler version. (Not 3.01, 4.02, 4.52, 5.5, 5.51, 5.6 or 5.64).
#endif /* __BORLANDC__ == 0x410 */
#define __SYNSOFT_DVS_COMPILER_VER                  __SYNSOFT_DVS_COMPILER_VER_BORLAND
/* END: Borland compilers */

/* DEC C compiler */
#elif defined(__DECC)
#undef __SYNSOFT_DVS_COMPILER_VER_DECC
#define __sscomp_VER_DECC__                         ((__VER_DECC) / 100000)
#define __SYNSOFT_DVS_COMPILER                      __SYNSOFT_VAL_COMPILER_DECC
#if __sscomp_VER_DECC__ == 509
#define __SYNSOFT_DVS_COMPILER_VER_DECC             __SYNSOFT_GEN_COMPILER_VERSION(5, 9)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Dec C, Version: 5.09")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#else
#error Could not recognise the DEC C compiler version. (Not 5.09).
#endif /* __sscomp_VER_DECC__ == 601 | 602 */
#define __SYNSOFT_DVS_COMPILER_VER                  __SYNSOFT_DVS_COMPILER_VER_DECC
/* END: DEC C compilers */

/* DEC C++ compiler */
#elif defined(__DECCXX)
#undef __SYNSOFT_DVS_COMPILER_VER_DECCXX
#define __sscomp_VER_DECCXX__                       ((__DECCXX_VER) / 100000)
#define __SYNSOFT_DVS_COMPILER                      __SYNSOFT_VAL_COMPILER_DECCXX
#if __sscomp_VER_DECCXX__ == 601
#define __SYNSOFT_DVS_COMPILER_VER_DECCXX           __SYNSOFT_GEN_COMPILER_VERSION(6, 1)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Dec C++, Version: 6.01")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __sscomp_VER_DECCXX__ == 602
#define __SYNSOFT_DVS_COMPILER_VER_DECCXX           __SYNSOFT_GEN_COMPILER_VERSION(6, 2)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Dec C++, Version: 6.02")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#else
#error Could not recognise the DEC C++ compiler version. (Not 6.01, or 6.02).
#endif /* __sscomp_VER_DECCXX__ == 601 | 602 */
#define __SYNSOFT_DVS_COMPILER_VER                  __SYNSOFT_DVS_COMPILER_VER_DECCXX
/* END: Dec C++ compilers */

/* Digital Mars C/C++ compiler */
#elif defined(__DMC__)
#undef __SYNSOFT_DVS_COMPILER_VER_DMC
#define __SYNSOFT_DVS_COMPILER                      __SYNSOFT_VAL_COMPILER_DMC
#if __DMC__ == 0x0826
#define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 26)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Digital Mars C/C++, Version: 8.26")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __DMC__ == 0x0827
#define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 27)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Digital Mars C/C++, Version: 8.27")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __DMC__ == 0x0828
#define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 28)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Digital Mars C/C++, Version: 8.28")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __DMC__ == 0x0829
#define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 29)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Digital Mars C/C++, Version: 8.29")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __DMC__ == 0x0830
#define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 30)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Digital Mars C/C++, Version: 8.30")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __DMC__ == 0x0831
#define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 31)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: Digital Mars C/C++, Version: 8.31")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __DMC__ >= 0x0832
# if __DMC__ == 0x0832
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 32)
# elif __DMC__ == 0x0833
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 33)
# elif __DMC__ == 0x0834
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 34)
# elif __DMC__ == 0x0835
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 35)
# elif __DMC__ == 0x0836
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 36)
# elif __DMC__ == 0x0837
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 37)
# elif __DMC__ == 0x0838
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 38)
# elif __DMC__ == 0x0839
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 39)
# elif __DMC__ == 0x0840
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 40)
# elif __DMC__ == 0x0841
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 41)
# elif __DMC__ == 0x0842
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 42)
# elif __DMC__ == 0x0843
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 43)
# elif __DMC__ == 0x0844
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 44)
# elif __DMC__ == 0x0845
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 45)
# elif __DMC__ == 0x0846
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 46)
# elif __DMC__ == 0x0847
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 47)
# elif __DMC__ == 0x0848
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 48)
# elif __DMC__ == 0x0849
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 49)
# elif __DMC__ == 0x0850
#  define __SYNSOFT_DVS_COMPILER_VER_DMC              __SYNSOFT_GEN_COMPILER_VERSION(8, 50)
# else
#  error Need to update this file!
# endif /* __DMC__ */
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Compiler: " __DMC_VERSION_STRING__)
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#else
# error Could not recognise the Digital Mars C++ compiler version. (Not 8.26 - 8.32).
#endif /* __DMC__ */
#define __SYNSOFT_DVS_COMPILER_VER                  __SYNSOFT_DVS_COMPILER_VER_DMC
/* END: Digital Mars C/C++ compilers */

/* Gnu C++ compilers */
#elif defined(__GNUC__)
# undef __SYNSOFT_DVS_COMPILER_VER_GNUC
# define __sscomp_VER_GNUC__                         (((__GNUC__) * 100000) + (__GNUC_MINOR__))
# define __SYNSOFT_DVS_COMPILER                      __SYNSOFT_VAL_COMPILER_GNUC
# if __sscomp_VER_GNUC__ == 200091
#  define __SYNSOFT_DVS_COMPILER_VER_GNUC             __SYNSOFT_GEN_COMPILER_VERSION(2, 91)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Gnu C++, Version: 2.91")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_GNUC__ == 200095
#  define __SYNSOFT_DVS_COMPILER_VER_GNUC             __SYNSOFT_GEN_COMPILER_VERSION(2, 95)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Gnu C++, Version: 2.95")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_GNUC__ == 200096
#  define __SYNSOFT_DVS_COMPILER_VER_GNUC             __SYNSOFT_GEN_COMPILER_VERSION(2, 96)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Gnu C++, Version: 2.96")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_GNUC__ == 300002
#  define __SYNSOFT_DVS_COMPILER_VER_GNUC             __SYNSOFT_GEN_COMPILER_VERSION(3, 2)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Gnu C++, Version: 3.2")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_GNUC__ == 300003
#  define __SYNSOFT_DVS_COMPILER_VER_GNUC             __SYNSOFT_GEN_COMPILER_VERSION(3, 3)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Gnu C++, Version: 3.3")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_GNUC__ == 300004
#  define __SYNSOFT_DVS_COMPILER_VER_GNUC             __SYNSOFT_GEN_COMPILER_VERSION(3, 4)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Gnu C++, Version: 3.4")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# else
#  error Could not recognise the GNU compiler version. (Not one of 2.91, 2.95 or 2.96).
# endif /* __sscomp_VER_GNUC__ == 291 | 295 */
#define __SYNSOFT_DVS_COMPILER_VER                  __SYNSOFT_DVS_COMPILER_VER_GNUC
/* END: Gnu C++ compilers */

/* Intel C/C++ compiler */
#elif defined(__INTEL_COMPILER)
# undef __SYNSOFT_DVS_COMPILER_VER_INTEL
# define __SYNSOFT_DVS_COMPILER                     __SYNSOFT_VAL_COMPILER_INTEL
# if __INTEL_COMPILER == 600
#  define __SYNSOFT_DVS_COMPILER_VER_INTEL          __SYNSOFT_GEN_COMPILER_VERSION(6, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Intel C++ 6.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __INTEL_COMPILER == 700
#  define __SYNSOFT_DVS_COMPILER_VER_INTEL          __SYNSOFT_GEN_COMPILER_VERSION(7, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Intel C++ 7.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __INTEL_COMPILER == 800
#  define __SYNSOFT_DVS_COMPILER_VER_INTEL          __SYNSOFT_GEN_COMPILER_VERSION(8, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Intel C++ 8.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# else
#  error Could not recognise the Intel C/C++ compiler version. (Not 6.0, 7.0 or 7.1)
#  endif /* __INTEL__ */
/* END: Intel C/C++ compiler */

/* Metworks C/C++ compiler */
#elif defined(__MWERKS__)
# undef __SYNSOFT_DVS_COMPILER_VER_MWERKS
# define __SYNSOFT_DVS_COMPILER                     __SYNSOFT_VAL_COMPILER_MWERKS
# if ((__MWERKS__ & 0xFF00) == 0x2400)
#  define __SYNSOFT_DVS_COMPILER_VER_MWERKS         __SYNSOFT_GEN_COMPILER_VERSION(2, 4)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Metrowerks C++ 2.4")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif ((__MWERKS__ & 0xFF00) == 0x3000)
#  define __SYNSOFT_DVS_COMPILER_VER_MWERKS         __SYNSOFT_GEN_COMPILER_VERSION(3, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Metrowerks C++ 3.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# else
#  error Could not recognise the Metrowerks C/C++ compiler version. (Not 2.4 (7) or 3.0 (8))
#  endif /* __MWERKS__ && 0xJNxx */
/* END: Metworks C/C++ compiler */

/* Watcom C/C++ compiler */
#elif defined(__WATCOMC__)
# undef __SYNSOFT_DVS_COMPILER_VER_WATCOM
# define __sscomp_VER_WATCOMC__                      (__WATCOMC__)
# define __SYNSOFT_DVS_COMPILER                      __SYNSOFT_VAL_COMPILER_WATCOM
# if __sscomp_VER_WATCOMC__ == 1100
#  define __SYNSOFT_DVS_COMPILER_VER_WATCOM           __SYNSOFT_GEN_COMPILER_VERSION(11, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Watcom C/C++, Version: 11.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_WATCOMC__ == 1200
#  define __SYNSOFT_DVS_COMPILER_VER_WATCOM           __SYNSOFT_GEN_COMPILER_VERSION(12, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Open Watcom C/C++ 1.0 (Watcom 12.0)")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_WATCOMC__ == 1210
#  define __SYNSOFT_DVS_COMPILER_VER_WATCOM           __SYNSOFT_GEN_COMPILER_VERSION(12, 1)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Open Watcom C/C++ 1.1 (Watcom 12.1)")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_WATCOMC__ == 1220
#  define __SYNSOFT_DVS_COMPILER_VER_WATCOM           __SYNSOFT_GEN_COMPILER_VERSION(12, 2)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Open Watcom C/C++ 1.2 (Watcom 12.2)")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __sscomp_VER_WATCOMC__ == 1230
#  define __SYNSOFT_DVS_COMPILER_VER_WATCOM           __SYNSOFT_GEN_COMPILER_VERSION(12, 3)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Open Watcom C/C++ 1.3 (Watcom 12.3)")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# else
#  error Could not recognise the Watcom C/C++ compiler version. (Not one of 11.0, 12.0, 12.1, 12.2, 12.3).
# endif /* __sscomp_VER_WATCOMC__ == 291 | 295 */
# define __SYNSOFT_DVS_COMPILER_VER                  __SYNSOFT_DVS_COMPILER_VER_WATCOM
/* END: Watcom C/C++ compiler */

/* IBM xlC compiler */
#elif defined(__xlC__)
#undef __SYNSOFT_DVS_COMPILER_VER_XLC
#define __sscomp_VER_XLC__                          (__xlC__)
#define __SYNSOFT_DVS_COMPILER                      __SYNSOFT_VAL_COMPILER_XLC
#if __sscomp_VER_XLC__ == 0x0301
#define __SYNSOFT_DVS_COMPILER_VER_XLC              __SYNSOFT_GEN_COMPILER_VERSION(3, 1)
#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#pragma message("    Compiler: IBM xlC, Version: 3.01")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#else
#error Could not recognise the xlC compiler version. (Not 3.01).
#endif /* __sscomp_VER_XLC__ == 291 | 295 */
#define __SYNSOFT_DVS_COMPILER_VER                  __SYNSOFT_DVS_COMPILER_VER_XLC
/* END: IBM xlC compiler */

/* Microsoft compilers. This is placed last, since other compilers are
 * (commercially) forced to emulate Visual C++ support and specify _MSC_VER
 */
#elif defined(_MSC_VER)
# undef __SYNSOFT_DVS_COMPILER_VER_MSVC
# define __SYNSOFT_DVS_COMPILER                     __SYNSOFT_VAL_COMPILER_MSVC
# if _MSC_VER == 800
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(1, 51)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 1.51")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 900
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(2, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 2.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 1000
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(4, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 4.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 1020
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(4, 2)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 4.2")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 1100
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(5, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 5.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 1200
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(6, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 6.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 1300
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(7, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 7.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 1310
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(7, 1)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 7.1")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif _MSC_VER == 1400
#  define __SYNSOFT_DVS_COMPILER_VER_MSVC           __SYNSOFT_GEN_COMPILER_VERSION(8, 0)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Microsoft C++, Version: 8.0")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# else
#  error Could not recognise the Microsoft compiler version. (Not 1.51, 4.0, 4.2, 5.0, 6.0, 7.0, 7.1 or 8.0).
# endif /* _MSC_VER == 800 */
# define __SYNSOFT_DVS_COMPILER_VER                 __SYNSOFT_DVS_COMPILER_VER_MSVC
/* END: Microsoft compilers */

/* No recognised compiler */
#elif !defined(__COMO__)
# undef __SYNSOFT_DVS_COMPILER
# define __SYNSOFT_DVS_COMPILER
# undef __SYNSOFT_DVS_COMPILER_VER
# define __SYNSOFT_DVS_COMPILER_VER
# error Compiler not recognised
#endif /* Compiler */

/* Comeau compiler */
#if defined(__COMO__)
# undef __SYNSOFT_DVS_COMPILER_VER_COMO
# define __sscomp_VER_COMO__                        (__COMO__)
# undef __SYNSOFT_DVS_COMPILER
# define __SYNSOFT_DVS_COMPILER                     __SYNSOFT_VAL_COMPILER_COMO
# if __COMO_VERSION__ == 4300
#  define __SYNSOFT_DVS_COMPILER_VER_COMO           __SYNSOFT_GEN_COMPILER_VERSION(3, 1)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Comeau, Version: 4.3.0.1")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# elif __COMO_VERSION__ == 4303
#  define __SYNSOFT_DVS_COMPILER_VER_COMO           __SYNSOFT_GEN_COMPILER_VERSION(3, 1)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Compiler: Comeau, Version: 4.3.3")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# else
#  error Could not recognise the Comeau compiler version. (Not 4.3.0.1 or 4.3.3).
# endif /* __COMO_VERSION__ == 4300 */
# undef __SYNSOFT_DVS_COMPILER_VER
# define __SYNSOFT_DVS_COMPILER_VER                 __SYNSOFT_DVS_COMPILER_VER_COMO
#endif /* __COMO__ */
/* END: Comeau compiler */


/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 3. architecture discrimination
 *
 * The currently supported architectures are as follows:
 *
 * Architecture         Symbols
 * ------------         -------
 *
 * Alpha                __alpha, __alpha__, _M_ALPHA
 *
 * HP                   __hppa, __hppac__
 *
 * Power PC             _POWER, _M_PPC
 *
 * Sparc                __sparc, __sparc__
 *
 * x86                  __i386, __i386__, _M_IX86, M_I86, _M_I86,
 */

/* __SYNSOFT_DVS_ARCHITECTURE is the symbol that indicates which architecture is
 * detected, and is one of:
 *
 * - __SYNSOFT_VAL_ARCHITECTURE_ALPHA   (0x2000)
 * - __SYNSOFT_VAL_ARCHITECTURE_HP      (0x4000)
 * - __SYNSOFT_VAL_ARCHITECTURE_POWERPC (0x5000)
 * - __SYNSOFT_VAL_ARCHITECTURE_SPARC   (0x3000)
 * - __SYNSOFT_VAL_ARCHITECTURE_X86     (0x1000)
 */

#define __SYNSOFT_VAL_ARCHITECTURE_ALPHA            (0x2000)
#define __SYNSOFT_VAL_ARCHITECTURE_HP               (0x4000)
#define __SYNSOFT_VAL_ARCHITECTURE_POWERPC          (0x5000)
#define __SYNSOFT_VAL_ARCHITECTURE_SPARC            (0x3000)
#define __SYNSOFT_VAL_ARCHITECTURE_X86              (0x1000)

/* Determine the architecture, validating that it is recognised */

/* Alpha architecture */
#if defined(__alpha) || \
    defined(__alpha__) || \
    defined(_M_ALPHA)
# define __SYNSOFT_DVS_ARCHITECTURE                 __SYNSOFT_VAL_ARCHITECTURE_ALPHA
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Architecture: Alpha")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* HP architecture */
#elif defined(__hppa) || \
      defined(__hppa__)
# define __SYNSOFT_DVS_ARCHITECTURE                 __SYNSOFT_VAL_ARCHITECTURE_HP
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Architecture: HP")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* PowerPC architecture */
#elif defined(__POWERPC__) || \
      defined(_POWER) || \
      defined(_M_PPC)
# define __SYNSOFT_DVS_ARCHITECTURE                 __SYNSOFT_VAL_ARCHITECTURE_POWERPC
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Architecture: Power PC")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* Sparc architecture */
#elif defined(__sparc) || \
      defined(__sparc__)
# define __SYNSOFT_DVS_ARCHITECTURE                 __SYNSOFT_VAL_ARCHITECTURE_SPARC
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Architecture: Sparc")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* Intel X86 architecture */
#elif defined(__i386) || \
      defined(__i386__) || \
      defined(_M_IX86) || \
      defined(M_I86) || \
      defined(_M_I86)
# define __SYNSOFT_DVS_ARCHITECTURE                 __SYNSOFT_VAL_ARCHITECTURE_X86
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Architecture: Intel x86")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

# ifdef _M_IX86
#  if _M_IX86 >= 300
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_386_OR_LATER
#  endif /* _M_IX86 >= 300 */
#  if _M_IX86 >= 400
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_486_OR_LATER
#  endif /* _M_IX86 >= 400 */
#  if _M_IX86 >= 500
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_586_OR_LATER
#  endif /* _M_IX86 >= 500 */
#  if _M_IX86 >= 600
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_686_OR_LATER
#  endif /* _M_IX86 >= 600 */
#  if _M_IX86 >= 300
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_386
#  elif _M_IX86 >= 400
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_486
#  elif _M_IX86 >= 500
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_586
#  elif _M_IX86 >= 600
#   define __SYNSOFT_DBS_ARCH_IS_INTEL_686
#  endif /* _M_IX86 */
# endif /* _M_IX86 */

/* No recognised architecture */
#else
# error Architecture not recognised
#endif /* __SYNSOFT_DVS_ARCHITECTURE */

/* /////////////////////////////////////////////////////////////////////////////
 * 3.1. byte-order discrimination
 *
 * The discriminated byte-ordering schemes are as follows:
 *
 * (i) big-endian       -   The bytes at lower addresses have higher significance
 *                          within a structure/type.
 * (ii) little-endian   -   The bytes at lower addre.sses have lower significance
 *                          within a structure/type
 * (iii) middle-endian  -   Neither big-endian nor little-endian, but using
 *                          perverse byte ordering (e.g. 0, 3, 2, 1).
 * (iv) ambi-endian     -   Hardware has the ability to be either little-endian
 *                          or big-endian (usually subject to a machine
 *                          instruction. Also (sadly) known as byte-sexual.
 */

/* The symbol __SYNSOFT_DVS_ARCHITECTURE_BYTEORDER is the symbol that indicates
 * the byte-order of the architecture, and is one of:
 *
 * - __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_BIGENDIAN     (1)
 * - __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_LITTLEENDIAN  (2)
 * - __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_MIDDLEENDIAN  (3)
 * - __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_AMBIENDIAN    (4)
 */

#define __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_BIGENDIAN      (1)
#define __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_LITTLEENDIAN   (2)
#define __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_MIDDLEENDIAN   (3)
#define __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_AMBIENDIAN     (4)

/* Determine the byte-order */

/* Little-endian. */
#if (   (__SYNSOFT_DVS_ARCHITECTURE == __SYNSOFT_VAL_ARCHITECTURE_ALPHA) || \
        (__SYNSOFT_DVS_ARCHITECTURE == __SYNSOFT_VAL_ARCHITECTURE_X86))
# define __SYNSOFT_DVS_ARCHITECTURE_BYTEORDER               __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_LITTLEENDIAN
# define __SYNSOFT_DBS_ARCHITECTURE_BYTEORDER_LITTLEENDIAN
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Architecture byte-ordering: Little-endian")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* Big-endian. */
#elif ( (__SYNSOFT_DVS_ARCHITECTURE == __SYNSOFT_VAL_ARCHITECTURE_HP) || \
        (__SYNSOFT_DVS_ARCHITECTURE == __SYNSOFT_VAL_ARCHITECTURE_POWERPC) || \
        (__SYNSOFT_DVS_ARCHITECTURE == __SYNSOFT_VAL_ARCHITECTURE_SPARC))
# define __SYNSOFT_DVS_ARCHITECTURE_BYTEORDER               __SYNSOFT_VAL_ARCHITECTURE_BYTEORDER_BIGENDIAN
# define __SYNSOFT_DBS_ARCHITECTURE_BYTEORDER_BIGENDIAN
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Architecture byte-ordering: Big-endian")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* No recognised byte-order */
#else
# error Byte-order not discriminated
#endif /* __SYNSOFT_DVS_ARCHITECTURE */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 4. operating system discrimination
 *
 * The currently supported operating systems are as follows:
 *
 * Operating system     Versions                    Symbols
 * ----------------     --------                    -------
 *
 * Unix                 Generic unix                __unix__, __unix, unix
 *                      BSD                         __unix__, __unix, unix, BSD, SYSTYPE_BSD, _SYSTYPE_BSD
 *                      System V                    __unix__, __unix, unix
 *                      SCO                         __unix__, __unix, unix
 *
 *  with the variants   OSF                         __osf__
 *                      Solaris                     sun
 *                      Linux                       __linux__
 *                      HPUX                        __hpux, __hpux__
 *                      AIX                         _AIX (currently only with xlC on the Power PC)
 *
 * VMS                  -                           __vms,  __VMS
 *
 * Windows              16-bit                      _WINDOWS, _Windows
 *                      32-bit                      _WINDOWS, _Windows, _WIN32
 *                      64-bit                      _WINDOWS, _Windows, _WIN64
 */

/* The currently distinguished operating systems are Unix, VMS, and Windows
 * (Win16, Win16-console (DOS), Win32, Win32-console).
 */

/* Console ? */
#define __SYNSOFT_VAL_OS_GENERIC                    (0x00000000)
#define __SYNSOFT_VAL_OS_CONSOLE                    (0x00000002)

/* __SYNSOFT_DVS_OS_BITSIZE is the ambient memory bit-size of the current
 * operating system, and is one of:
 *
 * - __SYNSOFT_VAL_OS_8     (0x00000010)
 * - __SYNSOFT_VAL_OS_16    (0x00000020)
 * - __SYNSOFT_VAL_OS_32    (0x00000040)
 * - __SYNSOFT_VAL_OS_64    (0x00000080)
 */

#define __SYNSOFT_VAL_OS_8                          (0x00000010)
#define __SYNSOFT_VAL_OS_16                         (0x00000020)
#define __SYNSOFT_VAL_OS_32                         (0x00000040)
#define __SYNSOFT_VAL_OS_64                         (0x00000080)
#define __SYNSOFT_VAL_OS_BITMASK                    (0x000003F0)

/* __SYNSOFT_DVS_OS is the version independent symbol that indicates which
 * operating system is detected, and is one of:
 *
 * - __SYNSOFT_VAL_OS_UNIX  (0x00002000)
 * - __SYNSOFT_VAL_OS_VMS   (0x00004000)
 * - __SYNSOFT_VAL_OS_WIN   (0x00001000)
 */

#define __SYNSOFT_VAL_OS_UNIX                       (0x00002000)
#define __SYNSOFT_VAL_OS_VMS                        (0x00004000)
#define __SYNSOFT_VAL_OS_WIN                        (0x00001000)

/* __SYNSOFT_DVS_OS_VER is the version-dependant symbol that indicates which
 * operating system is detected, and is currently one of:
 *
 * - __SYNSOFT_VAL_OS_UNIX32
 * - __SYNSOFT_VAL_OS_UNIX64
 * - __SYNSOFT_VAL_OS_VMS32
 * - __SYNSOFT_VAL_OS_VMS64
 * - __SYNSOFT_VAL_OS_WIN16
 * - __SYNSOFT_VAL_OS_WIN16C
 * - __SYNSOFT_VAL_OS_WIN32
 * - __SYNSOFT_VAL_OS_WIN32C
 * - __SYNSOFT_VAL_OS_WIN64
 * - __SYNSOFT_VAL_OS_WIN64C
 */

#define __SYNSOFT_VAL_OS_UNIX32                     (__SYNSOFT_VAL_OS_UNIX | __SYNSOFT_VAL_OS_GENERIC | __SYNSOFT_VAL_OS_32)
#define __SYNSOFT_VAL_OS_UNIX64                     (__SYNSOFT_VAL_OS_UNIX | __SYNSOFT_VAL_OS_GENERIC | __SYNSOFT_VAL_OS_64)
#define __SYNSOFT_VAL_OS_VMS32                      (__SYNSOFT_VAL_OS_VMS  | __SYNSOFT_VAL_OS_GENERIC | __SYNSOFT_VAL_OS_32)
#define __SYNSOFT_VAL_OS_VMS64                      (__SYNSOFT_VAL_OS_VMS  | __SYNSOFT_VAL_OS_GENERIC | __SYNSOFT_VAL_OS_64)
#define __SYNSOFT_VAL_OS_WIN16                      (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_GENERIC | __SYNSOFT_VAL_OS_16)
#define __SYNSOFT_VAL_OS_WIN16C                     (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_CONSOLE | __SYNSOFT_VAL_OS_16)
#define __SYNSOFT_VAL_OS_WIN32                      (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_GENERIC | __SYNSOFT_VAL_OS_32)
#define __SYNSOFT_VAL_OS_WIN32C                     (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_CONSOLE | __SYNSOFT_VAL_OS_32)
#define __SYNSOFT_VAL_OS_WIN64                      (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_GENERIC | __SYNSOFT_VAL_OS_64)
#define __SYNSOFT_VAL_OS_WIN64C                     (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_CONSOLE | __SYNSOFT_VAL_OS_64)

/* For UNIX, the following variants are discriminated:
 *
 * - AIX        (__SYNSOFT_DBS_UNIX_VARIANT_AIX)
 * - HPUX       (__SYNSOFT_DBS_UNIX_VARIANT_HPUX)
 * - Linux      (__SYNSOFT_DBS_UNIX_VARIANT_LINUX)
 * - OSF        (__SYNSOFT_DBS_UNIX_VARIANT_OSF)
 * - Solaris    (__SYNSOFT_DBS_UNIX_VARIANT_SOLARIS)
 */


/* Determine the operating system, validating that it is recognised */

/* Shortcut check for Windows. */
#if (	!defined(UNIX) && \
		!defined(__unix__) && \
		!defined(__unix) && \
		!defined(unix) \
		) && \
    (   defined(_Windows) || \
        defined(_WINDOWS) || \
        defined(WIN32) || \
        /* defined(_WIN32) || */ \
        defined(__WIN32__) || \
        defined(WIN64) || \
        defined(_WIN64) || \
        defined(__WIN64__) || \
        defined(_DOS) || \
        defined(__MSDOS__))
# ifndef _SYB_NO_EARLY_WINDOWS_DETECTION
#  define __SYNSOFT_DVS_OS                           __SYNSOFT_VAL_OS_WIN
# endif /* !_SYB_NO_EARLY_WINDOWS_DETECTION */
#endif /* _Windows || _WINDOWS || WIN32 || _WIN32 || _WIN64 */

/* Windows operating system */
#if defined(__SYNSOFT_DVS_OS) && \
    __SYNSOFT_DVS_OS == __SYNSOFT_VAL_OS_WIN
/* All Synesis development in Windows has STRICT defined. */
# ifndef STRICT
#  define STRICT                                    (1)
# endif /* !STRICT */
/* Now determine the version of Windows */
#if defined(WIN64) || \
    defined(_WIN64) || \
    defined(__WIN64__)
# if defined(_CONSOLE) || \
     defined(__CONSOLE__)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Operating system: Windows Console")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_WIN64C
# else
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Operating system: Windows")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_WIN64
# endif /* _CONSOLE || __CONSOLE__ */
#elif defined(WIN32) || \
      defined(_WIN32) || \
      defined(__WIN32__)
# if defined(_CONSOLE) || \
     defined(__CONSOLE__)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Operating system: Windows Console")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_WIN32C
# else
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Operating system: Windows")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_WIN32
# endif /* _CONSOLE || __CONSOLE__ */
#else /* !WIN32 */
# if defined(_DOS) || \
     defined(__MSDOS__)
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Operating system: DOS")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_WIN16C
# else
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Operating system: Windows")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_WIN16
# endif /* _DOS || __MSDOS__ */
#endif /* WIN32 */

/* UNIX operating system */
#elif defined(unix) || \
      defined(UNIX) || \
      defined(__unix) || \
      defined(__unix__) || \
      ( defined(__xlC__) && \
        defined(_POWER) && \
        defined(_AIX))
# undef __SYNSOFT_DVS_OS
# define __SYNSOFT_DVS_OS                           __SYNSOFT_VAL_OS_UNIX
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Operating system: UNIX")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# ifdef __arch64__
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_UNIX64
# else
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_UNIX32
# endif /* __arch64__ */
/* Now determine UNIX variant. */
# if defined(__osf__)
#  define __SYNSOFT_DBS_UNIX_VARIANT_OSF
# elif defined(sun)
#  define __SYNSOFT_DBS_UNIX_VARIANT_SOLARIS
# elif defined(__linux__)
#  define __SYNSOFT_DBS_UNIX_VARIANT_LINUX
# elif defined(__hpux) || \
       defined(__hpux__)
#  define __SYNSOFT_DBS_UNIX_VARIANT_HPUX
# elif defined(_AIX)
#  define __SYNSOFT_DBS_UNIX_VARIANT_AIX
# endif /*  */
/* END: Unix operating system (DEC or Gnu C++ compiler) */

/* VMS operating system */
#elif defined(__vms) || \
      defined(__VMS)
# undef __SYNSOFT_DVS_OS
# define __SYNSOFT_DVS_OS                           __SYNSOFT_VAL_OS_VMS
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Operating system: VMS")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# ifdef __arch64__
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_VMS64
# else
#  define __SYNSOFT_DVS_OS_VER                      __SYNSOFT_VAL_OS_VMS32
# endif /* __arch64__ */
/* END: VMS operating system (DEC or Gnu C++ compiler) */

/* Unknown operating system */
#else
# undef __SYNSOFT_DVS_OS
# define __SYNSOFT_DVS_OS
# define __SYNSOFT_DVS_OS_VER
# error Operating system not recognised
#endif /* __SYNSOFT_DVS_OS */

/* /////////////////////////////////////////////////////////////////////////////
 * 4.1. operating system bit-size
 *
 * The currently recognised operating system bit-sizes are:
 *
 * - 8
 * - 16
 * - 32
 * - 64
 */

#if __SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_64
# define __SYNSOFT_DVS_OS_BITSIZE                   (64)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Operating system bit-size: 64")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_32
# define __SYNSOFT_DVS_OS_BITSIZE                   (32)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Operating system bit-size: 32")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_16
# define __SYNSOFT_DVS_OS_BITSIZE                   (16)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Operating system bit-size: 16")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#elif __SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_8
# define __SYNSOFT_DVS_OS_BITSIZE                   (8)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Operating system bit-size: 8")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#else
# error Could not determine platform bit-size
#endif /* __SYNSOFT_DVS_OS_VER */

/* /////////////////////////////////////////////////////////////////////////////
 * 4.2. operating system object alignment determination
 *
 * The operating system object-alignment is determined assumed to be 8 bytes on
 * all current platforms.
 */

#define __SYNSOFT_DVS_OBJECT_ALIGNMENT              (8)

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# if __SYNSOFT_DVS_OBJECT_ALIGNMENT == 128
#  pragma message("    Operating system object alignment size: 128")
# elif __SYNSOFT_DVS_OBJECT_ALIGNMENT == 64
#  pragma message("    Operating system object alignment size: 64")
# elif __SYNSOFT_DVS_OBJECT_ALIGNMENT == 32
#  pragma message("    Operating system object alignment size: 32")
# elif __SYNSOFT_DVS_OBJECT_ALIGNMENT == 16
#  pragma message("    Operating system object alignment size: 16")
# elif __SYNSOFT_DVS_OBJECT_ALIGNMENT == 8
#  pragma message("    Operating system object alignment size: 8")
# elif __SYNSOFT_DVS_OBJECT_ALIGNMENT == 4
#  pragma message("    Operating system object alignment size: 4")
# elif __SYNSOFT_DVS_OBJECT_ALIGNMENT == 2
#  pragma message("    Operating system object alignment size: 2")
# elif __SYNSOFT_DVS_OBJECT_ALIGNMENT == 1
#  pragma message("    Operating system object alignment size: 1")
# endif /* __SYNSOFT_DVS_OS_VER */
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 5. function specifications
 *
 * Function specifications consist of:
 *
 * - linkage class (eg. extern "C")
 * - storage class (eg. __declspec(naked))
 * - type (eg. int)
 * - memory model (eg. _near)
 * - calling convention (eg. __fastcall)
 */

/* /////////////////////////////////////////////////////////////////////////////
 * 5.1. linkage class
 *
 * The three linkage classes are:
 *
 * - __SYNSOFT_MOD_LINKAGE_NULL     (null for both C & C++)
 * - __SYNSOFT_MOD_LINKAGE_C        (extern for C, extern "C" for C++)
 * - __SYNSOFT_MOD_LINKAGE_CPP      (does not exist for C, extern "C++" for C++)
 */

#define __SYNSOFT_MOD_LINKAGE_NULL
#ifdef __cplusplus
# define __SYNSOFT_MOD_LINKAGE_C                    extern "C"
# define __SYNSOFT_GEN_STARTLINKAGE_C               extern "C" {
# define __SYNSOFT_GEN_ENDLINKAGE_C                 }
# define __SYNSOFT_MOD_LINKAGE_CPP                  extern "C++"
# define __SYNSOFT_GEN_STARTLINKAGE_CPP             extern "C++" {
# define __SYNSOFT_GEN_ENDLINKAGE_CPP               }
#else
# define __SYNSOFT_MOD_LINKAGE_C                    extern
# define __SYNSOFT_GEN_STARTLINKAGE_C
# define __SYNSOFT_GEN_ENDLINKAGE_C
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * 5.2. storage class
 *
 * The three storage classes are:
 *
 * - __SYNSOFT_MOD_STGCLASS_NULL    (null)
 * - __SYNSOFT_MOD_STGCLASS_VIRTUAL (virtual in C++, null in C)
 * - __SYNSOFT_MOD_STGCLASS_STATIC  (static in C++, does not exist in C)
 * - __SYNSOFT_MOD_STGCLASS_IMPORT  (__declspec(dllimport/__dllimport) for Win32, or null)
 * - __SYNSOFT_MOD_STGCLASS_EXPORT  (__declspec(dllexport/__dllexport) for Win32, or __export for Win16, or null)
 * - __SYNSOFT_MOD_STGCLASS_NAKED   (__declspec(naked) for Win32, or null)
 */

#define __SYNSOFT_MOD_STGCLASS_NULL_
#define __SYNSOFT_MOD_STGCLASS_NULL                 __SYNSOFT_MOD_STGCLASS_NULL_
#ifdef __cplusplus
# define    __SYNSOFT_MOD_STGCLASS_VIRTUAL          virtual
# define    __SYNSOFT_MOD_STGCLASS_STATIC           static
#else
# define    __SYNSOFT_MOD_STGCLASS_VIRTUAL
#endif /* __cplusplus */
#if __SYNSOFT_DVS_OS == __SYNSOFT_VAL_OS_WIN
# if (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN32) == __SYNSOFT_VAL_OS_WIN32
#  if (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND) /* Borland C++. */ && \
      (__SYNSOFT_DVS_COMPILER_VER_BORLAND < __SYNSOFT_GEN_COMPILER_VERSION(5, 5))
#   define  __SYNSOFT_MOD_STGCLASS_IMPORT           __declspec(__dllimport)
#   define  __SYNSOFT_MOD_STGCLASS_EXPORT           __declspec(__dllexport)
#   define  __SYNSOFT_MOD_STGCLASS_NAKED
#  elif (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND) /* Borland C++. */ || \
        (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC)      /* Digital Mars C/C++. */ || \
        (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC)     /* GNU C/C++. */ || \
        (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL)    /* Intel C/C++. */ || \
        (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS)   /* Metrowerks C/C++. */ || \
        (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC)     /* Microsoft C++. */ || \
        (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_WATCOM)   /* Watcom C++. */
#   define  __SYNSOFT_MOD_STGCLASS_IMPORT           __declspec(dllimport)
#   define  __SYNSOFT_MOD_STGCLASS_EXPORT           __declspec(dllexport)
#   define  __SYNSOFT_MOD_STGCLASS_NAKED            __declspec(naked)
#  else
#   error Unrecognised Windows compiler
#  endif /* compiler */
# elif (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN16) == __SYNSOFT_VAL_OS_WIN16
#   define  __SYNSOFT_MOD_STGCLASS_IMPORT
#   define  __SYNSOFT_MOD_STGCLASS_EXPORT           __export
#   define  __SYNSOFT_MOD_STGCLASS_NAKED
# else
#  error __SYNSOFT_MOD_STGCLASS_IMPORT, __SYNSOFT_MOD_STGCLASS_EXPORT and __SYNSOFT_MOD_STGCLASS_NAKED not currently discriminated for other than 16-bit and 32-bit Windows
# endif /* __SYNSOFT_DVS_OS_VER */
#else
# define __SYNSOFT_MOD_STGCLASS_IMPORT
# define __SYNSOFT_MOD_STGCLASS_EXPORT
# define  __SYNSOFT_MOD_STGCLASS_NAKED
#endif /* __SYNSOFT_DVS_OS */

/* /////////////////////////////////////////////////////////////////////////////
 * 5.3. memory model
 *
 * The four memory models are:
 *
 * - __SYNSOFT_MOD_MEMMODEL_NULL
 * - __SYNSOFT_MOD_MEMMODEL_NEAR
 * - __SYNSOFT_MOD_MEMMODEL_FAR
 * - __SYNSOFT_MOD_MEMMODEL_HUGE
 */

#define __SYNSOFT_MOD_MEMMODEL_NEAR_()
#define __SYNSOFT_MOD_MEMMODEL_FAR_()
#define __SYNSOFT_MOD_MEMMODEL_HUGE_()

#define __SYNSOFT_MOD_MEMMODEL_NULL
#if (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN16) == __SYNSOFT_VAL_OS_WIN16
# define __SYNSOFT_MOD_MEMMODEL_NEAR                __near
# define __SYNSOFT_MOD_MEMMODEL_FAR                 __far
# define __SYNSOFT_MOD_MEMMODEL_HUGE                __huge
#else
# define __SYNSOFT_MOD_MEMMODEL_NEAR                __SYNSOFT_MOD_MEMMODEL_NEAR_()
# define __SYNSOFT_MOD_MEMMODEL_FAR                 __SYNSOFT_MOD_MEMMODEL_FAR_()
# define __SYNSOFT_MOD_MEMMODEL_HUGE                __SYNSOFT_MOD_MEMMODEL_HUGE_()
#endif /* __SYNSOFT_VAL_OS_WIN16 */

/* /////////////////////////////////////////////////////////////////////////////
 * 5.4. calling convention
 *
 * The three calling conventions are:
 *
 * - __SYNSOFT_MOD_CALLCONV_NULL
 * - __SYNSOFT_MOD_CALLCONV_CDECL       (cdecl)
 * - __SYNSOFT_MOD_CALLCONV_STDCALL     (standard call)
 * - __SYNSOFT_MOD_CALLCONV_FASTCALL    (fastcall)
 * - __SYNSOFT_MOD_CALLCONV_DEFAULT     (default in Synesis)
 * - __SYNSOFT_MOD_CALLCONV_VDECL       (used for virtual methods)
 * - __SYNSOFT_MOD_CALLCONV_STATDECL    (used for static methods)
 */

#define __SYNSOFT_MOD_CALLCONV_NULL_()
#define __SYNSOFT_MOD_CALLCONV_NULL                 __SYNSOFT_MOD_CALLCONV_NULL_()

#if __SYNSOFT_DVS_OS == __SYNSOFT_VAL_OS_WIN
# if (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN32) == __SYNSOFT_VAL_OS_WIN32
#  define __SYNSOFT_MOD_CALLCONV_CDECL              __cdecl
#  define __SYNSOFT_MOD_CALLCONV_STDCALL            __stdcall
#  define __SYNSOFT_MOD_CALLCONV_FASTCALL           __fastcall
#  define __SYNSOFT_MOD_CALLCONV_DEFAULT            __stdcall
# elif (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN16) == __SYNSOFT_VAL_OS_WIN16
#  define __SYNSOFT_MOD_CALLCONV_CDECL              _cdecl
#  define __SYNSOFT_MOD_CALLCONV_STDCALL            _pascal
#  define __SYNSOFT_MOD_CALLCONV_FASTCALL           _pascal
#  define __SYNSOFT_MOD_CALLCONV_DEFAULT            _pascal
# else
#  error Calling conventions not discriminated for platforms other than 16-bit and 32-bit Windows
# endif /* __SYNSOFT_DVS_OS_VER */
#else
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
#  define __SYNSOFT_MOD_CALLCONV_CDECL              __cdecl
#  define __SYNSOFT_MOD_CALLCONV_STDCALL            __stdcall
#  define __SYNSOFT_MOD_CALLCONV_FASTCALL           __fastcall
#  define __SYNSOFT_MOD_CALLCONV_DEFAULT            __stdcall
# else /* ? __SYNSOFT_DVS_COMPILER */
#  define __SYNSOFT_MOD_CALLCONV_CDECL              __SYNSOFT_MOD_CALLCONV_NULL
#  define __SYNSOFT_MOD_CALLCONV_STDCALL            __SYNSOFT_MOD_CALLCONV_NULL
#  define __SYNSOFT_MOD_CALLCONV_FASTCALL           __SYNSOFT_MOD_CALLCONV_NULL
#  define __SYNSOFT_MOD_CALLCONV_DEFAULT            __SYNSOFT_MOD_CALLCONV_NULL
# endif /* __SYNSOFT_DVS_COMPILER */
#endif /* __SYNSOFT_VAL_OS_WIN ?? */

#ifdef _SYB_VDECL_CALLCONV
# define __SYNSOFT_MOD_CALLCONV_VDECL               _SYB_VDECL_CALLCONV
#else /* ? _SYB_VDECL_CALLCONV */
# ifdef _SYB_VIRT_DECL
#  define __SYNSOFT_MOD_CALLCONV_VDECL              _SYB_VIRT_DECL
# else
#  define __SYNSOFT_MOD_CALLCONV_VDECL              __SYNSOFT_MOD_CALLCONV_DEFAULT
# endif /* _SYB_NO_VIRT_DECL */
#endif /* _SYB_VDECL_CALLCONV */

#ifdef _SYB_STAT_DECL
# define __SYNSOFT_MOD_CALLCONV_STATDECL            _SYB_STAT_DECL
#else
# define __SYNSOFT_MOD_CALLCONV_STATDECL            __SYNSOFT_MOD_CALLCONV_DEFAULT
#endif /* _SYB_NO_STAT_DECL */

/* /////////////////////////////////////////////////////////////////////////////
 * 5.5. function specification generator macros
 *
 * Since certain compilers require these items in different order, so one of the
 * macros SYFNDECL__(l, s, t, m, c) or SYFNDECLREF__(l, s, t, m, c) should be
 * used in the declaration and definition of functions, eg.
 *
 * SYFNDECL__(__SYNSOFT_MOD_LINKAGE_C, __SYNSOFT_MOD_STGCLASS_NULL, int, __SYNSOFT_MOD_MEMMODEL_FAR, __SYNSOFT_MOD_CALLCONV_FASTCALL) func(void)
 *
 * or by using one of the derived macros, eg.
 *
 * SYFNCDECL(int) func(void)
 *
 * The linkage class, storage class, memory model and calling convention are
 * discriminated in the following area.
 */

/* Fundamental function specification generator macros
 *
 * l - linkage class        (one of __SYNSOFT_MOD_LINKAGE_????)
 * s - storage class        (one of __SYNSOFT_MOD_STGCLASS_????)
 * t - type
 * m - memory model         (one of __SYNSOFT_MOD_MEMMODEL_????)
 * c - calling convention   (one of __SYNSOFT_MOD_CALLCONV_????)
 */
#if __SYNSOFT_DVS_OS == __SYNSOFT_VAL_OS_WIN
# if (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN32) == __SYNSOFT_VAL_OS_WIN32
#  define SYFNDECL__(l, s, t, m, c)                   l s t m c
#  define SYFNDECLREF__(l, s, t, m, c)                l s t m *c
# elif (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN16) == __SYNSOFT_VAL_OS_WIN16
#  define SYFNDECL__(l, s, t, m, c)                   l t m c s
#  define SYFNDECLREF__(l, s, t, m, c)                l t m *c s
# else
#  error Function specification generator macros not discriminated for platforms other than 16-bit and 32-bit Windows
# endif /* __SYNSOFT_DVS_OS_VER */
#else
 /* Other platforms are assumed to only require linkage and type */
#if 0
# define SYFNDECL__(l, s, t, m, c)                   l t
# define SYFNDECLREF__(l, s, t, m, c)                l t *
#else /* ? 0 */
# define SYFNDECL__(l, s, t, m, c)                   l s t m c
# define SYFNDECLREF__(l, s, t, m, c)                l s t m *c
#endif /* 0 */
#endif /* __SYNSOFT_VAL_OS_WIN16 */

/* SYFNDECL(REF)_ declares functions as extern "C" and far, and is suitable for C and non-overloaded C++ functions */
#define SYFNDECL_(t, c)                             SYFNDECL__(__SYNSOFT_MOD_LINKAGE_C, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
#define SYFNDECLREF_(t, c)                          SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_C, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)

/* SYFNCXXDECL(REF)_ declares functions as far, and is suitable for overloaded C++ functions */
#define SYFNCXXDECL_(t, c)                          SYFNDECL__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
#define SYFNCXXDECLREF_(t, c)                       SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)

/* SYFNVIRT_ declares functions for C++ virtual members that are compatible with
 * C interfaces.
 */
#define SYFNVIRT_(t, c)                             SYFNDECL__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_VIRTUAL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
#define SYFNVIRTREF_(t, c)                          SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_VIRTUAL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
# define SYFNVIRT_IMPL_(t, c)                       SYFNDECL__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
# define SYFNVIRTREF_IMPL_(t, c)                    SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
#else
# define SYFNVIRT_IMPL_(t, c)                       SYFNDECL__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_VIRTUAL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
# define SYFNVIRTREF_IMPL_(t, c)                    SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_VIRTUAL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
#endif /* __SYNSOFT_VAL_COMPILER_BORLAND */

/* SYFNSTAT_ declares functions for C++ static members.
 */
#ifdef __cplusplus
# define SYFNSTAT_(t, c)                            SYFNDECL__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_STATIC, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
# define SYFNSTATREF_(t, c)                         SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_STATIC, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
# define SYFNSTAT_IMPL_(t, c)                       SYFNDECL__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
# define SYFNSTATREF_IMPL_(t, c)                    SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_NULL, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
#else
# define SYFNSTAT_IMPL_(t, c)                       SYFNDECL__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_STATIC, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
# define SYFNSTATREF_IMPL_(t, c)                    SYFNDECLREF__(__SYNSOFT_MOD_LINKAGE_NULL, __SYNSOFT_MOD_STGCLASS_STATIC, t, __SYNSOFT_MOD_MEMMODEL_FAR, c)
#endif /* __SYNSOFT_VAL_COMPILER_BORLAND */
#endif /* __cplusplus */


/* Functions are extern "C" and far with __SYNSOFT_MOD_CALLCONV_DEFAULT */
#define SYFNDECL(t)                                 SYFNDECL_(t, __SYNSOFT_MOD_CALLCONV_DEFAULT)
#define SYFNDECLREF(t)                              SYFNDECLREF_(t, __SYNSOFT_MOD_CALLCONV_DEFAULT)

/* Functions are extern "C" and far with __SYNSOFT_MOD_CALLCONV_CDECL */
#define SYFNCDECL(t)                                SYFNDECL_(t, __SYNSOFT_MOD_CALLCONV_CDECL)
#define SYFNCDECLREF(t)                             SYFNDECLREF_(t, __SYNSOFT_MOD_CALLCONV_CDECL)

/* Functions are far with __SYNSOFT_MOD_CALLCONV_CDECL */
#define SYFNDECLXX(t)                               SYFNCXXDECL_(t, __SYNSOFT_MOD_CALLCONV_CDECL)
#define SYFNDECLXXREF(t)                            SYFNCXXDECLREF_(t, __SYNSOFT_MOD_CALLCONV_CDECL)

/* Functions are virtual and far with __SYNSOFT_MOD_CALLCONV_VDECL */
#define SYFNVIRT(t)                                 SYFNVIRT_(t, __SYNSOFT_MOD_CALLCONV_VDECL)
#define SYFNVIRTREF(t)                              SYFNVIRTREF_(t, __SYNSOFT_MOD_CALLCONV_VDECL)
#define SYFNVIRT_IMPL(t)                            SYFNVIRT_IMPL_(t, __SYNSOFT_MOD_CALLCONV_VDECL)
#define SYFNVIRTREF_IMPL(t)                         SYFNVIRTREF_IMPL_(t, __SYNSOFT_MOD_CALLCONV_VDECL)

/* Functions are static and far with __SYNSOFT_MOD_CALLCONV_STATDECL */
#ifdef __cplusplus
# define SYFNSTAT(t)                                SYFNSTAT_(t, __SYNSOFT_MOD_CALLCONV_STATDECL)
# define SYFNSTATREF(t)                             SYFNSTATREF_(t, __SYNSOFT_MOD_CALLCONV_STATDECL)
# define SYFNSTAT_IMPL(t)                           SYFNSTAT_IMPL_(t, __SYNSOFT_MOD_CALLCONV_STATDECL)
# define SYFNSTATREF_IMPL(t)                        SYFNSTATREF_IMPL_(t, __SYNSOFT_MOD_CALLCONV_STATDECL)
#endif /* __cplusplus */

/* Functions are extern "C" and far with __SYNSOFT_MOD_CALLCONV_DEFAULT, returning HRESULT */
#define SYFNCOMDECL                                 SYFNDECL(HRESULT)
#define SYFNCOMDECLXX                               SYFNDECLXX(HRESULT)

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 6. Casting
 *
 * Compiler support for C++ static_cast, dynamic_cast, const_cast,
 * reinterpret_cast and C-style cast.
 *
 * Macros are defined for casting operations as follows:
 *
 * (i) static_cast          -   SyCastStatic
 * (ii) dynamic_cast        -   SyDynCast
 * (iii) const_cast         -   SyConstCast
 * (iv) const_cast          -   SyVolatileCast
 * (v) reinterpret_cast     -   SyRawCast
 * (vi) C-style case        -   SyCastC
 */

#if defined(__cplusplus) && \
    (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(4, 2)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_WERKS >= __SYNSOFT_GEN_COMPILER_VERSION(7, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_WATCOM >= __SYNSOFT_GEN_COMPILER_VERSION(11, 0)))
# define __SYNSOFT_DBS_CAST_OPERATOR_SUPPORT
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    C++: cast-operator support")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* __SYNSOFT_DVS_COMPILER && __cplusplus */

#ifdef __SYNSOFT_DBS_CAST_OPERATOR_SUPPORT
#define SyCastStatic(_type, _var)                   static_cast<_type>(_var)
#define SyCastDynamic(_type, _var)                  dynamic_cast<_type>(_var)
#define SyCastConst(_type, _var)                    const_cast<_type>(_var)
#define SyCastVolatile(_type, _var)                 const_cast<_type>(_var)
#define SyCastRaw(_type, _var)                      reinterpret_cast<_type>(_var)
#else
#define SyCastStatic(_type, _var)                   ((_type)(_var))
#define SyCastDynamic(_type, _var)                  ((_type)(_var))
#define SyCastConst(_type, _var)                    ((_type)(_var))
#define SyCastVolatile(_type, _var)                 ((_type)(_var))
#define SyCastRaw(_type, _var)                      ((_type)(_var))
#endif /* __SYNSOFT_DBS_CAST_OPERATOR_SUPPORT */
#define SyCastC(_type, _var)                        ((_type)(_var))

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 7. namespace support discrimination
 *
 * The following symbols are discriminated:
 *
 * - __SYNSOFT_DBS_NAMESPACE_SUPPORT is defined when the compiler supports
 *   namespaces. Define the symbol _SYB_USE_NAMESPACES to force namespace
 *   support, even when not recognised as a compiler capability.
 * - __SYNSOFT_DBS_NAMESPACE_SYNESIS is defined when Synesis code utilises
 *   namespaces. Define the symbol _SYB_NO_NAMESPACES to prevent namespace
 *   support in Synesis code.
 */

/* __SYNSOFT_DBS_NAMESPACE_SUPPORT */
#if defined(__cplusplus) && \
    (   (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26)) || \
        (__SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) || \
        (__SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)) || \
        (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)) || \
        (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)) || \
        (__SYNSOFT_DVS_COMPILER_VER_WATCOM >= __SYNSOFT_GEN_COMPILER_VERSION(11, 0)))
# define __SYNSOFT_DBS_NAMESPACE_SUPPORT
#endif /* __SYNSOFT_DVS_COMPILER && __cplusplus */

/* __SYNSOFT_DBS_NAMESPACE_SYNESIS */
#ifdef __SYNSOFT_DBS_NAMESPACE_SUPPORT
# ifndef _SYB_NO_NAMESPACES
#  define __SYNSOFT_DBS_NAMESPACE_SYNESIS
# endif /* !_SYB_NO_NAMESPACES */
#endif /* __SYNSOFT_DBS_NAMESPACE_SUPPORT */

#ifdef __SYNSOFT_DBS_NAMESPACE_SUPPORT
# define __SYNSOFT_GEN_NS_3PTY_START(x)                     namespace x {
# define __SYNSOFT_GEN_NS_3PTY_END(x)                       }
# define __SYNSOFT_GEN_NS_3PTY_SWAP(ns1, ns2)               } namespace ns2 {
# define __SYNSOFT_GEN_NS_3PTY_END_DECL(x)                  }
# define __SYNSOFT_GEN_NS_3PTY_END_IMPL(x)                  }
# define __SYNSOFT_GEN_NS_3PTY_ANON_START                   namespace {
# define __SYNSOFT_GEN_NS_3PTY_ANON_END                     }
# define __SYNSOFT_GEN_NS_3PTY_USING(x)                     using namespace x;
# define __SYNSOFT_GEN_USING_3PTY(x)                        using x;
# define __SYNSOFT_GEN_NS_3PTY_QUAL(ns, ident)              ns::ident
# define __SYNSOFT_GEN_NS_3PTY_QUAL_OR_GLBL(ns, ident)      ns::ident
#else
# define __SYNSOFT_GEN_NS_3PTY_START(x)
# define __SYNSOFT_GEN_NS_3PTY_END(x)
# define __SYNSOFT_GEN_NS_3PTY_SWAP(ns1, ns2)
# define __SYNSOFT_GEN_NS_3PTY_END_DECL(x)
# define __SYNSOFT_GEN_NS_3PTY_END_IMPL(x)
# define __SYNSOFT_GEN_NS_3PTY_ANON_START
# define __SYNSOFT_GEN_NS_3PTY_ANON_END
# define __SYNSOFT_GEN_NS_3PTY_USING(x)
# define __SYNSOFT_GEN_USING_3PTY(x)
# define __SYNSOFT_GEN_NS_3PTY_QUAL(ns, ident)              ident
# ifdef __cplusplus
#  define __SYNSOFT_GEN_NS_3PTY_QUAL_OR_GLBL(ns, ident)     ::ident
# else
#  define __SYNSOFT_GEN_NS_3PTY_QUAL_OR_GLBL(ns, ident)     ident
# endif /* __cplusplus */
#endif /* __SYNSOFT_DBS_NAMESPACE_SUPPORT */

#ifdef __SYNSOFT_DBS_NAMESPACE_SYNESIS
# define __SYNSOFT_GEN_NS_START(x)                          namespace x {
# define __SYNSOFT_GEN_NS_END(x)                            }
# define __SYNSOFT_GEN_NS_SWAP(ns1, ns2)                    } namespace ns2 {
# define __SYNSOFT_GEN_NS_END_DECL(x)                       }
# define __SYNSOFT_GEN_NS_END_IMPL(x)                       }
# define __SYNSOFT_GEN_NS_ANON_START                        namespace {
# define __SYNSOFT_GEN_NS_ANON_END                          }
# define __SYNSOFT_GEN_NS_USING(x)                          using namespace x;
# define __SYNSOFT_GEN_USING(x)                             using x;
# define __SYNSOFT_GEN_NS_QUAL(ns, ident)                   ns::ident
# define __SYNSOFT_GEN_NS_QUAL_OR_GLBL(ns, ident)           ns::ident
#else /* ? __SYNSOFT_DBS_NAMESPACE_SYNESIS */
# define __SYNSOFT_GEN_NS_START(x)
# define __SYNSOFT_GEN_NS_END(x)
# define __SYNSOFT_GEN_NS_SWAP(ns1, ns2)
# define __SYNSOFT_GEN_NS_END_DECL(x)
# define __SYNSOFT_GEN_NS_END_IMPL(x)
# define __SYNSOFT_GEN_NS_ANON_START
# define __SYNSOFT_GEN_NS_ANON_END
# define __SYNSOFT_GEN_NS_USING(x)
# define __SYNSOFT_GEN_USING(x)
# define __SYNSOFT_GEN_NS_QUAL(ns, ident)                   ident
# ifdef __cplusplus
#  define __SYNSOFT_GEN_NS_QUAL_OR_GLBL(ns, ident)          ::ident
# else
#  define __SYNSOFT_GEN_NS_QUAL_OR_GLBL(ns, ident)          ident
# endif /* __cplusplus */
#endif /* __SYNSOFT_DBS_NAMESPACE_SYNESIS */

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# ifdef __SYNSOFT_DBS_NAMESPACE_SUPPORT
#  ifdef __SYNSOFT_DBS_NAMESPACE_SYNESIS
#   pragma message("    C++: Using namespaces for Synesis code")
#  else
#   pragma message("    C++: Namespaces supported. Not used for Synesis code")
#  endif /* __SYNSOFT_DBS_NAMESPACE_SYNESIS */
# else
#  pragma message("    C++: Namespaces not supported")
# endif /* __SYNSOFT_DBS_NAMESPACE_SUPPORT */
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 8. multithreading support discrimination
 *
 * By default platforms are assumed to be not multi-threaded. Multi-threaded
 * support is discriminated by the definition of preprocessor symbols. The
 * symbols for each compiler are as follows:
 *
 * - Borland                __MT__
 * - Dec C                  _REENTRANT
 * - Dec C++                _REENTRANT
 * - CodeWarrior            _MT
 * - Digital Mars           _MT
 * - Gnu C                  _REENTRANT
 * - Metrowerks             _MT
 * - Microsoft Visual C++   _MT
 * - xlC                    _REENTRANT
 * - Watcom C/C++           unknown
 */

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND
# ifdef __MT__
#  define __SYNSOFT_DBS_MULTITHREADED_PLATFORM
#  define __SYNSOFT_DBS_MULTITHREADED_SUPPORT
# endif /* __MT__ */
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_WATCOM
# ifdef _MT
#  define __SYNSOFT_DBS_MULTITHREADED_PLATFORM
#  define __SYNSOFT_DBS_MULTITHREADED_SUPPORT
# endif /* _MT */
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_XLC
# ifdef _REENTRANT
#  define __SYNSOFT_DBS_MULTITHREADED_PLATFORM
#  define __SYNSOFT_DBS_MULTITHREADED_SUPPORT
# endif /* _REENTRANT */
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS
# if defined(_REENTRANT) || \
     defined(_MT)
#  define __SYNSOFT_DBS_MULTITHREADED_PLATFORM
#  define __SYNSOFT_DBS_MULTITHREADED_SUPPORT
# endif /* _REENTRANT || _MT */
#else
# error Compiler not recognised
#endif /* __SYNSOFT_VAL_COMPILER_MSVC */

#ifdef _SYB_NO_MULTITHREADING
# ifdef __SYNSOFT_DBS_MULTITHREADED_SUPPORT
#  undef __SYNSOFT_DBS_MULTITHREADED_SUPPORT
# endif /* __SYNSOFT_DBS_MULTITHREADED_SUPPORT */
#endif /* _SYB_NO_MULTITHREADING */

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# ifdef __SYNSOFT_DBS_MULTITHREADED_SUPPORT
#  if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
#   if defined(_DLL)
#    pragma message("    Multithreading (DLL)")
#   else
#    pragma message("    Multithreading (Static)")
#   endif /* (__SYNSOFT_VAL_COMPILER_DMC || __SYNSOFT_VAL_COMPILER_MSVC) && _DLL */
#  else
#   pragma message("    Multithreading")
#  endif /* __SYNSOFT_DVS_COMPILER */
# else
#  pragma message("    Not multithreading")
# endif /* __SYNSOFT_DBS_MULTITHREADED_SUPPORT */
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

#if defined(__SYNSOFT_DBS_MULTITHREADED_PLATFORM) && \
    !defined(__SYNSOFT_DBS_MULTITHREADED_SUPPORT)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Warning: Ambient operating system multithreading support has been overriden!")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* __SYNSOFT_DBS_MULTITHREADED_PLATFORM) && !__SYNSOFT_DBS_MULTITHREADED_SUPPORT */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 9. debugging support
 *
 * This section defines the symbols used for debugging in the Synesis Software
 * code-base.
 *
 * The symbol __SYNSOFT_DBS_DEBUG is defined when debugging is enabled in
 * preprocessing. This definition is made when the symbol _SYB_DEBUG is defined,
 * or, in some compilers, also automatically when _DEBUG is defined. When the
 * symbol NDEBUG is defined, __SYNSOFT_DBS_DEBUG is undefined.
 *
 * The macros SyAssert() and SyVerify() provide debug-time assertions. They are
 * intended to be redefined by a client in the file M3DbgExt.h, but are
 * otherwise defined to have no effect. In release code SyAssert() evaluates to
 * nothing and SyVerify() evaluates to the given expression.
 */

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECC || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC || \
    __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_XLC
# ifndef NDEBUG
#  define _SYB_DEBUG
# else
#  define _SYB_NDEBUG
# endif /* NDEBUG */
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_WATCOM
# ifdef NDEBUG
#  define _SYB_NDEBUG
# elif defined(_DEBUG)
#  define _SYB_DEBUG
# else
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Warning: Compilation with Borland, Digital Mars, Microsoft or Watcom compilers presumes the presence of either _DEBUG or NDEBUG. In the abscence of both, debug is assumed")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#  define _SYB_DEBUG
# endif /* NDEBUG */
#else
# error Compiler not recognised
#endif /* compiler */

#ifdef _SYB_NDEBUG
# define __SYNSOFT_DBS_NDEBUG
#elif defined(_SYB_DEBUG)
# define __SYNSOFT_DBS_DEBUG
#endif /* _SYNSOFT_NDEBUG / _SYNSOFT_DEBUG */

#ifndef _SYB_NO_CUSTOM_DBGEXT
#include <M3DbgExt.h>
#endif /* !_SYB_NO_CUSTOM_DBGEXT */

/* Standard debug macros. Define them to have no effect, if they have not been
 * defined in M3DbgExt.h.
 */
#ifndef SyAssert
# ifdef __SYNSOFT_DBS_DEBUG
#  define SyAssert(_expr)                           (SyCastStatic(void, ((_expr) ? (0) : (0))))
# else /* ? __SYNSOFT_DBS_DEBUG */
#  define SyAssert(_expr)                           (SyCastStatic(void, 0))
# endif /* __SYNSOFT_DBS_DEBUG */
#endif /* SyAssert */

#ifndef SyVerify
# ifdef __SYNSOFT_DBS_DEBUG
#  define SyVerify(_expr)                           SyAssert(_expr)
# else /* ? __SYNSOFT_DBS_DEBUG */
#  define SyVerify(_expr)                           ((void)((_expr) ? (0) : (0)))
# endif /* __SYNSOFT_DBS_DEBUG */
#endif /* SyVerify */

#define __SYNSOFT_DBS_STATIC_ASSERT_SUPPORT
#if defined(__SYNSOFT_DBS_STATIC_ASSERT_SUPPORT)
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC
#   define SyStaticAssert(ex)                       do { typedef int ai[(ex) ? 1 : -1]; } while(0)
#  else
#   define SyStaticAssert(ex)                       do { typedef int ai[(ex) ? 1 : 0]; } while(0)
# endif /* compiler */
#else /* ? __SYNSOFT_DBS_STATIC_ASSERT_SUPPORT */
# define SyStaticAssert(ex)                         SyAssert(ex)
#endif /* __SYNSOFT_DBS_STATIC_ASSERT_SUPPORT */

#define __SYNSOFT_DBS_MESSAGE_ASSERT_SUPPORT
#if defined(__SYNSOFT_DBS_MESSAGE_ASSERT_SUPPORT)
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_DVS_COMPILER_VER_WATCOM
#  define SyMessageAssert(msg, ex)                  SyAssert(ex)
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_DVS_COMPILER_VER_COMO || \
       __SYNSOFT_DVS_COMPILER == __SYNSOFT_DVS_COMPILER_VER_GNUC || \
       __SYNSOFT_DVS_COMPILER == __SYNSOFT_DVS_COMPILER_VER_MWERKS
#  define SyMessageAssert(msg, ex)                  SyAssert((msg && ex))
# else
#  define SyMessageAssert(msg, ex)                  SyAssert((msg, ex))
# endif /* __WATCOMC__ */
#else
# define SyMessageAssert(msg, ex)
#endif /* __SYNSOFT_DBS_MESSAGE_ASSERT_SUPPORT */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 10. preprocessor definitions
 */

/* Gives the number of elements in an array */
#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC
# define NUM_ELEMENTS(x)                            (sizeof(x) / sizeof((x)[0]))
#else /* ? __SYNSOFT_VAL_COMPILER_DMC */
# define NUM_ELEMENTS(x)                            (sizeof(x) / sizeof(0[(x)]))
#endif /* __SYNSOFT_VAL_COMPILER_DMC */

/* Default arguments for C/C++ functions. */
#ifdef __cplusplus
# define __SyDefArg(x)                              = (x)
#else
# define __SyDefArg(x)
#endif /* __cplusplus */

/* definition of interface as a keyword */
#ifndef interface
# define interface                                  struct
#endif /* !interface */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 11. API discrimination
 *
 * Defined here are boolean (..._DBS_...) symbols that declare whether a
 * particular API is supported/present.
 *
 * Currently discriminated APIs are:
 *
 * Win32            -   __SYNSOFT_DBS_API_WIN32
 * POSIX Threads    -   __SYNSOFT_DBS_API_PTHREADS
 * COM              -   __SYNSOFT_DBS_API_COM
 * CORBA            -   __SYNSOFT_DBS_API_CORBA (not currently available)
 *
 * In order to prevent discrimination of these APIs, the following symbols
 * should be defined:
 *
 * _SYB_NO_API_WIN32
 * _SYB_NO_API_PTHREADS
 * _SYB_NO_API_COM
 * _SYB_NO_API_CORBA
 */

/* Win32. */
#ifndef _SYB_NO_API_WIN32
# if (__SYNSOFT_DVS_OS_VER & (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_BITMASK)) == (__SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_32)
#  define __SYNSOFT_DBS_API_WIN32
# endif /* __SYNSOFT_VAL_OS_WIN  | __SYNSOFT_VAL_OS_32 */
#endif /* !_SYB_NO_API_WIN32 */

/* PThreads. */
#ifndef _SYB_NO_API_PTHREADS
# if defined(__SYNSOFT_DBS_MULTITHREADED_SUPPORT) && \
    (   defined(_REENTRANT) || \
        defined(_POSIX_THREADS))
#  define __SYNSOFT_DBS_API_PTHREADS
# endif /* __SYNSOFT_DBS_MULTITHREADED_SUPPORT && (_REENTRANT || _POSIX_THREADS) */
#endif /* !_SYB_NO_API_PTHREADS */

/* COM */
#ifndef _SYB_NO_API_COM
# if __SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(4, 2) || \
     __SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26) || \
     __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(1, 51) || \
     (  defined(__SYNSOFT_DBS_API_WIN32) && \
        (   __SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(3, 0) || \
            __SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0) || \
            __SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4) || \
            __SYNSOFT_DVS_COMPILER_VER_WATCOM >= __SYNSOFT_GEN_COMPILER_VERSION(12, 0)))
#  define __SYNSOFT_DBS_API_COM
#  ifdef _SYB_COMPILE_OBSOLETE
#   define __SYNSOFT_DBS_COM_SUPPORT
#  endif /* _SYB_COMPILE_OBSOLETE */
# endif /* _MSVC > 0 || _BORLAND >= 4.2 */
#endif /* !_SYB_NO_API_COM */

/* CORBA */
#ifndef _SYB_NO_API_CORBA
# if 0
#  define __SYNSOFT_DBS_API_CORBA
#  ifdef _SYB_COMPILE_OBSOLETE
#   define __SYNSOFT_DBS_CORBA_SUPPORT
#  endif /* _SYB_COMPILE_OBSOLETE */
# endif /* 0 */
#endif /* !_SYB_NO_API_CORBA */


#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE

#ifdef __SYNSOFT_DBS_API_WIN32
# pragma message("    API: Win32 supported")
#endif /* __SYNSOFT_DBS_API_WIN32 */

#ifdef __SYNSOFT_DBS_API_PTHREADS
# pragma message("    API: PThreads supported")
#endif /* __SYNSOFT_DBS_API_PTHREADS */

#ifdef __SYNSOFT_DBS_API_COM
# pragma message("    API: COM supported")
#endif /* __SYNSOFT_DBS_API_COM */

#ifdef __SYNSOFT_DBS_API_CORBA
# pragma message("    Using CORBA")
#endif /* __SYNSOFT_DBS_API_CORBA */

#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 12. C-language feature discrimination
 *
 * The following C language features are determined:
 *
 * Std C conformance    -   __SYNSOFT_DBS_STDC
 * inlining             -   __SYNSOFT_DBS_CINLINE_SUPPORT
 *
 *      In C, the inline keyword may be used. If compiling with the Microsoft
 *      compiler (version 4.0 and above), then __inline compilation is used,
 *      otherwise, it resolves to static. This is, however, a bad practise, since
 *      it can lead to a proliferation of identical implementations in separate
 *      compilations, and use of the should be restricted to MSVC 4.0+.
 */

#ifndef __cplusplus
# if (  __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND && \
        defined(__STDC__)) || \
    (   __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECC && \
        (   defined(__STDC__) || \
            (   defined(__STC_) && \
                __STDC_ == 1))) || \
    (   __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX && \
        defined(__STDC_ANSI)) || \
    (   __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC && \
        0/* defined(__STDC__) */) || \
    (   __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC && \
        defined(__STDC__))
#  define __SYNSOFT_DBS_STDC
# endif /* __SYNSOFT_DVS_COMPILER && std-c */
#endif /* !__cplusplus */

#if (   (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 2)) || \
        (   __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND && \
            defined(__MFC_COMPAT__)))
# ifndef _SYB_NO_CINLINE
#  define __SYNSOFT_DBS_CINLINE_SUPPORT
# endif /* !_SYB_NO_CINLINE */
#endif /* __SYNSOFT_DVS_COMPILER_VER_MSVC >= 4.2 || (__SYNSOFT_VAL_COMPILER_BORLAND && __MFC_COMPAT__) */

#ifndef __cplusplus
# ifdef __SYNSOFT_DBS_CINLINE_SUPPORT
#  if ( (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 2)) || \
        (   __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND && \
            defined(__MFC_COMPAT__)))
#   define inline                   __inline
#  endif /* __SYNSOFT_DVS_COMPILER_VER_MSVC >= 4.2 || (__SYNSOFT_VAL_COMPILER_BORLAND && __MFC_COMPAT__) */
# else
#  ifndef _SYB_NO_INLINE_STATIC
#   define inline                   static
#  endif /* _SYB_NO_INLINE_STATIC */
# endif /* __SYNSOFT_DBS_CINLINE_SUPPORT */
#endif /* __cplusplus */

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# ifdef __SYNSOFT_DBS_STDC
#  pragma message("    C: Std C conformance supported")
# endif /* __SYNSOFT_DBS_STDC */
# ifdef __SYNSOFT_DBS_CINLINE_SUPPORT
#  pragma message("    C: inline keyword supported")
# endif /* __SYNSOFT_DBS_CINLINE_SUPPORT */
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* Modifier for thread-affected variables. */
#ifdef __SYNSOFT_DBS_MULTITHREADED_SUPPORT
# define thread_var_t               volatile
#else
# define thread_var_t
#endif /* __SYNSOFT_DBS_MULTITHREADED_SUPPORT */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 13. type support discrimination
 *
 * Determine compiler type support. Types discriminated are:
 *
 *  (i) Integer sizes, including 64-bit and 128-bit integer support
 *  (ii) bool (native) type support
 *
 * Integer (pre-)types.
 *
 * Determining (pre-) types for 6 sizes of integers: 8, 16, 32, 64 & 128
 *
 * and for the eight variants of integers: short, int, long, long long, word,
 *                                           d-word, q-word and int-p
 */

#define __SYNSOFT_GEN_char_t                        char
#define __SYNSOFT_GEN_short_t                       short
#define __SYNSOFT_GEN_int_t                         int
#define __SYNSOFT_GEN_long_t                        long
#define __SYNSOFT_GEN_float_t                       float
#define __SYNSOFT_GEN_double_t                      double

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND
/* Borland compilers.
 * Short it 16-bits
 * Int is 16/32 bits
 * Long is 32-bits
 */
# define __SYNSOFT_GEN_long_double_t                long double
# if defined(_INTEGRAL_MAX_BITS) && \
     _INTEGRAL_MAX_BITS >= 64
#  define __SYNSOFT_GEN_long_long_t                 __int64
# endif /* _INTEGRAL_MAX_BITS >= 64 */
# define __SYNSOFT_DBS_SHORT_BITSIZE                16
# if __SYNSOFT_DVS_OS_BITSIZE == 32
#  define __SYNSOFT_DBS_INT_BITSIZE                 32
# elif __SYNSOFT_DVS_OS_BITSIZE == 16
#  define __SYNSOFT_DBS_INT_BITSIZE                 16
# else
#  error Cannot recognise Borland compiler for bit-size not of (16, 32)
# endif /* __SYNSOFT_DVS_OS_BITSIZE */
# define __SYNSOFT_DBS_LONG_BITSIZE                 32
# if defined(_INTEGRAL_MAX_BITS) && \
     _INTEGRAL_MAX_BITS >= 64
#  define __SYNSOFT_DBS_LLONG_BITSIZE               64
# else
#  define __SYNSOFT_DBS_LLONG_BITSIZE               32
# endif /* _INTEGRAL_MAX_BITS >= 64 */
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_WATCOM
/* Borland compilers.
 * Short it 16-bits
 * Int is 16/32 bits
 * Long is 32-bits
 */
# define __SYNSOFT_GEN_long_double_t                long double
# if defined(_INTEGRAL_MAX_BITS) && \
     _INTEGRAL_MAX_BITS >= 64
#  define __SYNSOFT_GEN_long_long_t                 __int64
# endif /* _INTEGRAL_MAX_BITS >= 64 */
# define __SYNSOFT_DBS_SHORT_BITSIZE                16
# if __SYNSOFT_DVS_OS_BITSIZE == 32
#  define __SYNSOFT_DBS_INT_BITSIZE                 32
# elif __SYNSOFT_DVS_OS_BITSIZE == 16
#  define __SYNSOFT_DBS_INT_BITSIZE                 16
# else
#  error Cannot recognise Borland compiler for bit-size not of (16, 32)
# endif /* __SYNSOFT_DVS_OS_BITSIZE */
# define __SYNSOFT_DBS_LONG_BITSIZE                 32
# if defined(_INTEGRAL_MAX_BITS) && \
     _INTEGRAL_MAX_BITS >= 64
#  define __SYNSOFT_DBS_LLONG_BITSIZE               64
# else
#  define __SYNSOFT_DBS_LLONG_BITSIZE               32
# endif /* _INTEGRAL_MAX_BITS >= 64 */
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECC
/* Dec C++ compilers.
 * Short it 16-bits
 * Int is 32 bits
 * Long is 32/64-bits
 * LLong is 64-bits
 */
#define __SYNSOFT_GEN_long_double_t                 double
#define __SYNSOFT_GEN_long_long_t                   long long
#define __SYNSOFT_DBS_SHORT_BITSIZE                 16
#define __SYNSOFT_DBS_INT_BITSIZE                   32
#if __SYNSOFT_DVS_OS_BITSIZE == 64
#define __SYNSOFT_DBS_LONG_BITSIZE                  64
#elif __SYNSOFT_DVS_OS_BITSIZE == 32
#define __SYNSOFT_DBS_LONG_BITSIZE                  32
#else
#error Cannot recognise Dec C++ compiler for bit-size not of (32, 64)
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
#define __SYNSOFT_DBS_LLONG_BITSIZE                 64
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX
/* Dec C++ compilers.
 * Short it 16-bits
 * Int is 32 bits
 * Long is 32/64-bits
 * LLong is 64-bits
 */
#define __SYNSOFT_GEN_long_double_t                 long double
#define __SYNSOFT_GEN_long_long_t                   long long
#define __SYNSOFT_DBS_SHORT_BITSIZE                 16
#define __SYNSOFT_DBS_INT_BITSIZE                   32
#if __SYNSOFT_DVS_OS_BITSIZE == 64
#define __SYNSOFT_DBS_LONG_BITSIZE                  64
#elif __SYNSOFT_DVS_OS_BITSIZE == 32
#define __SYNSOFT_DBS_LONG_BITSIZE                  32
#else
#error Cannot recognise Dec C++ compiler for bit-size not of (32, 64)
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
#define __SYNSOFT_DBS_LLONG_BITSIZE                 64
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC
/* Digital Mars C/C++ compilers.
 * Short it 16-bits
 * Int is 32 bits
 * Long is 32/64-bits
 * LLong is 64-bits
 */
#define __SYNSOFT_GEN_long_double_t                 long double
#define __SYNSOFT_GEN_long_long_t                   long long
#define __SYNSOFT_DBS_SHORT_BITSIZE                 16
#define __SYNSOFT_DBS_INT_BITSIZE                   32
#if __SYNSOFT_DVS_OS_BITSIZE == 64
#define __SYNSOFT_DBS_LONG_BITSIZE                  64
#elif __SYNSOFT_DVS_OS_BITSIZE == 32
#define __SYNSOFT_DBS_LONG_BITSIZE                  32
#else
#error Cannot recognise Digital Mars C/C++ compiler for bit-size not of (32, 64)
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
#define __SYNSOFT_DBS_LLONG_BITSIZE                 64

#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS
/* Metrowerks compilers.
 * Short it 16-bits
 * Int is 32 bits
 * Long is 32-bits
 */
#define __SYNSOFT_GEN_long_double_t                 long double
#define __SYNSOFT_GEN_long_long_t                   long long

#define __SYNSOFT_DBS_SHORT_BITSIZE                 16
#if __SYNSOFT_DVS_OS_BITSIZE == 64
#define __SYNSOFT_DBS_INT_BITSIZE                   64
#elif __SYNSOFT_DVS_OS_BITSIZE == 32
#define __SYNSOFT_DBS_INT_BITSIZE                   32
#elif __SYNSOFT_DVS_OS_BITSIZE == 16
#define __SYNSOFT_DBS_INT_BITSIZE                   16
#else
#error Cannot recognise Microsoft compiler for bit-size not of (16, 32, 64)
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
#define __SYNSOFT_DBS_LONG_BITSIZE                  32
#define __SYNSOFT_DBS_LLONG_BITSIZE                 64

#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL || \
      __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
/* Microsoft compilers.
 * Short it 16-bits
 * Int is 16/32 bits
 * Long is 32-bits
 */
#define __SYNSOFT_GEN_long_double_t                 long double
#if __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 0) || \
    __SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)
# define __SYNSOFT_GEN_long_long_t                  __int64
#endif /* __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 0) */

#define __SYNSOFT_DBS_SHORT_BITSIZE                 16
#if __SYNSOFT_DVS_OS_BITSIZE == 64
#define __SYNSOFT_DBS_INT_BITSIZE                   64
#elif __SYNSOFT_DVS_OS_BITSIZE == 32
#define __SYNSOFT_DBS_INT_BITSIZE                   32
#elif __SYNSOFT_DVS_OS_BITSIZE == 16
#define __SYNSOFT_DBS_INT_BITSIZE                   16
#else
#error Cannot recognise Microsoft compiler for bit-size not of (16, 32, 64)
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
#define __SYNSOFT_DBS_LONG_BITSIZE                  32
#if __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 0) || \
    __SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)
#define __SYNSOFT_DBS_LLONG_BITSIZE                 64
#else
#define __SYNSOFT_DBS_LLONG_BITSIZE 32
#endif /* __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 0) */
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC
/* Gnu C/C++ compilers.
 * Short it 16-bits
 * Int is 32 bits
 * Long is 32-bits
 * LLong is 64-bits
 */
#define __SYNSOFT_GEN_long_double_t                 long double
#define __SYNSOFT_GEN_long_long_t                   long long
#define __SYNSOFT_DBS_SHORT_BITSIZE                 16
#define __SYNSOFT_DBS_INT_BITSIZE                   32
#if __SYNSOFT_DVS_OS_BITSIZE == 32
#define __SYNSOFT_DBS_LONG_BITSIZE                  32
#else
#error Cannot recognise Gnu C++ compiler for bit-size not of (32)
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
#define __SYNSOFT_DBS_LLONG_BITSIZE                 64
#elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_XLC
/* Gnu C/C++ compilers.
 * Short it 16-bits
 * Int is 32 bits
 * Long is 32-bits
 * LLong is 64-bits
 */
#define __SYNSOFT_GEN_long_double_t                 long double
#define __SYNSOFT_GEN_long_long_t                   long long
#define __SYNSOFT_DBS_SHORT_BITSIZE                 16
#define __SYNSOFT_DBS_INT_BITSIZE                   32
#if __SYNSOFT_DVS_OS_BITSIZE == 32
# define __SYNSOFT_DBS_LONG_BITSIZE                 32
#else
# error Cannot recognise xlC compiler for bit-size not of (32)
#endif /* __SYNSOFT_DVS_OS_BITSIZE */
#define __SYNSOFT_DBS_LLONG_BITSIZE                 64
#else
# error Compiler not recognised
#endif /* __SYNSOFT_DVS_COMPILER */


/* 8-bit.
 *
 * All 8-bit types are supported on the supported compilers.
 *
 * All supported compilers use (unsigned/signed) char for 8-bits.
 *
 * NB. Since char, signed char and unsigned char are three distinct types, Int8
 * is defined to be signed char, so that it does not identically correspond to
 * the char type.
 */
#define __SYNSOFT_DBS_UINT8_SUPPORT
#define __SYNSOFT_DBS_SINT8_SUPPORT
#define __SYNSOFT_DBS_INT8_SUPPORT

#define __SYNSOFT_GEN_UINT8_T                       unsigned __SYNSOFT_GEN_char_t
#define __SYNSOFT_GEN_SINT8_T                       signed __SYNSOFT_GEN_char_t
#define __SYNSOFT_GEN_INT8_T                        __SYNSOFT_GEN_char_t

/* 16-bit.
 *
 * All 16-bit types are supported on the supported compilers.
 *
 * All supported compilers use (unsigned/signed/unadorned) short for 16-bits.
 */

#define __SYNSOFT_DBS_UINT16_SUPPORT
#define __SYNSOFT_DBS_SINT16_SUPPORT
#define __SYNSOFT_DBS_INT16_SUPPORT

#if __SYNSOFT_DBS_LONG_BITSIZE == 16
# define __SYNSOFT_GEN_UINT16_T                     unsigned __SYNSOFT_GEN_long_t
# define __SYNSOFT_GEN_SINT16_T                     signed __SYNSOFT_GEN_long_t
# define __SYNSOFT_GEN_INT16_T                      __SYNSOFT_GEN_long_t
#elif __SYNSOFT_DBS_INT_BITSIZE == 16
# define __SYNSOFT_GEN_UINT16_T                     unsigned __SYNSOFT_GEN_int_t
# define __SYNSOFT_GEN_SINT16_T                     signed __SYNSOFT_GEN_int_t
# define __SYNSOFT_GEN_INT16_T                      __SYNSOFT_GEN_int_t
#elif __SYNSOFT_DBS_SHORT_BITSIZE == 16
# define __SYNSOFT_GEN_UINT16_T                     unsigned __SYNSOFT_GEN_short_t
# define __SYNSOFT_GEN_SINT16_T                     signed __SYNSOFT_GEN_short_t
# define __SYNSOFT_GEN_INT16_T                      __SYNSOFT_GEN_short_t
#else
#error Cannot determine __SYNSOFT_GEN_xINT16_T
#endif /* 16 */

/* 32-bit.
 *
 * All 32-bit types are supported on the supported compilers.
 *
 * The int or long native types are used.
 *
 * Msvc     -
 * Dec C++  -
 * Borland  -
 * Gnu C++  -
 */

#define __SYNSOFT_DBS_UINT32_SUPPORT
#define __SYNSOFT_DBS_SINT32_SUPPORT
#define __SYNSOFT_DBS_INT32_SUPPORT

#if __SYNSOFT_DBS_LONG_BITSIZE == 32
# define __SYNSOFT_GEN_UINT32_T                     unsigned __SYNSOFT_GEN_long_t
# define __SYNSOFT_GEN_SINT32_T                     signed __SYNSOFT_GEN_long_t
# define __SYNSOFT_GEN_INT32_T                      __SYNSOFT_GEN_long_t
#elif __SYNSOFT_DBS_INT_BITSIZE == 32
# define __SYNSOFT_GEN_UINT32_T                     unsigned __SYNSOFT_GEN_int_t
# define __SYNSOFT_GEN_SINT32_T                     signed __SYNSOFT_GEN_int_t
# define __SYNSOFT_GEN_INT32_T                      __SYNSOFT_GEN_int_t
#elif __SYNSOFT_DBS_SHORT_BITSIZE == 32
# define __SYNSOFT_GEN_UINT32_T                     unsigned __SYNSOFT_GEN_short_t
# define __SYNSOFT_GEN_SINT32_T                     signed __SYNSOFT_GEN_short_t
# define __SYNSOFT_GEN_INT32_T                      __SYNSOFT_GEN_short_t
#else
#error Cannot determine __SYNSOFT_GEN_xINT32_T
#endif /* 32 */

/* 64-bit.
 *
 * 64-bit types are supported on the following compilers/architectures:
 *
 */


#if __SYNSOFT_DBS_LLONG_BITSIZE == 64
# define __SYNSOFT_GEN_UINT64_T                     unsigned __SYNSOFT_GEN_long_long_t
# define __SYNSOFT_GEN_SINT64_T                     signed __SYNSOFT_GEN_long_long_t
# define __SYNSOFT_GEN_INT64_T                      __SYNSOFT_GEN_long_long_t
#define __SYNSOFT_DBS_UINT64_SUPPORT
#define __SYNSOFT_DBS_SINT64_SUPPORT
#define __SYNSOFT_DBS_INT64_SUPPORT
#elif __SYNSOFT_DBS_LONG_BITSIZE == 64
# define __SYNSOFT_GEN_UINT64_T                     unsigned __SYNSOFT_GEN_long_t
# define __SYNSOFT_GEN_SINT64_T                     signed __SYNSOFT_GEN_long_t
# define __SYNSOFT_GEN_INT64_T                      __SYNSOFT_GEN_long_t
#define __SYNSOFT_DBS_UINT64_SUPPORT
#define __SYNSOFT_DBS_SINT64_SUPPORT
#define __SYNSOFT_DBS_INT64_SUPPORT
#elif __SYNSOFT_DBS_INT_BITSIZE == 64
# define __SYNSOFT_GEN_UINT64_T                     unsigned __SYNSOFT_GEN_int_t
# define __SYNSOFT_GEN_SINT64_T                     signed __SYNSOFT_GEN_int_t
# define __SYNSOFT_GEN_INT64_T                      __SYNSOFT_GEN_int_t
#define __SYNSOFT_DBS_UINT64_SUPPORT
#define __SYNSOFT_DBS_SINT64_SUPPORT
#define __SYNSOFT_DBS_INT64_SUPPORT
#elif __SYNSOFT_DBS_SHORT_BITSIZE == 64
# define __SYNSOFT_GEN_UINT64_T                     unsigned __SYNSOFT_GEN_short_t
# define __SYNSOFT_GEN_SINT64_T                     signed __SYNSOFT_GEN_short_t
# define __SYNSOFT_GEN_INT64_T                      __SYNSOFT_GEN_short_t
#define __SYNSOFT_DBS_UINT64_SUPPORT
#define __SYNSOFT_DBS_SINT64_SUPPORT
#define __SYNSOFT_DBS_INT64_SUPPORT
#endif /* __SYNSOFT_DBS_SHORT_BITSIZE */

/* 128-bit.
 *
 * No supported compilers support 128-bit integer types.
 */


/* Byte.
 *
 * All Byte types are supported on the supported compilers.
 *
 * All use the Int8 types.
 */
#define __SYNSOFT_DBS_UBYTE_SUPPORT
#define __SYNSOFT_DBS_SBYTE_SUPPORT
#define __SYNSOFT_DBS_BYTE_SUPPORT

#define __SYNSOFT_GEN_UBYTE_T                       __SYNSOFT_GEN_UINT8_T
#define __SYNSOFT_GEN_SBYTE_T                       __SYNSOFT_GEN_SINT8_T
#define __SYNSOFT_GEN_BYTE_T                        __SYNSOFT_GEN_UINT8_T

/* Short.
 *
 * All Short types are supported on the supported compilers.
 *
 * All use the native short types.
 */
#define __SYNSOFT_DBS_USHORT_SUPPORT
#define __SYNSOFT_DBS_SSHORT_SUPPORT
#define __SYNSOFT_DBS_SHORT_SUPPORT

#define __SYNSOFT_GEN_USHORT_T                      unsigned __SYNSOFT_GEN_short_t
#define __SYNSOFT_GEN_SSHORT_T                      signed __SYNSOFT_GEN_short_t
#define __SYNSOFT_GEN_SHORT_T                       __SYNSOFT_GEN_short_t

/* Int.
 *
 * All Int types are supported on the supported compilers.
 *
 * All use the native int types.
 */
#define __SYNSOFT_DBS_UINT_SUPPORT
#define __SYNSOFT_DBS_SINT_SUPPORT
#define __SYNSOFT_DBS_INT_SUPPORT

#define __SYNSOFT_GEN_UINT_T                        unsigned __SYNSOFT_GEN_int_t
#define __SYNSOFT_GEN_SINT_T                        signed __SYNSOFT_GEN_int_t
#define __SYNSOFT_GEN_INT_T                         __SYNSOFT_GEN_int_t

/* Long.
 *
 * All Long types are supported on the supported compilers.
 *
 * All use the native long types.
 */
#define __SYNSOFT_DBS_ULONG_SUPPORT
#define __SYNSOFT_DBS_SLONG_SUPPORT
#define __SYNSOFT_DBS_LONG_SUPPORT

#define __SYNSOFT_GEN_ULONG_T                       unsigned __SYNSOFT_GEN_long_t
#define __SYNSOFT_GEN_SLONG_T                       signed __SYNSOFT_GEN_long_t
#define __SYNSOFT_GEN_LONG_T                        __SYNSOFT_GEN_long_t

/* Long Long.
 *
 * All LLong types are supported on the supported compilers.
 *
 * All use the native long long types (or analogous) where available, otherwise
 * the long types are used.
 */
#define __SYNSOFT_DBS_ULLONG_SUPPORT
#define __SYNSOFT_DBS_SLLONG_SUPPORT
#define __SYNSOFT_DBS_LLONG_SUPPORT

#ifdef __SYNSOFT_GEN_long_long_t
#define __SYNSOFT_GEN_ULLONG_T                      unsigned __SYNSOFT_GEN_long_long_t
#else
#define __SYNSOFT_GEN_ULLONG_T                      unsigned __SYNSOFT_GEN_long_t
#endif /* __SYNSOFT_GEN_long_long_t */
#ifdef __SYNSOFT_GEN_long_long_t
#define __SYNSOFT_GEN_SLLONG_T                      signed __SYNSOFT_GEN_long_long_t
#else
#define __SYNSOFT_GEN_SLLONG_T                      signed __SYNSOFT_GEN_long_t
#endif /* __SYNSOFT_GEN_long_long_t */
#ifdef __SYNSOFT_GEN_long_long_t
#define __SYNSOFT_GEN_LLONG_T                       __SYNSOFT_GEN_long_long_t
#else
#define __SYNSOFT_GEN_LLONG_T                       __SYNSOFT_GEN_long_t
#endif /* __SYNSOFT_GEN_long_long_t */

/* Platform-pointer - sized integer. */
#define __SYNSOFT_DBS_UINTP_SUPPORT
#define __SYNSOFT_DBS_SINTP_SUPPORT
#define __SYNSOFT_DBS_INTP_SUPPORT

#if __SYNSOFT_DVS_OS_BITSIZE == 64
#define __SYNSOFT_GEN_SINTP_T                       __SYNSOFT_GEN_SINT64_T
#define __SYNSOFT_GEN_UINTP_T                       __SYNSOFT_GEN_UINT64_T
#define __SYNSOFT_GEN_INTP_T                        __SYNSOFT_GEN_INT64_T
#elif __SYNSOFT_DVS_OS_BITSIZE == 32
#define __SYNSOFT_GEN_SINTP_T                       __SYNSOFT_GEN_SINT32_T
#define __SYNSOFT_GEN_UINTP_T                       __SYNSOFT_GEN_UINT32_T
#define __SYNSOFT_GEN_INTP_T                        __SYNSOFT_GEN_INT32_T
#elif __SYNSOFT_DVS_OS_BITSIZE == 16
#define __SYNSOFT_GEN_SINTP_T                       __SYNSOFT_GEN_SINT16_T
#define __SYNSOFT_GEN_UINTP_T                       __SYNSOFT_GEN_UINT16_T
#define __SYNSOFT_GEN_INTP_T                        __SYNSOFT_GEN_INT16_T
#elif __SYNSOFT_DVS_OS_BITSIZE == 8
#define __SYNSOFT_GEN_SINTP_T                       __SYNSOFT_GEN_SINT8_T
#define __SYNSOFT_GEN_UINTP_T                       __SYNSOFT_GEN_UINT8_T
#define __SYNSOFT_GEN_INTP_T                        __SYNSOFT_GEN_INT8_T
#else
#error Cannot determine operating system bit-size. Further discrimination of operating system required.
#endif /* __SYNSOFT_DVS_OS_BITSIZE */

/* Floating point.
 *
 * All floating point types are supported on the supported compilers
 */
#define __SYNSOFT_DBS_FLOAT_SUPPORT
#define __SYNSOFT_DBS_DOUBLE_SUPPORT
#define __SYNSOFT_DBS_LDOUBLE_SUPPORT

#define __SYNSOFT_GEN_FLOAT_T                       __SYNSOFT_GEN_float_t
#define __SYNSOFT_GEN_DOUBLE_T                      __SYNSOFT_GEN_double_t
#define __SYNSOFT_GEN_LDOUBLE_T                     __SYNSOFT_GEN_long_double_t


/* Word, DWord & QWord.
 *
 * All supported compilers support Word and DWord.
 *
 * QWord is supported when UInt64 is supported.
 */

#define __SYNSOFT_DBS_WORD_SUPPORT
#define __SYNSOFT_DBS_DWORD_SUPPORT

#define __SYNSOFT_GEN_WORD_T                        UInt16
#define __SYNSOFT_GEN_DWORD_T                       UInt32

#ifdef __SYNSOFT_DBS_UINT64_SUPPORT
# define __SYNSOFT_DBS_QWORD_SUPPORT
# define __SYNSOFT_GEN_QWORD_T                      UInt64
#endif /* __SYNSOFT_DBS_UINT64_SUPPORT */


/* (ii) bool (native) type support */

/* bool. __SYNSOFT_NATIVE_BOOL_SUPPORT */

#if defined(__cplusplus)
# if __SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0)
#  define __SYNSOFT_NATIVE_BOOL_SUPPORT
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX && \
       defined(_BOOL_EXISTS)
#  define __SYNSOFT_NATIVE_BOOL_SUPPORT
# elif __SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 20)
#  define __SYNSOFT_NATIVE_BOOL_SUPPORT
# elif __SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 91)
#  define __SYNSOFT_NATIVE_BOOL_SUPPORT
# elif (__SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0)) || \
       (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(5, 0))
#  define __SYNSOFT_NATIVE_BOOL_SUPPORT
# elif __SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)
#  if __option(bool)
#   define __SYNSOFT_NATIVE_BOOL_SUPPORT
#  else
    /* Not defined */
#  endif /* __option(bool) */
# elif __SYNSOFT_DVS_COMPILER_VER_WATCOM >= __SYNSOFT_GEN_COMPILER_VERSION(11, 0)
#  define __SYNSOFT_NATIVE_BOOL_SUPPORT
# endif /* Compiler */
#endif /* __cplusplus */

#if defined(__SYNSOFT_DBS_SINT64_SUPPORT) || \
    defined(__SYNSOFT_DBS_UINT64_SUPPORT) || \
    defined(__SYNSOFT_DBS_INT64_SUPPORT)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    64-bit integer support")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* __SYNSOFT_DBS_SINT64_SUPPORT || __SYNSOFT_DBS_UINT64_SUPPORT || __SYNSOFT_DBS_INT64_SUPPORT */

#if defined(__SYNSOFT_DBS_SINT128_SUPPORT) || \
    defined(__SYNSOFT_DBS_UINT128_SUPPORT) || \
    defined(__SYNSOFT_DBS_INT128_SUPPORT)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    128-bit integer support")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* __SYNSOFT_DBS_SINT128_SUPPORT || __SYNSOFT_DBS_UINT128_SUPPORT || __SYNSOFT_DBS_INT128_SUPPORT */

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
# if defined(__SYNSOFT_NATIVE_BOOL_SUPPORT)
#  pragma message("    Native bool type support")
# else /* ? __SYNSOFT_NATIVE_BOOL_SUPPORT */
#  pragma message("    Native bool type not supported: typedef'd from int")
# endif /* __SYNSOFT_NATIVE_BOOL_SUPPORT */
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* /////////////////////////////////////
 * Type pointer/reference definitions
 */

#define __SyConst(x, X)     typedef x const                                                     C##X;
#define __SyVolatile(x, X)  typedef x volatile                                                  V##X;

#define __SyPtr(x, X)       typedef x*                                                          P##X;
#define __SyCPtr(x, X)      typedef x const*                                                    PC##X;
#define __SyVPtr(x, X)      typedef x volatile*                                                 PV##X;
#define __SyPtrPtr(x, X)    typedef x**                                                         PP##X;
#define __SyCPtrPtr(x, X)   typedef x const**                                                   PPC##X;
#define __SyVPtrPtr(x, X)   typedef x volatile**                                                PPV##X;
#define __SyPtrCPtr(x, X)   typedef x* const *                                                  PCP##X;
#define __SyPtrVPtr(x, X)   typedef x* volatile *                                               PVP##X;
#define __SyCPtrCPtr(x, X)  typedef x const* const *                                            PCPC##X;
#define __SyVPtrVPtr(x, X)  typedef x volatile* volatile *                                      PVPV##X;

#define __SyNPtr(x, X)      typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *                                      NP##X;
#define __SyNCPtr(x, X)     typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *                                NPC##X;
#define __SyNVPtr(x, X)     typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *                             NPV##X;
#define __SyNPtrPtr(x, X)   typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *                  NPP##X;
#define __SyNCPtrPtr(x, X)  typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *            NPPC##X;
#define __SyNVPtrPtr(x, X)  typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *         NPPV##X;
#define __SyNPtrCPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *            NPCP##X;
#define __SyNPtrVPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *         NPVP##X;
#define __SyNCPtrCPtr(x, X) typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *      NPCPC##X;
#define __SyNVPtrVPtr(x, X) typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *NPVPV##X;

#define __SyLPtr(x, X)      typedef x __SYNSOFT_MOD_MEMMODEL_FAR *                                       LP##X;
#define __SyLCPtr(x, X)     typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *                                 LPC##X;
#define __SyLVPtr(x, X)     typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *                              LPV##X;
#define __SyLPtrPtr(x, X)   typedef x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *                    LPP##X;
#define __SyLCPtrPtr(x, X)  typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *              LPPC##X;
#define __SyLVPtrPtr(x, X)  typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *           LPPV##X;
#define __SyLPtrCPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *              LPCP##X;
#define __SyLPtrVPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *           LPVP##X;
#define __SyLCPtrCPtr(x, X) typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *        LPCPC##X;
#define __SyLVPtrVPtr(x, X) typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *  LPVPV##X;

#ifdef __cplusplus
#define __SyRef(x, X)       typedef x&                                                          R##X;
#define __SyCRef(x, X)      typedef x const&                                                    RC##X;
#define __SyVRef(x, X)      typedef x volatile&                                                 RV##X;
#define __SyCVRef(x, X)     typedef x const volatile&                                           RCV##X;
#define __SyRefPtr(x, X)    typedef x*&                                                         RP##X;
#define __SyCRefPtr(x, X)   typedef x const*&                                                   RPC##X;
#define __SyVRefPtr(x, X)   typedef x volatile*&                                                RPV##X;
#define __SyRefCPtr(x, X)   typedef x* const &                                                  RCP##X;
#define __SyRefVPtr(x, X)   typedef x* volatile &                                               RVP##X;
#define __SyCRefCPtr(x, X)  typedef x const* const &                                            RCPC##X;
#define __SyVRefVPtr(x, X)  typedef x volatile* volatile &                                      RVPV##X;

#define __SyNRef(x, X)      typedef x __SYNSOFT_MOD_MEMMODEL_NEAR&                                       NR##X;
#define __SyNCRef(x, X)     typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR&                                 NRC##X;
#define __SyNVRef(x, X)     typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR&                              NRV##X;
#define __SyNRefPtr(x, X)   typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR&                   NRP##X;
#define __SyNCRefPtr(x, X)  typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR&             NRPC##X;
#define __SyNVRefPtr(x, X)  typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR&          NRPV##X;
#define __SyNRefCPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR&             NRCP##X;
#define __SyNRefVPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR&          NRVP##X;
#define __SyNCRefCPtr(x, X) typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR&       NRCPC##X;
#define __SyNVRefVPtr(x, X) typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR& NRVPV##X;

#define __SyLRef(x, X)      typedef x __SYNSOFT_MOD_MEMMODEL_FAR&                                        LR##X;
#define __SyLCRef(x, X)     typedef x const __SYNSOFT_MOD_MEMMODEL_FAR&                                  LRC##X;
#define __SyLVRef(x, X)     typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR&                               LRV##X;
#define __SyLRefPtr(x, X)   typedef x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR&                     LRP##X;
#define __SyLCRefPtr(x, X)  typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR&               LRPC##X;
#define __SyLVRefPtr(x, X)  typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR&            LRPV##X;
#define __SyLRefCPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR&               LRCP##X;
#define __SyLRefVPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR&            LRVP##X;
#define __SyLCRefCPtr(x, X) typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR&         LRCPC##X;
#define __SyLVRefVPtr(x, X) typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR&   LRVPV##X;
#else
#define __SyRef(x, X)       typedef x*                                                          R##X;
#define __SyCRef(x, X)      typedef x const*                                                    RC##X;
#define __SyVRef(x, X)      typedef x volatile*                                                 RV##X;
#define __SyCVRef(x, X)     typedef x const volatile*                                           RCV##X;
#define __SyRefPtr(x, X)    typedef x**                                                         RP##X;
#define __SyCRefPtr(x, X)   typedef x const**                                                   RPC##X;
#define __SyVRefPtr(x, X)   typedef x volatile**                                                RPV##X;
#define __SyRefCPtr(x, X)   typedef x* const *                                                  RCP##X;
#define __SyRefVPtr(x, X)   typedef x* volatile *                                               RVP##X;
#define __SyCRefCPtr(x, X)  typedef x const* const *                                            RCPC##X;
#define __SyVRefVPtr(x, X)  typedef x volatile* volatile *                                      RVPV##X;

#define __SyNRef(x, X)      typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *                                      NR##X;
#define __SyNCRef(x, X)     typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *                                NRC##X;
#define __SyNVRef(x, X)     typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *                             NRV##X;
#define __SyNRefPtr(x, X)   typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *                  NRP##X;
#define __SyNCRefPtr(x, X)  typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *            NRPC##X;
#define __SyNVRefPtr(x, X)  typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *         NRPV##X;
#define __SyNRefCPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *            NRCP##X;
#define __SyNRefVPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *         NRVP##X;
#define __SyNCRefCPtr(x, X) typedef x const __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *      NRCPC##X;
#define __SyNVRefVPtr(x, X) typedef x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *NRVPV##X;

#define __SyLRef(x, X)      typedef x __SYNSOFT_MOD_MEMMODEL_FAR *                                       LR##X;
#define __SyLCRef(x, X)     typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *                                 LRC##X;
#define __SyLVRef(x, X)     typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *                              LRV##X;
#define __SyLRefPtr(x, X)   typedef x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *                    LRP##X;
#define __SyLCRefPtr(x, X)  typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *              LRPC##X;
#define __SyLVRefPtr(x, X)  typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *           LRPV##X;
#define __SyLRefCPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *              LRCP##X;
#define __SyLRefVPtr(x, X)  typedef x __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *           LRVP##X;
#define __SyLCRefCPtr(x, X) typedef x const __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *        LRCPC##X;
#define __SyLVRefVPtr(x, X) typedef x volatile __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *  LRVPV##X;
#endif /* __cplusplus */

#define __SyPointer(x, X) \
    __SyPtr(x, X) \
    __SyCPtr(x, X) \
    __SyVPtr(x, X) \
    __SyPtrPtr(x, X) \
    __SyCPtrPtr(x, X) \
    __SyVPtrPtr(x, X) \
    __SyPtrCPtr(x, X) \
    __SyPtrVPtr(x, X) \
    __SyCPtrCPtr(x, X) \
    __SyVPtrVPtr(x, X) \
    __SyNPtr(x, X) \
    __SyNCPtr(x, X) \
    __SyNVPtr(x, X) \
    __SyNPtrPtr(x, X) \
    __SyNCPtrPtr(x, X) \
    __SyNVPtrPtr(x, X) \
    __SyNPtrCPtr(x, X) \
    __SyNPtrVPtr(x, X) \
    __SyNCPtrCPtr(x, X) \
    __SyNVPtrVPtr(x, X) \
    __SyLPtr(x, X) \
    __SyLCPtr(x, X) \
    __SyLVPtr(x, X) \
    __SyLPtrPtr(x, X) \
    __SyLCPtrPtr(x, X) \
    __SyLVPtrPtr(x, X) \
    __SyLPtrCPtr(x, X) \
    __SyLPtrVPtr(x, X) \
    __SyLCPtrCPtr(x, X) \
    __SyLVPtrVPtr(x, X)

#define __SyReference1(x, X) \
    __SyRef(x, X) \
    __SyCRef(x, X) \
    __SyVRef(x, X) \
    __SyCVRef(x, X) \
    __SyNRef(x, X) \
    __SyNCRef(x, X) \
    __SyNVRef(x, X) \
    __SyLRef(x, X) \
    __SyLCRef(x, X) \
    __SyLVRef(x, X)

#define __SyReference2(x, X) \
    __SyRefPtr(x, X) \
    __SyCRefPtr(x, X) \
    __SyVRefPtr(x, X) \
    __SyNRefPtr(x, X) \
    __SyNCRefPtr(x, X) \
    __SyNVRefPtr(x, X) \
    __SyNRefCPtr(x, X) \
    __SyNRefVPtr(x, X) \
    __SyNCRefCPtr(x, X) \
    __SyNVRefVPtr(x, X) \
    __SyLRefPtr(x, X) \
    __SyLCRefPtr(x, X) \
    __SyLVRefPtr(x, X) \
    __SyLRefCPtr(x, X) \
    __SyLRefVPtr(x, X) \
    __SyLCRefCPtr(x, X) \
    __SyLVRefVPtr(x, X)

#define __SyPtrType1(x) \
    __SyConst(x,x) \
    __SyVolatile(x,x) \
    __SyPointer(x,x) \
    __SyReference1(x,x) \
    __SyReference2(x,x)

#define __SyPtrType2(x, X) \
    __SyConst(x, X) \
    __SyVolatile(x, X) \
    __SyPointer(x, X) \
    __SyReference2(x, X)

/* Macros for use in using types and their derived types.
 */
#define __SYNSOFT_GEN_USING__SyCONST(N, X)       __SYNSOFT_GEN_USING(N::C##X)
#define __SYNSOFT_GEN_USING__SyPTR(N, X)         __SYNSOFT_GEN_USING(N::P##X)
#define __SYNSOFT_GEN_USING__SyCPTR(N, X)        __SYNSOFT_GEN_USING(N::PC##X)
#define __SYNSOFT_GEN_USING__SyPTRPTR(N, X)      __SYNSOFT_GEN_USING(N::PP##X)
#define __SYNSOFT_GEN_USING__SyCPTRPTR(N, X)     __SYNSOFT_GEN_USING(N::PPC##X)
#define __SYNSOFT_GEN_USING__SyPTRCPTR(N, X)     __SYNSOFT_GEN_USING(N::PCP##X)
#define __SYNSOFT_GEN_USING__SyCPTRCPTR(N, X)    __SYNSOFT_GEN_USING(N::PCPC##X)
#define __SYNSOFT_GEN_USING__SyLPTR(N, X)        __SYNSOFT_GEN_USING(N::LP##X)
#define __SYNSOFT_GEN_USING__SyLCPTR(N, X)       __SYNSOFT_GEN_USING(N::LPC##X)
#define __SYNSOFT_GEN_USING__SyLPTRPTR(N, X)     __SYNSOFT_GEN_USING(N::LPP##X)
#define __SYNSOFT_GEN_USING__SyLCPTRPTR(N, X)    __SYNSOFT_GEN_USING(N::LPPC##X)
#define __SYNSOFT_GEN_USING__SyLPTRCPTR(N, X)    __SYNSOFT_GEN_USING(N::LPCP##X)
#define __SYNSOFT_GEN_USING__SyLCPTRCPTR(N, X)   __SYNSOFT_GEN_USING(N::LPCPC##X)
#define __SYNSOFT_GEN_USING__SyREF(N, X)         __SYNSOFT_GEN_USING(N::R##X)
#define __SYNSOFT_GEN_USING__SyCREF(N, X)        __SYNSOFT_GEN_USING(N::RC##X)
#define __SYNSOFT_GEN_USING__SyREFPTR(N, X)      __SYNSOFT_GEN_USING(N::RP##X)
#define __SYNSOFT_GEN_USING__SyCREFPTR(N, X)     __SYNSOFT_GEN_USING(N::RPC##X)
#define __SYNSOFT_GEN_USING__SyREFCPTR(N, X)     __SYNSOFT_GEN_USING(N::RCP##X)
#define __SYNSOFT_GEN_USING__SyCREFCPTR(N, X)    __SYNSOFT_GEN_USING(N::RCPC##X)
#define __SYNSOFT_GEN_USING__SyLREF(N, X)        __SYNSOFT_GEN_USING(N::LR##X)
#define __SYNSOFT_GEN_USING__SyLCREF(N, X)       __SYNSOFT_GEN_USING(N::LRC##X)
#define __SYNSOFT_GEN_USING__SyLREFPTR(N, X)     __SYNSOFT_GEN_USING(N::LRP##X)
#define __SYNSOFT_GEN_USING__SyLCREFPTR(N, X)    __SYNSOFT_GEN_USING(N::LRPC##X)
#define __SYNSOFT_GEN_USING__SyLREFCPTR(N, X)    __SYNSOFT_GEN_USING(N::LRCP##X)
#define __SYNSOFT_GEN_USING__SyLCREFCPTR(N, X)   __SYNSOFT_GEN_USING(N::LRCPC##X)

#define __SYNSOFT_GEN_USING__SyPOINTER(N, X) \
    __SYNSOFT_GEN_USING__SyPTR(N, X) \
    __SYNSOFT_GEN_USING__SyCPTR(N, X) \
    __SYNSOFT_GEN_USING__SyPTRPTR(N, X) \
    __SYNSOFT_GEN_USING__SyCPTRPTR(N, X) \
    __SYNSOFT_GEN_USING__SyPTRCPTR(N, X) \
    __SYNSOFT_GEN_USING__SyCPTRCPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLCPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLPTRPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLCPTRPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLPTRCPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLCPTRCPTR(N, X)

#define __SYNSOFT_GEN_USING__SyREFERENCE1(N, X) \
    __SYNSOFT_GEN_USING__SyREF(N, X) \
    __SYNSOFT_GEN_USING__SyCREF(N, X) \
    __SYNSOFT_GEN_USING__SyLREF(N, X) \
    __SYNSOFT_GEN_USING__SyLCREF(N, X)

#define __SYNSOFT_GEN_USING__SyREFERENCE2(N, X) \
    __SYNSOFT_GEN_USING__SyREFPTR(N, X) \
    __SYNSOFT_GEN_USING__SyCREFPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLREFPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLCREFPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLREFCPTR(N, X) \
    __SYNSOFT_GEN_USING__SyLCREFCPTR(N, X)

#define __SYNSOFT_GEN_USING__SyPtrType1(N, X) \
    __SYNSOFT_GEN_USING__SyCONST(N, X) \
    __SYNSOFT_GEN_USING__SyPOINTER(N, X) \
    __SYNSOFT_GEN_USING__SyREFERENCE1(N, X) \
    __SYNSOFT_GEN_USING__SyREFERENCE2(N, X)

#define __SYNSOFT_GEN_USING__SyPtrType2(N, X) \
    __SYNSOFT_GEN_USING__SyCONST(N, X) \
    __SYNSOFT_GEN_USING__SyPOINTER(N, X) \
    __SYNSOFT_GEN_USING__SyREFERENCE2(N, X)

/* Local pointer wrappers. */

#define _mC(x)      x const
#define _mV(x)      x volatile
#define _mP(x)      x*
#define _mPC(x)     x const*
#define _mPV(x)     x volatile*
#define _mCP(x)     x* const
#define _mVP(x)     x* volatile
#define _mPP(x)     x**
#define _mPPC(x)    x const**
#define _mPPV(x)    x volatile**
#define _mPCP(x)    x* const *
#define _mPVP(x)    x* volatile *
#define _mPCPC(x)   x const* const *
#define _mPVPV(x)   x volatile* volatile *
#define _mCPP(x)    x** const
#define _mVPP(x)    x** volatile

#define _mNP(x)     x __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPC(x)    x const __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPV(x)    x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNCP(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR *const
#define _mNVP(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile
#define _mNPP(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPPC(x)   x const __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPPV(x)   x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPCP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPVP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPCPC(x)  x const __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNPVPV(x)  x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNCPP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *const
#define _mNVPP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *volatile

#define _mLP(x)     x __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPC(x)    x const __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPV(x)    x volatile __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLCP(x)    x __SYNSOFT_MOD_MEMMODEL_FAR *const
#define _mLVP(x)    x __SYNSOFT_MOD_MEMMODEL_FAR *volatile
#define _mLPP(x)    x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPPC(x)   x const __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPPV(x)   x volatile __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPCP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPVP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPCPC(x)  x const __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLPVPV(x)  x volatile __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLCPP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *const
#define _mLVPP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *volatile

#ifdef __cplusplus
#define _mR(x)      x&
#define _mRC(x)     x const&
#define _mRV(x)     x volatile&
#define _mCR(x)     x& const
#define _mVR(x)     x& volatile
#define _mRP(x)     x*&
#define _mRPC(x)    x const*&
#define _mRPV(x)    x volatile*&
#define _mRCP(x)    x* const &
#define _mRVP(x)    x* volatile &
#define _mRCPC(x)   x const* const &
#define _mRVPV(x)   x volatile* volatile &

#define _mNR(x)     x __SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRC(x)    x const __SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRV(x)    x volatile __SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNCR(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR& const
#define _mNVR(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR& volatile
#define _mNRP(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRPC(x)   x const __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRPV(x)   x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRCP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRVP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRCPC(x)  x const __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR&
#define _mNRVPV(x)  x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR&

#define _mLR(x)     x __SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRC(x)    x const __SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRV(x)    x volatile __SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLCR(x)    x __SYNSOFT_MOD_MEMMODEL_FAR& const
#define _mLVR(x)    x __SYNSOFT_MOD_MEMMODEL_FAR& volatile
#define _mLRP(x)    x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRPC(x)   x const __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRPV(x)   x volatile __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRCP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRVP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRCPC(x)  x const __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR&
#define _mLRVPV(x)  x volatile __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR&
#else
#define _mR(x)      x*
#define _mRC(x)     x const*
#define _mRV(x)     x volatile*
#define _mCR(x)     x* const
#define _mVR(x)     x* volatile
#define _mRP(x)     x**
#define _mRPC(x)    x const**
#define _mRPV(x)    x volatile**
#define _mRCP(x)    x* const *
#define _mRVP(x)    x* volatile *
#define _mRCPC(x)   x const* const *
#define _mRVPV(x)   x volatile* volatile *

#define _mNR(x)     x __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRV(x)    x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRC(x)    x const __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNCR(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR *const
#define _mNVR(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile
#define _mNRP(x)    x __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRPC(x)   x const __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRPV(x)   x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *__SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRCP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRVP(x)   x __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRCPC(x)  x const __SYNSOFT_MOD_MEMMODEL_NEAR *const __SYNSOFT_MOD_MEMMODEL_NEAR *
#define _mNRVPV(x)  x volatile __SYNSOFT_MOD_MEMMODEL_NEAR *volatile __SYNSOFT_MOD_MEMMODEL_NEAR *

#define _mLR(x)     x __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRV(x)    x volatile __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRC(x)    x const __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLCR(x)    x __SYNSOFT_MOD_MEMMODEL_FAR *const
#define _mLVR(x)    x __SYNSOFT_MOD_MEMMODEL_FAR *volatile
#define _mLRP(x)    x __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRPC(x)   x const __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRPV(x)   x volatile __SYNSOFT_MOD_MEMMODEL_FAR *__SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRCP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRVP(x)   x __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRCPC(x)  x const __SYNSOFT_MOD_MEMMODEL_FAR *const __SYNSOFT_MOD_MEMMODEL_FAR *
#define _mLRVPV(x)  x volatile __SYNSOFT_MOD_MEMMODEL_FAR *volatile __SYNSOFT_MOD_MEMMODEL_FAR *
#endif /* __cplusplus */

/* ////////////////////////////////// */

/* This allows the development organisation to override any of the type specification
 * macros, for instance to remove the const (CXXX) as CSize clashes with the garbage,
 * non-namespaced, library MFC.
 */
#ifndef _SYB_NO_CUSTOM_TYPEDEFS
# include <M3CstTyp.h>
#endif /* !_SYB_NO_CUSTOM_TYPEDEFS */

/* /////////////////////////////////////
 * Compiler inclusions
 *
 * These includes may be used for compiler-specific definitions, compiler-
 * directives, etc.
 */

#ifndef _SYB_NO_CUSTOM_COMPILER
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND
#  include <M3Bland.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECC
#  include <M3DecC.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX
#  include <M3DecCxx.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC
#  include <M3Dmc.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC
#  include <M3GnuC.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_INTEL
#  include <M3Intel.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MWERKS
#  include <M3Mwerks.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
#  include <M3Msvc.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_WATCOM
#  include <M3Watcom.h>
# elif __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_XLC
#  include <M3Xlc.h>
# else
#  error Compiler not discriminated
# endif /* __SYNSOFT_DVS_COMPILER */
#endif /* !_SYB_NO_CUSTOM_COMPILER */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 16. type specification
 */

/* Regrettably, certain compiler vendors and third-party software vendors define
 * their types using the preprocessor rather than using the C/C++ typedef
 * keyword. Therefore all the Synesis types are "undef'd" in order to prevent
 * problems caused by such practises.
 */

/* Synesis base types.
 */
#undef UInt8
#undef SInt8
#undef Int8
#undef UInt16
#undef SInt16
#undef Int16
#undef UInt32
#undef SInt32
#undef Int32
#undef SInt64
#undef UInt64
#undef Int64
#undef SInt128
#undef UInt128
#undef Int128
#undef SIntP
#undef UIntP
#undef IntP

#undef UByte
#undef SByte
#undef Byte
#undef UShort
#undef SShort
#undef Short
#undef UInt
#undef SInt
#undef Int
#undef ULong
#undef SLong
#undef Long
#undef SLLong
#undef ULLong
#undef LLong

#undef Float
#undef Double
#undef LDouble

/* Synesis additional types.
 */
#undef Size
#undef Bool
#undef Boolean
#undef Word
#undef DWord
#undef QWord
#undef StreamPos
#undef StreamOff
#undef Opaque
#undef Ptr
#undef TypedPtr
#undef PtrDiff
#undef Handle
#undef MemHnd
#undef ThreadId
#undef HAtor

/* Synesis char types.
 */
#undef AChar
#undef WChar
#undef Char
#undef NChar

#define __SYNSOFT_GEN_OPAQUE(_htype)    typedef const struct __synesis_htype##_htype{ int i;} __SYNSOFT_MOD_MEMMODEL_FAR *_htype; __SyPtrType1(_htype)

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
#pragma warning(disable : 4237)   /* Suppresses: "nonstandard extension used : 'keyword' keyword is reserved for future use" */
#endif /* __SYNSOFT_VAL_COMPILER_MSVC */

/* Synesis base types.
 */

/* 8-bit. */
typedef __SYNSOFT_GEN_UINT8_T                       UInt8;
typedef __SYNSOFT_GEN_SINT8_T                       SInt8;
typedef __SYNSOFT_GEN_INT8_T                        Int8;

/* 16-bit. */
typedef __SYNSOFT_GEN_UINT16_T                      UInt16;
typedef __SYNSOFT_GEN_SINT16_T                      SInt16;
typedef __SYNSOFT_GEN_INT16_T                       Int16;

/* 32-bit. */
typedef __SYNSOFT_GEN_UINT32_T                      UInt32;
typedef __SYNSOFT_GEN_SINT32_T                      SInt32;
typedef __SYNSOFT_GEN_INT32_T                       Int32;

/* 64-bit. */
#ifdef __SYNSOFT_DBS_UINT64_SUPPORT
typedef __SYNSOFT_GEN_UINT64_T                      UInt64;
#endif /* __SYNSOFT_DBS_UINT64_SUPPORT */
#ifdef __SYNSOFT_DBS_SINT64_SUPPORT
typedef __SYNSOFT_GEN_SINT64_T                      SInt64;
#endif /* __SYNSOFT_DBS_SINT64_SUPPORT */
#ifdef __SYNSOFT_DBS_INT64_SUPPORT
typedef __SYNSOFT_GEN_INT64_T                       Int64;
#endif /* __SYNSOFT_DBS_INT64_SUPPORT */

/* 128-bit. */
#ifdef __SYNSOFT_DBS_UINT128_SUPPORT
typedef __SYNSOFT_GEN_UINT128                       UInt128;
#endif /* __SYNSOFT_DBS_UINT128_SUPPORT */
#ifdef __SYNSOFT_DBS_SINT128_SUPPORT
typedef __SYNSOFT_GEN_SINT128                       SInt128;
#endif /* __SYNSOFT_DBS_SINT128_SUPPORT */
#ifdef __SYNSOFT_DBS_INT128_SUPPORT
typedef __SYNSOFT_GEN_INT128                        Int128;
#endif /* __SYNSOFT_DBS_INT128_SUPPORT */

/* Byte. */
typedef __SYNSOFT_GEN_UBYTE_T                       UByte;
typedef __SYNSOFT_GEN_SBYTE_T                       SByte;
typedef __SYNSOFT_GEN_BYTE_T                        Byte;

/* Short. */
typedef __SYNSOFT_GEN_USHORT_T                      UShort;
typedef __SYNSOFT_GEN_SSHORT_T                      SShort;
typedef __SYNSOFT_GEN_SHORT_T                       Short;

/* Int. */
typedef __SYNSOFT_GEN_UINT_T                        UInt;
typedef __SYNSOFT_GEN_SINT_T                        SInt;
typedef __SYNSOFT_GEN_INT_T                         Int;

/* Long. */
typedef __SYNSOFT_GEN_ULONG_T                       ULong;
typedef __SYNSOFT_GEN_SLONG_T                       SLong;
typedef __SYNSOFT_GEN_LONG_T                        Long;

/* Long Long. */
typedef __SYNSOFT_GEN_ULLONG_T                      ULLong;
typedef __SYNSOFT_GEN_SLLONG_T                      SLLong;
typedef __SYNSOFT_GEN_LLONG_T                       LLong;

/* Platform-pointer - sized integers. */
typedef __SYNSOFT_GEN_UINTP_T                       UIntP;
typedef __SYNSOFT_GEN_SINTP_T                       SIntP;
typedef __SYNSOFT_GEN_INTP_T                        IntP;

/* Floating point. */
typedef __SYNSOFT_GEN_FLOAT_T                       Float;
typedef __SYNSOFT_GEN_DOUBLE_T                      Double;
typedef __SYNSOFT_GEN_LDOUBLE_T                     LDouble;

/* Synesis additional types.
 *
 * Some of the additional types rely on types commonly found in stddef.h. Where
 * this is not desired, the symbol _SYB_NO_INCL_STDLIBS may be defined to
 * prevent the inclusion of stddef.h, in which case these types are defined in
 * terms of Synesis integer types.
 */

#ifdef _SYB_NO_INCL_STDLIBS

# define __SYNSOFT_GEN_PTRDIFF_T                    __SYNSOFT_GEN_NS_QUAL(::SynesisStd, SIntP)

# define __SYNSOFT_GEN_SIZE_T                       __SYNSOFT_GEN_NS_QUAL(::SynesisStd, UIntP)

# if (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND) || \
     (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC) || \
     (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC)
#  define __SYNSOFT_GEN_WCHAR_T                     __SYNSOFT_GEN_NS_QUAL(::SynesisStd, UInt16)
# elif (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC) || \
     (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECC) || \
     (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX)
#  define __SYNSOFT_GEN_WCHAR_T                     __SYNSOFT_GEN_NS_QUAL(::SynesisStd, UInt32)
# else
#  error Compiler not discriminated
# endif /* __SYNSOFT_DVS_COMPILER */

__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)
typedef __SYNSOFT_GEN_PTRDIFF_T                     ptrdiff_t;
typedef __SYNSOFT_GEN_SIZE_T                        size_t;
# ifndef _WCHAR_T_DEFINED
typedef __SYNSOFT_GEN_WCHAR_T                       wchar_t;
# endif /* !_WCHAR_T_DEFINED */
#else /* ? _SYB_NO_INCL_STDLIBS */
__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)
# include <stddef.h>
#endif /* !_SYB_NO_INCL_STDLIBS */
__SYNSOFT_GEN_NS_START(SynesisStd)

typedef __SYNSOFT_GEN_WORD_T                        Word;
typedef __SYNSOFT_GEN_DWORD_T                       DWord;
#ifdef __SYNSOFT_DBS_QWORD_SUPPORT
typedef __SYNSOFT_GEN_QWORD_T                       QWord;
#endif /* __SYNSOFT_DBS_QWORD_SUPPORT */
typedef size_t                                      Size;       /* Size (bytes) */
#ifndef __SYNSOFT_NATIVE_BOOL_SUPPORT
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
#  pragma warning(disable : 4237)
# endif /* __SYNSOFT_VAL_COMPILER_MSVC */
# if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC
#  define bool                                      __SYNSOFT_GEN_UINT_T
# else /* __SYNSOFT_VAL_COMPILER_DMC */
typedef  __SYNSOFT_GEN_UINT_T                       bool;
# endif /* __SYNSOFT_VAL_COMPILER_DMC */
#endif /* !__SYNSOFT_NATIVE_BOOL_SUPPORT */
typedef bool                                        Bool;       /* Boolean (C++-specific) */
#ifndef __SYNSOFT_NATIVE_BOOL_SUPPORT
#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
# pragma warning(default : 4237)
#endif /* __SYNSOFT_VAL_COMPILER_MSVC */
#endif /* !__SYNSOFT_NATIVE_BOOL_SUPPORT */
typedef UIntP                                       Boolean;    /* Boolean (language-independent). */
typedef Long                                        StreamPos;  /* Stream position. */
typedef Long                                        StreamOff;  /* Stream offset. */
typedef _mP(void)                                   Opaque;     /* Opaque type. */
typedef _mP(void)                                   Ptr;        /* Opaque ptr. */
typedef _mP(Byte)                                   TypedPtr;   /* Ptr to bytes, i.e. with size. */
typedef ptrdiff_t                                   PtrDiff;    /* Pointer difference type. */
#if (__SYNSOFT_DVS_OS_VER & __SYNSOFT_VAL_OS_WIN16) == __SYNSOFT_VAL_OS_WIN16
# if defined(_INC_WINDOWS)
typedef HLOCAL                                      Handle;     /* Opaque handle. */
typedef HANDLE                                      MemHnd;     /* Memory handle. */
# else /* ? _INC_WINDOWS */
#  ifdef STRICT
typedef const void __SYNSOFT_MOD_MEMMODEL_NEAR      *Handle;
typedef const void __SYNSOFT_MOD_MEMMODEL_NEAR      *MemHnd;
#  else /* ? STRICT */
typedef UINT                                        Handle;
typedef UINT                                        MemHnd;
#  endif /* STRICT */
# endif /* _INC_WINDOWS */
#else /* ? __SYNSOFT_DVS_OS_VER */
typedef _mP(void)                                   Handle;     /* Opaque handle. */
typedef _mP(void)                                   MemHnd;     /* Memory handle. */
#endif /* WIN16 */
typedef UIntP                                       ThreadId;   /* Thread identifier. */
__SYNSOFT_GEN_OPAQUE(HAtor)                                     /* Allocator handle type. */

/* Synesis char types.
 */
typedef char                                        AChar;
typedef wchar_t                                     WChar;

#ifndef _SYB_NO_UNICODE_FIXUP
/* This must be defined to prevent UNICODE/_UNICODE inconsistencies from being
 * automatically corrected.
 */
# if defined(_UNICODE) || \
     defined(UNICODE)
#  if !defined(_UNICODE) || \
      !defined(UNICODE)
#   ifdef __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE
#    pragma message("    Only one of UNICODE and _UNICODE defined. Defining the other here.")
#    pragma message("        Define _SYB_NO_UNICODE_FIXUP to prevent this automatic fix.")
#   endif /* __SYNSOFT_DBS_COMPILER_SUPPORTS_PRAGMA_MESSAGE */
#   ifndef UNICODE
#    define UNICODE
#   endif /* UNICODE */
#   ifndef _UNICODE
#    define _UNICODE
#   endif /* _UNICODE */
#  endif /* !_UNICODE || !UNICODE */
# endif /* _UNICODE || UNICODE */
#endif /* _SYB_NO_UNICODE_FIXUP */

/* Ansi currently supported on all operating systems. */
#define __SYNSOFT_DBS_ANSI_SUPPORT

/* Unicode currently supported on MSVC 4.0+, Borland 5.5+, Dec C++ 6.1+. */
#if (   __SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5) || \
        __SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1) || \
        __SYNSOFT_DVS_COMPILER_VER_DMC >= __SYNSOFT_GEN_COMPILER_VERSION(8, 26) || \
        __SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(3, 2) || \
        __SYNSOFT_DVS_COMPILER_VER_INTEL >= __SYNSOFT_GEN_COMPILER_VERSION(6, 0) || \
        __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0) || \
        __SYNSOFT_DVS_COMPILER_VER_MWERKS >= __SYNSOFT_GEN_COMPILER_VERSION(2, 4)) && \
    !defined(_SYB_NO_UNICODE)
# define __SYNSOFT_DBS_UNICODE_SUPPORT
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Unicode support")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* (__SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)) || (__SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5)) || __SYNSOFT_DVS_COMPILER_VER_DECCXX >= __SYNSOFT_GEN_COMPILER_VERSION(6, 1)) */

/* SBCS currently supported on all operating systems. */
#define __SYNSOFT_DBS_SBCS_SUPPORT

/* MBCS currently supported on MSVC 4.0+. */
#if __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0) && \
    !defined(_SYB_NO_MBCS)
# define __SYNSOFT_DBS_MBCS_SUPPORT
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    MBCS support")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* __SYNSOFT_DVS_COMPILER_VER_MSVC >= 4.0 */

#if defined(__SYNSOFT_DBS_UNICODE_SUPPORT) && \
    defined(__SYNSOFT_DBS_ANSI_SUPPORT)
# define __SYNSOFT_DBS_MULTILANGUAGE_SUPPORT
#endif /* __SYNSOFT_DBS_UNICODE_SUPPORT && __SYNSOFT_DBS_ANSI_SUPPORT */


#ifdef __SYNSOFT_DBS_UNICODE_SUPPORT
# define __SyL(x)                                   L ## x
#else
# define __SyL(x)                                   x
#endif /* __SYNSOFT_DBS_UNICODE_SUPPORT */
#define _SyL(x)                                     __SyL(x)

/* Ambient character set. */
#if defined(_UNICODE) || \
    defined(UNICODE)
# define  __SYNSOFT_DBS_UNICODE
 typedef   WChar                                    Char;
 typedef   AChar                                    NChar;
# define  mT(x)                                     _SyL(x)
# define  mC(x)                                     _SyL(x)
# ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#  pragma message("    Character handling: Unicode")
# endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#else
# define  __SYNSOFT_DBS_ANSI
 typedef   AChar                                    Char;
 typedef   WChar                                    NChar;
# define  mT(x)                                     x
# define  mC(x)                                     x
# if defined(_MBCS) || \
     defined(MBCS)
#  define   __SYNSOFT_DBS_MBCS
#  ifdef _SYB_STRICT
#   error Synesis Software libraries do not support MBCS, only SBCS (Ansi) and Unicode (UCS-16)
#  elif defined(__SYNSOFT_DBS_COMPILATION_IS_VERBOSE)
#   pragma message("    Character handling: MBCS")
#   pragma message("        Note: Synesis Software libraries do not ")
#   pragma message("        support MBCS, only SBCS & Unicode")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# else
#  define __SYNSOFT_DBS_SBCS
#  ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#   pragma message("    Character handling: SBCS")
#  endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
# endif /* _MBCS || MBCS */
#endif /* Character type */

/* Pointer types. */

/* Synesis char types.
 */
__SyPtrType1(AChar)
__SyPtrType1(WChar)
__SyPtrType1(Char)
__SyPtrType1(NChar)

/* Synesis base types.
 */
__SyPtrType1(UByte)
__SyPtrType1(SByte)
__SyPtrType1(Byte)
__SyPtrType1(UInt8)
__SyPtrType1(SInt8)
__SyPtrType1(Int8)
__SyPtrType1(UShort)
__SyPtrType1(SShort)
__SyPtrType1(UInt16)
__SyPtrType1(SInt16)
__SyPtrType1(Int16)
__SyPtrType1(Short)
__SyPtrType1(ULong)
__SyPtrType1(SLong)
__SyPtrType1(UInt32)
__SyPtrType1(SInt32)
__SyPtrType1(Int32)
__SyPtrType1(Long)
#ifdef __SYNSOFT_DBS_SINT64_SUPPORT
__SyPtrType1(SInt64)
#endif /* __SYNSOFT_DBS_SINT64_SUPPORT */
#ifdef __SYNSOFT_DBS_UINT64_SUPPORT
__SyPtrType1(UInt64)
#endif /* __SYNSOFT_DBS_UINT64_SUPPORT */
#ifdef __SYNSOFT_DBS_INT64_SUPPORT
__SyPtrType1(Int64)
#endif /* __SYNSOFT_DBS_INT64_SUPPORT */
__SyPtrType1(SLLong)
__SyPtrType1(ULLong)
__SyPtrType1(LLong)
__SyPtrType1(UInt)
__SyPtrType1(SInt)
__SyPtrType1(Int)
__SyPtrType1(SIntP)
__SyPtrType1(UIntP)
__SyPtrType1(IntP)
__SyPtrType1(Float)
__SyPtrType1(Double)
__SyPtrType1(LDouble)

/* Synesis additional types.
 */
__SyPtrType2(void, Void)
__SyPtrType1(Size)
__SyPtrType1(Bool)
__SyPtrType1(Boolean)
__SyPtrType1(Word)
__SyPtrType1(DWord)
#ifdef __SYNSOFT_DBS_QWORD_SUPPORT
__SyPtrType1(QWord)
#endif /* __SYNSOFT_DBS_QWORD_SUPPORT */
__SyPtrType1(StreamPos)
__SyPtrType1(StreamOff)
__SyPtrType1(Opaque)
__SyPtrType1(Ptr)
__SyPtrType1(TypedPtr)
__SyPtrType1(PtrDiff)
__SyPtrType1(Handle)
__SyPtrType1(MemHnd)
__SyPtrType1(ThreadId)

#if __SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC
#pragma warning(default : 4237)   /* Suppresses: "nonstandard extension used : 'keyword' keyword is reserved for future use" */
#endif /* __SYNSOFT_VAL_COMPILER_MSVC */

/* The following sections are macros that provide for the automatic inclusion
 * (using) of the Synesis types, and derived ptr types, into the global
 * namespace and named namespaces.
 */

/* Type using macros
 */
#ifdef __SYNSOFT_DBS_SINT128_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_SINT128 \
    __SYNSOFT_GEN_USING(::SynesisStd::SInt128)
#else
#define __SYNSOFT_GEN_USINGTYPES_SINT128
#endif /* __SYNSOFT_DBS_SINT128_SUPPORT */

#ifdef __SYNSOFT_DBS_UINT128_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_UINT128 \
    __SYNSOFT_GEN_USING(::SynesisStd::UInt128)
#else
#define __SYNSOFT_GEN_USINGTYPES_UINT128
#endif /* __SYNSOFT_DBS_UINT128_SUPPORT */

#ifdef __SYNSOFT_DBS_INT128_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_INT128 \
    __SYNSOFT_GEN_USING(::SynesisStd::Int128)
#else
#define __SYNSOFT_GEN_USINGTYPES_INT128
#endif /* __SYNSOFT_DBS_INT128_SUPPORT */

#ifdef __SYNSOFT_DBS_SINT64_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_SINT64 \
    __SYNSOFT_GEN_USING(::SynesisStd::SInt64)
#else
#define __SYNSOFT_GEN_USINGTYPES_SINT64
#endif /* __SYNSOFT_DBS_SINT64_SUPPORT */

#ifdef __SYNSOFT_DBS_UINT64_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_UINT64 \
    __SYNSOFT_GEN_USING(::SynesisStd::UInt64)
#else
#define __SYNSOFT_GEN_USINGTYPES_UINT64
#endif /* __SYNSOFT_DBS_UINT64_SUPPORT */

#ifdef __SYNSOFT_DBS_INT64_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_INT64 \
    __SYNSOFT_GEN_USING(::SynesisStd::Int64)
#else
#define __SYNSOFT_GEN_USINGTYPES_INT64
#endif /* __SYNSOFT_DBS_INT64_SUPPORT */

#ifdef __SYNSOFT_DBS_SLLONG_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_SLLONG \
    __SYNSOFT_GEN_USING(::SynesisStd::SLLong)
#else
#define __SYNSOFT_GEN_USINGTYPES_SLLONG
#endif /* __SYNSOFT_DBS_SLLONG_SUPPORT */

#ifdef __SYNSOFT_DBS_ULLONG_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_ULLONG \
    __SYNSOFT_GEN_USING(::SynesisStd::ULLong)
#else
#define __SYNSOFT_GEN_USINGTYPES_ULLONG
#endif /* __SYNSOFT_DBS_ULLONG_SUPPORT */

#ifdef __SYNSOFT_DBS_LLONG_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_LLONG \
    __SYNSOFT_GEN_USING(::SynesisStd::LLong)
#else
#define __SYNSOFT_GEN_USINGTYPES_LLONG
#endif /* __SYNSOFT_DBS_LLONG_SUPPORT */

#ifdef __SYNSOFT_DBS_QWORD_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_QWORD \
    __SYNSOFT_GEN_USING(::SynesisStd::QWord)
#else
#define __SYNSOFT_GEN_USINGTYPES_QWORD
#endif /* __SYNSOFT_DBS_QWORD_SUPPORT */


#define __SYNSOFT_GEN_USINGTYPES_STD_BASETYPES \
    __SYNSOFT_GEN_USING(::SynesisStd::UInt8) \
    __SYNSOFT_GEN_USING(::SynesisStd::SInt8) \
    __SYNSOFT_GEN_USING(::SynesisStd::Int8) \
    __SYNSOFT_GEN_USING(::SynesisStd::UInt16) \
    __SYNSOFT_GEN_USING(::SynesisStd::SInt16) \
    __SYNSOFT_GEN_USING(::SynesisStd::Int16) \
    __SYNSOFT_GEN_USING(::SynesisStd::UInt32) \
    __SYNSOFT_GEN_USING(::SynesisStd::SInt32) \
    __SYNSOFT_GEN_USING(::SynesisStd::Int32) \
    __SYNSOFT_GEN_USINGTYPES_SINT64 \
    __SYNSOFT_GEN_USINGTYPES_UINT64 \
    __SYNSOFT_GEN_USINGTYPES_INT64 \
    __SYNSOFT_GEN_USINGTYPES_SINT128 \
    __SYNSOFT_GEN_USINGTYPES_UINT128 \
    __SYNSOFT_GEN_USINGTYPES_INT128 \
    __SYNSOFT_GEN_USING(::SynesisStd::UByte) \
    __SYNSOFT_GEN_USING(::SynesisStd::SByte) \
    __SYNSOFT_GEN_USING(::SynesisStd::Byte) \
    __SYNSOFT_GEN_USING(::SynesisStd::UShort) \
    __SYNSOFT_GEN_USING(::SynesisStd::SShort) \
    __SYNSOFT_GEN_USING(::SynesisStd::Short) \
    __SYNSOFT_GEN_USING(::SynesisStd::ULong) \
    __SYNSOFT_GEN_USING(::SynesisStd::SLong) \
    __SYNSOFT_GEN_USING(::SynesisStd::Long) \
    __SYNSOFT_GEN_USINGTYPES_SLLONG \
    __SYNSOFT_GEN_USINGTYPES_ULLONG \
    __SYNSOFT_GEN_USINGTYPES_LLONG \
    __SYNSOFT_GEN_USING(::SynesisStd::UInt) \
    __SYNSOFT_GEN_USING(::SynesisStd::SInt) \
    __SYNSOFT_GEN_USING(::SynesisStd::Int) \
    __SYNSOFT_GEN_USING(::SynesisStd::SIntP) \
    __SYNSOFT_GEN_USING(::SynesisStd::UIntP) \
    __SYNSOFT_GEN_USING(::SynesisStd::IntP) \
    __SYNSOFT_GEN_USING(::SynesisStd::Float) \
    __SYNSOFT_GEN_USING(::SynesisStd::Double) \
    __SYNSOFT_GEN_USING(::SynesisStd::LDouble)

#define __SYNSOFT_GEN_USINGTYPES_STD_ADDITIONALTYPES \
    __SYNSOFT_GEN_USING(::SynesisStd::Word) \
    __SYNSOFT_GEN_USING(::SynesisStd::DWord) \
    __SYNSOFT_GEN_USINGTYPES_QWORD \
    __SYNSOFT_GEN_USING(::SynesisStd::Size) \
    __SYNSOFT_GEN_USING(::SynesisStd::Bool) \
    __SYNSOFT_GEN_USING(::SynesisStd::Boolean) \
    __SYNSOFT_GEN_USING(::SynesisStd::StreamPos) \
    __SYNSOFT_GEN_USING(::SynesisStd::StreamOff) \
    __SYNSOFT_GEN_USING(::SynesisStd::Opaque) \
    __SYNSOFT_GEN_USING(::SynesisStd::Ptr) \
    __SYNSOFT_GEN_USING(::SynesisStd::TypedPtr) \
    __SYNSOFT_GEN_USING(::SynesisStd::PtrDiff) \
    __SYNSOFT_GEN_USING(::SynesisStd::Handle) \
    __SYNSOFT_GEN_USING(::SynesisStd::MemHnd) \
    __SYNSOFT_GEN_USING(::SynesisStd::ThreadId) \
    __SYNSOFT_GEN_USING(::SynesisStd::HAtor)

#define __SYNSOFT_GEN_USINGTYPES_STD_CHARTYPES \
    __SYNSOFT_GEN_USING(::SynesisStd::AChar) \
    __SYNSOFT_GEN_USING(::SynesisStd::WChar) \
    __SYNSOFT_GEN_USING(::SynesisStd::Char) \
    __SYNSOFT_GEN_USING(::SynesisStd::NChar)

/* Pointer type using macros
 */
#ifdef __SYNSOFT_DBS_SINT64_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_PTR_SINT64 \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SInt64)
#else
#define __SYNSOFT_GEN_USINGTYPES_PTR_SINT64
#endif /* __SYNSOFT_DBS_SINT64_SUPPORT */

#ifdef __SYNSOFT_DBS_UINT64_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_PTR_UINT64 \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UInt64)
#else
#define __SYNSOFT_GEN_USINGTYPES_PTR_UINT64
#endif /* __SYNSOFT_DBS_UINT64_SUPPORT */

#ifdef __SYNSOFT_DBS_INT64_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_PTR_INT64 \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Int64)
#else
#define __SYNSOFT_GEN_USINGTYPES_PTR_INT64
#endif /* __SYNSOFT_DBS_INT64_SUPPORT */

#ifdef __SYNSOFT_DBS_SLLONG_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_PTR_SLLONG \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SLLong)
#else
#define __SYNSOFT_GEN_USINGTYPES_PTR_SLLONG
#endif /* __SYNSOFT_DBS_SLLONG_SUPPORT */

#ifdef __SYNSOFT_DBS_ULLONG_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_PTR_ULLONG \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, ULLong)
#else
#define __SYNSOFT_GEN_USINGTYPES_PTR_ULLONG
#endif /* __SYNSOFT_DBS_ULLONG_SUPPORT */

#ifdef __SYNSOFT_DBS_LLONG_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_PTR_LLONG \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, LLong)
#else
#define __SYNSOFT_GEN_USINGTYPES_PTR_LLONG
#endif /* __SYNSOFT_DBS_LLONG_SUPPORT */

#ifdef __SYNSOFT_DBS_QWORD_SUPPORT
#define __SYNSOFT_GEN_USINGTYPES_PTR_QWORD \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, QWord)
#else
#define __SYNSOFT_GEN_USINGTYPES_PTR_QWORD
#endif /* __SYNSOFT_DBS_QWORD_SUPPORT */

#define __SYNSOFT_GEN_USINGTYPES_STD_PTR_BASETYPES \
    __SYNSOFT_GEN_USING__SyPtrType2(::SynesisStd, Void) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UInt8) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SInt8) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Int8) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UInt16) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SInt16) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Int16) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UInt32) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SInt32) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Int32) \
    __SYNSOFT_GEN_USINGTYPES_PTR_SINT64 \
    __SYNSOFT_GEN_USINGTYPES_PTR_UINT64 \
    __SYNSOFT_GEN_USINGTYPES_PTR_INT64 \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UByte) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SByte) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Byte) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UShort) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SShort) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Short) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, ULong) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SLong) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Long) \
    __SYNSOFT_GEN_USINGTYPES_PTR_SLLONG \
    __SYNSOFT_GEN_USINGTYPES_PTR_ULLONG \
    __SYNSOFT_GEN_USINGTYPES_PTR_LLONG \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UInt) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SInt) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Int) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, SIntP) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, UIntP) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, IntP) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Float) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Double) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, LDouble)

#define __SYNSOFT_GEN_USINGTYPES_STD_PTR_ADDITIONALTYPES \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Word) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, DWord) \
    __SYNSOFT_GEN_USINGTYPES_PTR_QWORD \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Size) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Bool) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Boolean) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, StreamPos) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, StreamOff) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Opaque) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Ptr) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, TypedPtr) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Handle) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, MemHnd) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, ThreadId) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, HAtor)

#define __SYNSOFT_GEN_USINGTYPES_STD_PTR_CHARTYPES \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, AChar) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, WChar) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, Char) \
    __SYNSOFT_GEN_USING__SyPtrType1(::SynesisStd, NChar)

/* The above macros are now used. By default the standard types (and their
 * derived pointer types) are introduced, and the additional types (and their
 * derived pointer types) are not introduced.
 *
 * To introduce the base types (and their derived pointer types), the symbol
 * _SYB_USING_BASE_TYPES should be defined.
 *
 * To introduce the additional types (and their derived pointer types), the
 * symbol _SYB_USING_ADDITIONAL_TYPES should be defined.
 *
 * To introduce the char types (and their derived pointer types), the
 * symbol _SYB_USING_CHAR_TYPES should be defined.
 *
 * To introduce all types (and their derived pointer types), the symbol
 * _SYB_USING_STD_ALL_TYPES should be defined.
 */

__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)

/* All Synesis types. */
#ifdef _SYB_USING_ALL_TYPES
#define _SYB_USING_STD_ALL_TYPES
#if defined(__SYNSOFT_DBS_COMPILATION_IS_VERBOSE)
#pragma message("    Introducing all Synesis types into the global namespace. Un-define _SYB_USING_ALL_TYPES to suppress.")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* _SYB_USING_ALL_TYPES || _SYB_USING_STD_TYPES */

/* All Synesis standard types. */
#ifdef _SYB_USING_STD_ALL_TYPES
#define _SYB_USING_STD_BASE_TYPES
#define _SYB_USING_STD_ADDITIONAL_TYPES
#define _SYB_USING_STD_CHAR_TYPES
#if defined(__SYNSOFT_DBS_COMPILATION_IS_VERBOSE) && \
    !defined(_SYB_USING_ALL_TYPES)
#pragma message("    Introducing all Synesis standard (SynesisStd) types into the global namespace. Un-define _SYB_USING_STD_ALL_TYPES to suppress.")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
#endif /* _SYB_USING_STD_ALL_TYPES */

/* Synesis standard base types. */
#ifdef _SYB_USING_STD_BASE_TYPES
#if defined(__SYNSOFT_DBS_COMPILATION_IS_VERBOSE) && \
    !defined(_SYB_USING_STD_ALL_TYPES)
#pragma message("    Introducing base Synesis standard (SynesisStd) types into the global namespace. Un-define _SYB_USING_STD_BASE_TYPES to suppress.")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
__SYNSOFT_GEN_USINGTYPES_STD_BASETYPES
__SYNSOFT_GEN_USINGTYPES_STD_PTR_BASETYPES
#endif /* _SYB_USING_BASE_TYPES */

/* Synesis standard additional types. */
#ifdef _SYB_USING_STD_ADDITIONAL_TYPES
#if defined(__SYNSOFT_DBS_COMPILATION_IS_VERBOSE) && \
    !defined(_SYB_USING_STD_ALL_TYPES)
#pragma message("    Introducing additional Synesis standard (SynesisStd) types into the global namespace. Un-define _SYB_USING_STD_ADDITIONAL_TYPES to suppress.")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
__SYNSOFT_GEN_USINGTYPES_STD_ADDITIONALTYPES
__SYNSOFT_GEN_USINGTYPES_STD_PTR_ADDITIONALTYPES
#endif /* _SYB_USING_ADDITIONAL_TYPES */

/* Synesis standard char types. */
#ifdef _SYB_USING_STD_CHAR_TYPES
#if defined(__SYNSOFT_DBS_COMPILATION_IS_VERBOSE) && \
    !defined(_SYB_USING_STD_ALL_TYPES)
#pragma message("    Introducing additional Synesis standard (SynesisStd) types into the global namespace. Un-define _SYB_USING_STD_CHAR_TYPES to suppress.")
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */
__SYNSOFT_GEN_USINGTYPES_STD_CHARTYPES
__SYNSOFT_GEN_USINGTYPES_STD_PTR_CHARTYPES
#endif /* _SYB_USING_CHAR_TYPES */

__SYNSOFT_GEN_NS_START(SynesisStd)

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 14. C & C++-language feature discrimination
 */

/* Inline support */
#if defined(__cplusplus) || \
    defined(__SYNSOFT_DBS_CINLINE_SUPPORT)
#define __SYNSOFT_DBS_INLINE_SUPPORT
#endif /* __cplusplus) || __SYNSOFT_DBS_CINLINE_SUPPORT */

#ifdef __SYNSOFT_DBS_COMPILATION_IS_VERBOSE
#ifdef __SYNSOFT_DBS_INLINE_SUPPORT
#pragma message("    C/C++: inline keyword supported")
#endif /* __SYNSOFT_DBS_INLINE_SUPPORT */
#endif /* __SYNSOFT_DBS_COMPILATION_IS_VERBOSE */

/* Function implementation const parameters
 *
 * The compilers allow a function implementation to have parameters const that
 * are declared non-const in the declaration.
 *
 * Borland C++ 5.5 and above supports this
 * Digital Mars C/C++ 8.26 does not support this
 * GNU C/C++ version 2.95 and above supports this
 * Microsoft version 4.0 and above supports this
 * Watcom C/C++ 11.0 does not support this
 */

#if __SYNSOFT_DVS_COMPILER_VER_BORLAND >= __SYNSOFT_GEN_COMPILER_VERSION(5, 5) || \
    __SYNSOFT_DVS_COMPILER_VER_GNUC >= __SYNSOFT_GEN_COMPILER_VERSION(2, 95) || \
    __SYNSOFT_DVS_COMPILER_VER_MSVC >= __SYNSOFT_GEN_COMPILER_VERSION(4, 0)
# define __SYNSOFT_DBS_IMPLEMENTATION_PARAMETERS_CONST_SUPPORT
#endif /* __SYNSOFT_DVS_COMPILER_VER_MSVC */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 15. C++-language feature discrimination
 */

#ifdef __cplusplus
# ifndef _SYB_NO_CPP
__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)
#  include <MLCpp.h>
__SYNSOFT_GEN_NS_START(SynesisStd)
# else
#  ifndef typename_arg_t
#   define typename_arg_t               class
#  endif /* typename_arg_t */
#  ifndef typename_type_t
#   define typename_type_t
#  endif /* typename_type_t */
# endif /* !_SYB_NO_CPP */
#endif /* __cplusplus */

/* C-compatible Interface vtable layout
 */

#if (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_BORLAND) || \
    (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECC) || \
    (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DECCXX) || \
    (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_DMC) || \
    (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_GNUC) || \
    (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_MSVC) || \
    (__SYNSOFT_DVS_COMPILER == __SYNSOFT_VAL_COMPILER_WATCOM)
#define __SYNSOFT_DBS_CCOMPATIBLE_VTABLE_LAYOUT
#endif /* __SYNSOFT_DVS_ARCHITECTURE */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 17. constant definitions
 */

/* NULL */
#ifndef NULL
# ifdef __cplusplus
#  define NULL                                      (0)
# else /* ? __cplusplus */
#  define NULL                                      ((PVoid)0)
# endif /* __cplusplus */
#endif /* NULL */

/* TRUE & FALSE */
#ifndef TRUE
# define TRUE                                       (1)
#endif /* TRUE */

#ifndef FALSE
# define FALSE                                      (0)
#endif /* FALSE */

/* Bool_true & Bool_false */
#undef Bool_false
#undef Bool_true

#ifdef __cplusplus
const Bool   Bool_false         =                   SyCastStatic(Bool, 0);
const Bool   Bool_true          =                   SyCastStatic(Bool, 1);
#else
# define Bool_false                                 SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Bool), 0)
# define Bool_true                                  SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Bool), 1)
#endif /* __cplusplus */

/* Boolean_true & Boolean_false */
#undef Boolean_false
#undef Boolean_true

#ifdef __cplusplus
const Boolean   Boolean_false   =                   SyCastStatic(Boolean, 0);
const Boolean   Boolean_true    =                   SyCastStatic(Boolean, 1);
#else
# define Boolean_false                              SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Boolean), 0)
# define Boolean_true                               SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Boolean), 1)
#endif /* __cplusplus */

/* (bool) true & false */
#undef false
#undef true

#ifndef __SYNSOFT_NATIVE_BOOL_SUPPORT
# define false                                      SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Bool), 0)
# define true                                       SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Bool), 1)
#endif /* __SYNSOFT_NATIVE_BOOL_SUPPORT */

/* Conversion functions/macros */

#ifdef __cplusplus
inline Bool Boolean2Bool(Boolean b)
{
    return b != 0 ? true : false;
}
inline Boolean Bool2Boolean(Bool b)
{
    return b != 0;
}
#else
# define Boolean2Bool(b)                            SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Bool), ((b) != 0))
# define Bool2Boolean(b)                            SyCastStatic(__SYNSOFT_GEN_NS_QUAL(::SynesisStd, Boolean), ((b) != 0))
#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * /////////////////////////////////////////////////////////////////////////////
 * 18. standard function/macro definition
 */

#ifdef __cplusplus
inline LPCVoid PtrByteOffset(LPCVoid p, PtrDiff n)
{
    return SyCastStatic(LPVoid, SyCastStatic(LPByte, SyCastConst(LPVoid, p)) + n);
}

inline PtrDiff PtrByteDiff(LPCVoid p1, LPCVoid p2)
{
    return SyCastStatic(LPCByte, p1) - SyCastStatic(LPCByte, p2);
}
#else
# define PtrByteOffset(p, n)                        SyCastConst(LPCVoid, SyCastStatic(LPVoid, SyCastStatic(LPByte, SyCastConst(LPVoid, p)) + n))
# define PtrByteDiff(p1, p2)                        ((PtrDiff)(SyCastStatic(LPCByte, p1) - SyCastStatic(LPCByte, p2)))
#endif /* __cplusplus */

#if defined(__cplusplus) && \
    defined(__SYNSOFT_DBS_TEMPLATE_SUPPORT)
template <typename_arg_t T>
inline _mPC(T) PtrOffset(_mPC(T) p, PtrDiff n)
{
    return p + n;
}
#else
# define PtrOffset(p, n)                            ((LPVoid)((p) + (n)))
#endif /* __SYNSOFT_DBS_TEMPLATE_SUPPORT */

/* ////////////////////////////////////////////////////////////////////////// */

__SYNSOFT_GEN_NS_END_IMPL(SynesisStd)

__SYNSOFT_GEN_USING(::SynesisStd::Boolean2Bool)
__SYNSOFT_GEN_USING(::SynesisStd::Bool2Boolean)

/* /////////////////////////////////////////////////////////////////////////////
 * Obsolete items.
 */

#ifdef _SYB_COMPILE_OBSOLETE
# include <MLObsolt.h>
#endif /* _SYB_COMPILE_OBSOLETE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _SYNSOFT_INCL_H_SLBASE */

/* ////////////////////////////////////////////////////////////////////////// */
